@@ -1,5 +1,5 @@
 /* acconfig.h - used by autoheader to make config.h.in
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifndef G10_CONFIG_H
-#define G10_CONFIG_H
+#ifndef GNUPG_CONFIG_H
+#define GNUPG_CONFIG_H
 
 /* need this, because some autoconf tests rely on this (e.g. stpcpy)
  * and it should be used for new programs
@@ -29,6 +29,8 @@
 
 #undef M_DEBUG
 #undef M_GUARD
+#undef VERSION
+#undef PACKAGE
 #undef PRINTABLE_OS_NAME
 #undef IS_DEVELOPMENT_VERSION
 
@@ -53,6 +55,7 @@
 
 #undef HAVE_MLOCK
 
+
 #undef BIG_ENDIAN_HOST
 #undef LITTLE_ENDIAN_HOST
 
@@ -72,9 +75,6 @@
 /* Linux has an ioctl */
 #undef HAVE_DEV_RANDOM_IOCTL
 
-/* see cipher/rndegd.c */
-#undef EGD_SOCKET_NAME
-
 
 #undef USE_DYNAMIC_LINKING
 #undef HAVE_DL_DLOPEN
@@ -109,6 +109,6 @@
 
 @BOTTOM@
 
-#include "g10defs.h"
+#include "gnupg-defs.h"
 
-#endif /*G10_CONFIG_H*/
+#endif /*GNUPG_CONFIG_H*/
