@@ -1,5 +1,9 @@
 /* blowfish.c  -  Blowfish encryption
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *
+ * For a description of the algorithm, see:
+ *   Bruce Schneier: Applied Cryptography. John Wiley & Sons, 1996.
+ *   ISBN 0-471-11709-9. Pages 336 ff.
  *
  * This file is part of GnuPG.
  *
@@ -16,10 +20,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- *
- * For a description of the algorithm, see:
- *   Bruce Schneier: Applied Cryptography. John Wiley & Sons, 1996.
- *   ISBN 0-471-11709-9. Pages 336 ff.
  */
 
 /* Test values:
@@ -35,7 +35,7 @@
 #include <string.h>
 #include <assert.h>
 #include "types.h"
-#include "errors.h"
+#include "g10lib.h"
 #include "blowfish.h"
 #include "dynload.h"
 
@@ -43,6 +43,9 @@
 
 #define CIPHER_ALGO_BLOWFISH	 4  /* blowfish 128 bit key */
 
+#define FNCCAST_SETKEY(f)  (int(*)(void*, byte*, unsigned))(f)
+#define FNCCAST_CRYPT(f)   (void(*)(void*, byte*, byte*))(f)
+
 #define BLOWFISH_BLOCKSIZE 8
 #define BLOWFISH_ROUNDS 16
 
@@ -278,20 +281,9 @@ function_F( BLOWFISH_context *bc, u32 x )
 #endif
 #define R(l,r,i)  do { l ^= p[i]; r ^= F(l); } while(0)
 
-static void
-burn_stack (int bytes)
-{
-    char buf[64];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
 
 static void
-do_encrypt(  BLOWFISH_context *bc, u32 *ret_xl, u32 *ret_xr )
+encrypt(  BLOWFISH_context *bc, u32 *ret_xl, u32 *ret_xr )
 {
   #if BLOWFISH_ROUNDS == 16
     u32 xl, xr, *s0, *s1, *s2, *s3, *p;
@@ -424,13 +416,13 @@ decrypt(  BLOWFISH_context *bc, u32 *ret_xl, u32 *ret_xr )
 #undef R
 
 static void
-do_encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
+encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 {
     u32 d1, d2;
 
     d1 = inbuf[0] << 24 | inbuf[1] << 16 | inbuf[2] << 8 | inbuf[3];
     d2 = inbuf[4] << 24 | inbuf[5] << 16 | inbuf[6] << 8 | inbuf[7];
-    do_encrypt( bc, &d1, &d2 );
+    encrypt( bc, &d1, &d2 );
     outbuf[0] = (d1 >> 24) & 0xff;
     outbuf[1] = (d1 >> 16) & 0xff;
     outbuf[2] = (d1 >>	8) & 0xff;
@@ -441,15 +433,9 @@ do_encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
     outbuf[7] =  d2	   & 0xff;
 }
 
-static void
-encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
-{
-    do_encrypt_block (bc, outbuf, inbuf);
-    burn_stack (64);
-}
 
 static void
-do_decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
+decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 {
     u32 d1, d2;
 
@@ -466,13 +452,6 @@ do_decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
     outbuf[7] =  d2	   & 0xff;
 }
 
-static void
-decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
-{
-    do_decrypt_block (bc, outbuf, inbuf);
-    burn_stack (64);
-}
-
 
 static const char*
 selftest(void)
@@ -502,13 +481,10 @@ selftest(void)
     return NULL;
 }
 
-#ifdef __riscos__
-/* need to switch off CSE optimisation for Norcroft C (Acorn/Pace) */
-#pragma no_optimise_cse
-#endif /* __riscos__ */
+
 
 static int
-do_bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
+bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 {
     int i, j;
     u32 data, datal, datar;
@@ -522,7 +498,7 @@ do_bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 	    fprintf(stderr,"%s\n", selftest_failed );
     }
     if( selftest_failed )
-	return G10ERR_SELFTEST_FAILED;
+	return GCRYERR_SELFTEST;
 
     for(i=0; i < BLOWFISH_ROUNDS+2; i++ )
 	c->p[i] = ps[i];
@@ -551,27 +527,27 @@ do_bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 
     datal = datar = 0;
     for(i=0; i < BLOWFISH_ROUNDS+2; i += 2 ) {
-	do_encrypt( c, &datal, &datar );
+	encrypt( c, &datal, &datar );
 	c->p[i]   = datal;
 	c->p[i+1] = datar;
     }
     for(i=0; i < 256; i += 2 )	{
-	do_encrypt( c, &datal, &datar );
+	encrypt( c, &datal, &datar );
 	c->s0[i]   = datal;
 	c->s0[i+1] = datar;
     }
     for(i=0; i < 256; i += 2 )	{
-	do_encrypt( c, &datal, &datar );
+	encrypt( c, &datal, &datar );
 	c->s1[i]   = datal;
 	c->s1[i+1] = datar;
     }
     for(i=0; i < 256; i += 2 )	{
-	do_encrypt( c, &datal, &datar );
+	encrypt( c, &datal, &datar );
 	c->s2[i]   = datal;
 	c->s2[i+1] = datar;
     }
     for(i=0; i < 256; i += 2 )	{
-	do_encrypt( c, &datal, &datar );
+	encrypt( c, &datal, &datar );
 	c->s3[i]   = datal;
 	c->s3[i+1] = datar;
     }
@@ -583,24 +559,13 @@ do_bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 	for( j=i+1; j < 256; j++) {
 	    if( (c->s0[i] == c->s0[j]) || (c->s1[i] == c->s1[j]) ||
 		(c->s2[i] == c->s2[j]) || (c->s3[i] == c->s3[j]) )
-		return G10ERR_WEAK_KEY;
+		return GCRYERR_WEAK_KEY;
 	}
     }
 
     return 0;
 }
 
-#ifdef __riscos__
-#pragma optimise_cse
-#endif /* __riscos__ */
-
-static int
-bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
-{
-    int rc = do_bf_setkey (c, key, keylen);
-    burn_stack (64);
-    return rc;
-}
 
 /****************
  * Return some information about the algorithm.  We need algo here to
