@@ -1,5 +1,5 @@
 /* cast5.c  -  CAST5 cipher (RFC2144)
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -39,13 +39,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "g10lib.h"
 #include "types.h"
-#include "errors.h"
 #include "cast5.h"
 
 
 #define CIPHER_ALGO_CAST5	 3
 
+#define FNCCAST_SETKEY(f)  (int(*)(void*, byte*, unsigned))(f)
+#define FNCCAST_CRYPT(f)   (void(*)(void*, byte*, byte*))(f)
+
 #define CAST5_BLOCKSIZE 8
 
 typedef struct {
@@ -355,19 +358,7 @@ rol(int n, u32 x)
     (((s1[I >> 24] + s2[(I>>16)&0xff]) ^ s3[(I>>8)&0xff]) - s4[I&0xff]) )
 
 static void
-burn_stack (int bytes)
-{
-    char buf[64];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
-
-static void
-do_encrypt_block( CAST5_context *c, byte *outbuf, byte *inbuf )
+encrypt_block( CAST5_context *c, byte *outbuf, byte *inbuf )
 {
     u32 l, r, t;
     u32 I;   /* used by the Fx macros */
@@ -421,14 +412,7 @@ do_encrypt_block( CAST5_context *c, byte *outbuf, byte *inbuf )
 }
 
 static void
-encrypt_block( CAST5_context *c, byte *outbuf, byte *inbuf )
-{
-    do_encrypt_block (c, outbuf, inbuf);
-    burn_stack (20+4*sizeof(void*));
-}
-
-static void
-do_decrypt_block (CAST5_context *c, byte *outbuf, byte *inbuf )
+decrypt_block(	CAST5_context *c, byte *outbuf, byte *inbuf )
 {
     u32 l, r, t;
     u32 I;
@@ -468,12 +452,6 @@ do_decrypt_block (CAST5_context *c, byte *outbuf, byte *inbuf )
     outbuf[7] =  l	  & 0xff;
 }
 
-static void
-decrypt_block( CAST5_context *c, byte *outbuf, byte *inbuf )
-{
-    do_decrypt_block (c, outbuf, inbuf);
-    burn_stack (20+4*sizeof(void*));
-}
 
 
 static const char*
@@ -572,7 +550,7 @@ key_schedule( u32 *x, u32 *z, u32 *k )
 
 
 static int
-do_cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
+cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
 {
   static int initialized;
   static const char* selftest_failed;
@@ -588,10 +566,10 @@ do_cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
 	    fprintf(stderr,"CAST5 selftest failed (%s).\n", selftest_failed );
     }
     if( selftest_failed )
-	return G10ERR_SELFTEST_FAILED;
+	return GCRYERR_SELFTEST;
 
     if( keylen != 16 )
-	return G10ERR_WRONG_KEYLEN;
+	return GCRYERR_INV_KEYLEN;
 
     x[0] = key[0]  << 24 | key[1]  << 16 | key[2]  << 8 | key[3];
     x[1] = key[4]  << 24 | key[5]  << 16 | key[6]  << 8 | key[7];
@@ -614,13 +592,6 @@ do_cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
     return 0;
 }
 
-static int
-cast_setkey( CAST5_context *c, byte *key, unsigned keylen )
-{
-    int rc = do_cast_setkey (c, key, keylen);
-    burn_stack (96+7*sizeof(void*));
-    return rc;
-}
 
 /****************
  * Return some information about the algorithm.  We need algo here to
