@@ -1,5 +1,12 @@
 /* des.c - DES and Triple-DES encryption/decryption Algorithm
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *
+ * Please see below for more legal information!
+ *
+ * According to the definition of DES in FIPS PUB 46-2 from December 1993.
+ * For a description of triple encryption, see:
+ *   Bruce Schneier: Applied Cryptography. Second Edition.
+ *   John Wiley & Sons, 1996. ISBN 0-471-12845-7. Pages 358 ff.
  *
  * This file is part of GnuPG.
  *
@@ -16,12 +23,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- *
- *
- * According to the definition of DES in FIPS PUB 46-2 from December 1993.
- * For a description of triple encryption, see:
- *   Bruce Schneier: Applied Cryptography. Second Edition.
- *   John Wiley & Sons, 1996. ISBN 0-471-12845-7. Pages 358 ff.
  */
 
 
@@ -115,7 +116,7 @@
 #include <stdio.h>
 #include <string.h>	       /* memcpy, memcmp */
 #include "types.h"             /* for byte and u32 typedefs */
-#include "errors.h"
+#include "g10lib.h"
 #include "des.h"
 
 #if defined(__GNUC__) && defined(__GNU_LIBRARY__)
@@ -146,6 +147,9 @@ working_memcmp( const char *a, const char *b, size_t n )
 #endif
 
 
+/* Macros used by the info function. */
+#define FNCCAST_SETKEY(f)  ((int(*)(void*, byte*, unsigned))(f))
+#define FNCCAST_CRYPT(f)   ((void(*)(void*, byte*, byte*))(f))
 
 
 /*
@@ -448,16 +452,9 @@ static byte weak_keys[64][8] =
 #define tripledes_ecb_decrypt(ctx, from, to)	tripledes_ecb_crypt(ctx, from, to, 1)
 
 
-static void
-burn_stack (int bytes)
-{
-    char buf[64];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
+
+
+
 
 /*
  * des_key_schedule():	  Calculate 16 subkeys pairs (even/odd) for
@@ -561,10 +558,9 @@ des_setkey (struct _des_ctx *ctx, const byte * key)
   int i;
 
   if( selftest_failed )
-    return G10ERR_SELFTEST_FAILED;
+    return GCRYERR_SELFTEST;
 
   des_key_schedule (key, ctx->encrypt_subkeys);
-  burn_stack (32);
 
   for(i=0; i<32; i+=2)
     {
@@ -623,7 +619,6 @@ tripledes_set2keys (struct _tripledes_ctx *ctx,
 
   des_key_schedule (key1, ctx->encrypt_subkeys);
   des_key_schedule (key2, &(ctx->decrypt_subkeys[32]));
-  burn_stack (32);
 
   for(i=0; i<32; i+=2)
     {
@@ -661,7 +656,6 @@ tripledes_set3keys (struct _tripledes_ctx *ctx,
   des_key_schedule (key1, ctx->encrypt_subkeys);
   des_key_schedule (key2, &(ctx->decrypt_subkeys[32]));
   des_key_schedule (key3, &(ctx->encrypt_subkeys[64]));
-  burn_stack (32);
 
   for(i=0; i<32; i+=2)
     {
@@ -950,17 +944,14 @@ static int
 do_tripledes_setkey ( struct _tripledes_ctx *ctx, byte *key, unsigned keylen )
 {
     if( selftest_failed )
-	return G10ERR_SELFTEST_FAILED;
+	return GCRYERR_SELFTEST;
     if( keylen != 24 )
-	return G10ERR_WRONG_KEYLEN;
+	return GCRYERR_INV_KEYLEN;
 
     tripledes_set3keys ( ctx, key, key+8, key+16);
 
-    if( is_weak_key( key ) || is_weak_key( key+8 ) || is_weak_key( key+16 ) ) {
-        burn_stack (64);
-	return G10ERR_WEAK_KEY;
-    }
-    burn_stack (64); 
+    if( is_weak_key( key ) || is_weak_key( key+8 ) || is_weak_key( key+16 ) )
+	return GCRYERR_WEAK_KEY;
 
     return 0;
 }
@@ -970,14 +961,12 @@ static void
 do_tripledes_encrypt( struct _tripledes_ctx *ctx, byte *outbuf, byte *inbuf )
 {
     tripledes_ecb_encrypt ( ctx, inbuf, outbuf );
-    burn_stack (32);
 }
 
 static void
 do_tripledes_decrypt( struct _tripledes_ctx *ctx, byte *outbuf, byte *inbuf )
 {
     tripledes_ecb_decrypt ( ctx, inbuf, outbuf );
-    burn_stack (32);
 }
 
 
@@ -1007,6 +996,7 @@ des_get_info( int algo, size_t *keylen,
 	}
     }
 
+
     if( algo == CIPHER_ALGO_3DES ) {
 	*keylen = 192;
 	*blocksize = 8;
