@@ -1,5 +1,5 @@
 /* dynload.c - load cipher extensions
- *	Copyright (C) 1998, 1999, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -31,10 +31,7 @@
   #include <dl.h>
   #include <errno.h>
 #endif
-#ifdef __MINGW32__
-  #include <windows.h>
-#endif
-#include "util.h"
+#include "g10lib.h"
 #include "cipher.h"
 #include "dynload.h"
 
@@ -70,57 +67,6 @@ dlsym(void *handle, char *name)
 }
 #endif /*HAVE_DL_SHL_LOAD*/
 
-#ifdef __MINGW32__
-#define HAVE_DL_DLOPEN
-#define USE_DYNAMIC_LINKING
-
-static int last_error = 0;
-    
-void*
-dlopen(const char *pathname, int mode)
-{
-	void *h = LoadLibrary( pathname );
-	if (!h) {
-	log_error( "LoadLibrary failed ec=%d\n", (int)GetLastError() );
-	last_error = 1;
-	return NULL;
-	}
-	return h;
-}
-
-int
-dlclose( void *handle )
-{
-	last_error = 0;
-	return	FreeLibrary( handle );
-}
-
-char*
-dlerror(void)
-{
-	static char dlerrstr[10];
-	if (last_error) {
-	sprintf(dlerrstr, "%d", (int)GetLastError() );
-	return dlerrstr;
-	}
-	return NULL;
-}
-
-void*
-dlsym( void *handle, const char *name )
-{
-	void *h = GetProcAddress( handle, name );
-	if (!h) {
-	log_error( "GetProcAddress failed ec=%d\n", (int)GetLastError() );
-	last_error = 1;
-	return NULL;
-	}
-	return h;
-}
-#endif /*__MINGW32__*/
-
-
-
 
 
 typedef struct ext_list {
@@ -155,6 +101,20 @@ static int dld_available;
 #endif
 
 
+static int
+cmp_filenames( const char *a, const char *b )
+{
+    /* ? check whether this is an absolute filename and
+     * resolve symlinks?
+     */
+  #ifdef HAVE_DRIVE_LETTERS
+    return stricmp(a,b);
+  #else
+    return strcmp(a,b);
+  #endif
+}
+
+
 /****************
  * Register an extension module.  The last registered module will
  * be loaded first.  A name may have a list of classes
@@ -178,21 +138,9 @@ register_cipher_extension( const char *mainpgm, const char *fname )
     if( !mainpgm_path && mainpgm && *mainpgm )
 	mainpgm_path = m_strdup(mainpgm);
   #endif
-    if( *fname != DIRSEP_C ) { /* do tilde expansion etc */
-	char *tmp;
+    el = g10_xcalloc( 1, sizeof *el + strlen(fname) );
+    strcpy(el->name, fname );
 
-	if( strchr(fname, DIRSEP_C) )
-	    tmp = make_filename(fname, NULL);
-	else
-	    tmp = make_filename(GNUPG_LIBDIR, fname, NULL);
-	el = m_alloc_clear( sizeof *el + strlen(tmp) );
-	strcpy(el->name, tmp );
-	m_free(tmp);
-    }
-    else {
-	el = m_alloc_clear( sizeof *el + strlen(fname) );
-	strcpy(el->name, fname );
-    }
     /* check whether we have a class hint */
     if( (p=strchr(el->name,'(')) && (pe=strchr(p+1,')')) && !pe[1] ) {
 	*p = *pe = 0;
@@ -204,9 +152,9 @@ register_cipher_extension( const char *mainpgm, const char *fname )
     /* check that it is not already registered */
     intex = NULL;
     for(r = extensions; r; r = r->next ) {
-	if( !compare_filenames(r->name, el->name) ) {
+	if( !cmp_filenames(r->name, el->name) ) {
 	    log_info("extension `%s' already registered\n", el->name );
-	    m_free(el);
+	    g10_free(el);
 	    return;
 	}
 	else if( r->internal )
@@ -234,15 +182,15 @@ register_internal_cipher_extension(
 {
     EXTLIST r, el;
 
-    el = m_alloc_clear( sizeof *el + strlen(module_id) );
+    el = g10_xcalloc( 1, sizeof *el + strlen(module_id) );
     strcpy(el->name, module_id );
     el->internal = 1;
 
     /* check that it is not already registered */
     for(r = extensions; r; r = r->next ) {
-	if( !compare_filenames(r->name, el->name) ) {
+	if( !cmp_filenames(r->name, el->name) ) {
 	    log_info("extension `%s' already registered\n", el->name );
-	    m_free(el);
+	    g10_free(el);
 	    return;
 	}
     }
@@ -273,11 +221,9 @@ load_extension( EXTLIST el )
     int rc;
   #endif
 
-  #ifndef __MINGW32__
     /* make sure we are not setuid */
     if( getuid() != geteuid() )
 	log_bug("trying to load an extension while still setuid\n");
-  #endif
 
     /* now that we are not setuid anymore, we can safely load modules */
   #ifdef HAVE_DL_DLOPEN
@@ -324,7 +270,7 @@ load_extension( EXTLIST el )
     name = (char**)addr;
   #endif
 
-    if( g10_opt_verbose > 1 )
+    if( g10_log_verbosity( 2 ) )
 	log_info("%s: %s%s%s%s\n", el->name, *name,
 		  el->hintstr? " (":"",
 		  el->hintstr? el->hintstr:"",
@@ -355,7 +301,7 @@ load_extension( EXTLIST el )
   #endif
 
   #ifdef HAVE_DL_DLOPEN
-    if( g10_opt_verbose > 2 ) {
+    if( g10_log_verbosity( 3 ) ) {
 	/* list the contents of the module */
 	while( (sym = (*el->enumfunc)(0, &seq, &class, &vers)) ) {
 	    if( vers != 1 ) {
@@ -392,13 +338,7 @@ load_extension( EXTLIST el )
     return -1;
 }
 
-#ifdef __riscos__
-typedef
-const char *(*DIGESTS_CAST)(int, size_t*,byte**, int*, int*,
-                            void (**)(void*),
-                            void (**)(void*,byte*,size_t),
-                            void (**)(void*),byte *(**)(void*));
-#endif /* __riscos__ */
+
 
 int
 enum_gnupgext_digests( void **enum_context,
@@ -412,13 +352,13 @@ enum_gnupgext_digests( void **enum_context,
     ENUMCONTEXT *ctx;
 
     if( !*enum_context ) { /* init context */
-	ctx = m_alloc_clear( sizeof( *ctx ) );
+	ctx = g10_xcalloc( 1, sizeof( *ctx ) );
 	ctx->r = extensions;
 	ctx->reqalgo = *algo;
 	*enum_context = ctx;
     }
     else if( !algo ) { /* release the context */
-	m_free(*enum_context);
+	g10_free(*enum_context);
 	*enum_context = NULL;
 	return 0;
     }
@@ -441,11 +381,7 @@ enum_gnupgext_digests( void **enum_context,
 	    if( vers != 1 || class != 10 )
 		continue;
 	  inner_loop:
-#ifndef __riscos__
 	    *r_get_info = ctx->sym;
-#else /* __riscos__ */
-	    *r_get_info = (DIGESTS_CAST) ctx->sym;
-#endif /* __riscos__ */
 	    while( (sym = (*r->enumfunc)(11, &ctx->seq2, &class, &vers)) ) {
 		if( vers != 1 || class != 11 )
 		    continue;
@@ -461,14 +397,6 @@ enum_gnupgext_digests( void **enum_context,
     return 0;
 }
 
-#ifdef __riscos__
-typedef
-const char *(*CIPHERS_CAST)(int, size_t*, size_t*, size_t*,
-                            int  (**)( void *, byte *, unsigned),
-                            void (**)( void *, byte *, byte *),
-                            void (**)( void *, byte *, byte *));
-#endif /* __riscos__ */
-
 const char *
 enum_gnupgext_ciphers( void **enum_context, int *algo,
 		       size_t *keylen, size_t *blocksize, size_t *contextsize,
@@ -485,12 +413,12 @@ enum_gnupgext_ciphers( void **enum_context, int *algo,
 			  void (**)( void *, byte *, byte *));
 
     if( !*enum_context ) { /* init context */
-	ctx = m_alloc_clear( sizeof( *ctx ) );
+	ctx = g10_xcalloc( 1, sizeof( *ctx ) );
 	ctx->r = extensions;
 	*enum_context = ctx;
     }
     else if( !algo ) { /* release the context */
-	m_free(*enum_context);
+	g10_free(*enum_context);
 	*enum_context = NULL;
 	return NULL;
     }
@@ -513,11 +441,7 @@ enum_gnupgext_ciphers( void **enum_context, int *algo,
 	    if( vers != 1 || class != 20 )
 		continue;
 	  inner_loop:
-#ifndef __riscos__
 	    finfo = ctx->sym;
-#else /* __riscos__ */
-	    finfo = (CIPHERS_CAST) ctx->sym;
-#endif /* __riscos__ */
 	    while( (sym = (*r->enumfunc)(21, &ctx->seq2, &class, &vers)) ) {
 		const char *algname;
 		if( vers != 1 || class != 21 )
@@ -538,19 +462,6 @@ enum_gnupgext_ciphers( void **enum_context, int *algo,
     return NULL;
 }
 
-#ifdef __riscos__
-typedef
-const char *(*PUBKEYS_CAST)(int, int *, int *, int *, int *, int *,
-                            int (**)(int, unsigned, MPI *, MPI **),
-                            int (**)(int, MPI *),
-                            int (**)(int, MPI *, MPI , MPI *),
-                            int (**)(int, MPI *, MPI *, MPI *),
-                            int (**)(int, MPI *, MPI , MPI *),
-                            int (**)(int, MPI , MPI *, MPI *,
-	                             int (*)(void*,MPI), void *),
-                            unsigned (**)( int , MPI *));
-#endif /* __riscos__ */
-
 const char *
 enum_gnupgext_pubkeys( void **enum_context, int *algo,
     int *npkey, int *nskey, int *nenc, int *nsig, int *use,
@@ -576,12 +487,12 @@ enum_gnupgext_pubkeys( void **enum_context, int *algo,
 			   unsigned (**)( int , MPI * ) );
 
     if( !*enum_context ) { /* init context */
-	ctx = m_alloc_clear( sizeof( *ctx ) );
+	ctx = g10_xcalloc( 1, sizeof( *ctx ) );
 	ctx->r = extensions;
 	*enum_context = ctx;
     }
     else if( !algo ) { /* release the context */
-	m_free(*enum_context);
+	g10_free(*enum_context);
 	*enum_context = NULL;
 	return NULL;
     }
@@ -603,11 +514,7 @@ enum_gnupgext_pubkeys( void **enum_context, int *algo,
 	    if( vers != 1 || class != 30 )
 		continue;
 	  inner_loop:
-#ifndef __riscos__
 	    finfo = ctx->sym;
-#else /* __riscos__ */
-	    finfo = (PUBKEYS_CAST) ctx->sym;
-#endif /* __riscos__ */
 	    while( (sym = (*r->enumfunc)(31, &ctx->seq2, &class, &vers)) ) {
 		const char *algname;
 		if( vers != 1 || class != 31 )
