@@ -1,5 +1,5 @@
 /* elgamal.c  -  ElGamal Public Key encryption
- *	Copyright (C) 1998, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
  *
  * For a description of the algorithm, see:
  *   Bruce Schneier: Applied Cryptography. John Wiley & Sons, 1996.
@@ -26,7 +26,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "util.h"
+#include "g10lib.h"
 #include "mpi.h"
 #include "cipher.h"
 #include "elgamal.h"
@@ -50,7 +50,7 @@ static void test_keys( ELG_secret_key *sk, unsigned nbits );
 static MPI gen_k( MPI p );
 static void generate( ELG_secret_key *sk, unsigned nbits, MPI **factors );
 static int  check_secret_key( ELG_secret_key *sk );
-static void do_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey );
+static void encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey );
 static void decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey );
 static void sign(MPI a, MPI b, MPI input, ELG_secret_key *skey);
 static int  verify(MPI a, MPI b, MPI input, ELG_public_key *pkey);
@@ -78,7 +78,7 @@ progress( int c )
 
 
 /****************
- * Michael Wiener's table about subgroup sizes to match field sizes
+ * Michael Wiener's table on subgroup sizes to match field sizes
  * (floating around somewhere - Fixme: need a reference)
  */
 static unsigned int
@@ -130,13 +130,9 @@ test_keys( ELG_secret_key *sk, unsigned nbits )
     pk.g = sk->g;
     pk.y = sk->y;
 
-    /*mpi_set_bytes( test, nbits, get_random_byte, 0 );*/
-    {	char *p = get_random_bits( nbits, 0, 0 );
-	mpi_set_buffer( test, p, (nbits+7)/8, 0 );
-	m_free(p);
-    }
+    gcry_mpi_randomize( test, nbits, GCRY_WEAK_RANDOM );
 
-    do_encrypt( out1_a, out1_b, test, &pk );
+    encrypt( out1_a, out1_b, test, &pk );
     decrypt( out2, out1_a, out1_b, sk );
     if( mpi_cmp( test, out2 ) )
 	log_fatal("ElGamal operation: encrypt, decrypt failed\n");
@@ -163,8 +159,7 @@ gen_k( MPI p )
     MPI temp = mpi_alloc( mpi_get_nlimbs(p) );
     MPI p_1 = mpi_copy(p);
     unsigned int orig_nbits = mpi_get_nbits(p);
-    unsigned int nbits;
-    unsigned int nbytes;
+    unsigned int nbits, nbytes;
     char *rndbuf = NULL;
 
     /* IMO using a k much lesser than p is sufficient and it greatly
@@ -177,22 +172,22 @@ gen_k( MPI p )
 
     nbytes = (nbits+7)/8;
     if( DBG_CIPHER )
-	log_debug("choosing a random k of %u bits", nbits);
+	log_debug("choosing a random k ");
     mpi_sub_ui( p_1, p, 1);
     for(;;) {
 	if( !rndbuf || nbits < 32 ) {
-	    m_free(rndbuf);
-	    rndbuf = get_random_bits( nbits, 1, 1 );
+	    g10_free(rndbuf);
+	    rndbuf = gcry_random_bytes_secure( nbytes, GCRY_STRONG_RANDOM );
 	}
 	else { /* change only some of the higher bits */
-	    /* we could impprove this by directly requesting more memory
-	     * at the first call to get_random_bits() and use this the here
+	    /* we could improve this by directly requesting more memory
+	     * at the first call to get_random_bytes() and use this the here
 	     * maybe it is easier to do this directly in random.c
 	     * Anyway, it is highly inlikely that we will ever reach this code
 	     */
-	    char *pp = get_random_bits( 32, 1, 1 );
-	    memcpy( rndbuf,pp, 4 );
-	    m_free(pp);
+	    char *pp = gcry_random_bytes_secure( 4, GCRY_STRONG_RANDOM );
+	    memcpy( rndbuf, pp, 4 );
+	    g10_free(pp);
 	    log_debug("gen_k: tsss, never expected to reach this\n");
 	}
 	mpi_set_buffer( k, rndbuf, nbytes, 0 );
@@ -219,7 +214,7 @@ gen_k( MPI p )
 	}
     }
   found:
-    m_free(rndbuf);
+    g10_free(rndbuf);
     if( DBG_CIPHER )
 	progress('\n');
     mpi_free(p_1);
@@ -279,24 +274,28 @@ generate(  ELG_secret_key *sk, unsigned int nbits, MPI **ret_factors )
 	    progress('.');
 	if( rndbuf ) { /* change only some of the higher bits */
 	    if( xbits < 16 ) {/* should never happen ... */
-		m_free(rndbuf);
-		rndbuf = get_random_bits( xbits, 2, 1 );
+		g10_free(rndbuf);
+		rndbuf = gcry_random_bytes_secure( (xbits+7)/8,
+						   GCRY_VERY_STRONG_RANDOM );
 	    }
 	    else {
-		char *r = get_random_bits( 16, 2, 1 );
-		memcpy(rndbuf, r, 16/8 );
-		m_free(r);
+		char *r = gcry_random_bytes_secure( 2,
+						   GCRY_VERY_STRONG_RANDOM );
+		memcpy(rndbuf, r, 2 );
+		g10_free(r);
 	    }
 	}
-	else
-	    rndbuf = get_random_bits( xbits, 2, 1 );
+	else {
+	    rndbuf = gcry_random_bytes_secure( (xbits+7)/8,
+					       GCRY_VERY_STRONG_RANDOM );
+	}
 	mpi_set_buffer( x, rndbuf, (xbits+7)/8, 0 );
 	mpi_clear_highbit( x, xbits+1 );
     } while( !( mpi_cmp_ui( x, 0 )>0 && mpi_cmp( x, p_min1 )<0 ) );
-    m_free(rndbuf);
+    g10_free(rndbuf);
 
     y = mpi_alloc(nbits/BITS_PER_MPI_LIMB);
-    mpi_powm( y, g, x, p );
+    gcry_mpi_powm( y, g, x, p );
 
     if( DBG_CIPHER ) {
 	progress('\n');
@@ -330,7 +329,7 @@ check_secret_key( ELG_secret_key *sk )
     int rc;
     MPI y = mpi_alloc( mpi_get_nlimbs(sk->y) );
 
-    mpi_powm( y, sk->g, sk->x, sk->p );
+    gcry_mpi_powm( y, sk->g, sk->x, sk->p );
     rc = !mpi_cmp( y, sk->y );
     mpi_free( y );
     return rc;
@@ -338,7 +337,7 @@ check_secret_key( ELG_secret_key *sk )
 
 
 static void
-do_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
+encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
 {
     MPI k;
 
@@ -348,13 +347,13 @@ do_encrypt(MPI a, MPI b, MPI input, ELG_public_key *pkey )
      */
 
     k = gen_k( pkey->p );
-    mpi_powm( a, pkey->g, k, pkey->p );
+    gcry_mpi_powm( a, pkey->g, k, pkey->p );
     /* b = (y^k * input) mod p
      *	 = ((y^k mod p) * (input mod p)) mod p
      * and because input is < p
      *	 = ((y^k mod p) * input) mod p
      */
-    mpi_powm( b, pkey->y, k, pkey->p );
+    gcry_mpi_powm( b, pkey->y, k, pkey->p );
     mpi_mulm( b, b, input, pkey->p );
   #if 0
     if( DBG_CIPHER ) {
@@ -378,7 +377,7 @@ decrypt(MPI output, MPI a, MPI b, ELG_secret_key *skey )
     MPI t1 = mpi_alloc_secure( mpi_get_nlimbs( skey->p ) );
 
     /* output = b/(a^x) mod p */
-    mpi_powm( t1, a, skey->x, skey->p );
+    gcry_mpi_powm( t1, a, skey->x, skey->p );
     mpi_invm( t1, t1, skey->p );
     mpi_mulm( output, b, t1, skey->p );
   #if 0
@@ -414,13 +413,9 @@ sign(MPI a, MPI b, MPI input, ELG_secret_key *skey )
     */
     mpi_sub_ui(p_1, p_1, 1);
     k = gen_k( skey->p );
-    mpi_powm( a, skey->g, k, skey->p );
+    gcry_mpi_powm( a, skey->g, k, skey->p );
     mpi_mul(t, skey->x, a );
     mpi_subm(t, input, t, p_1 );
-    while( mpi_is_neg(t) ) {
-	BUG();	/* That is nonsense code - left over from a very early test?*/
-	mpi_add(t, t, p_1);
-    }
     mpi_invm(inv, k, p_1 );
     mpi_mulm(b, t, inv, p_1 );
 
@@ -463,12 +458,12 @@ verify(MPI a, MPI b, MPI input, ELG_public_key *pkey )
 
   #if 0
     /* t1 = (y^a mod p) * (a^b mod p) mod p */
-    mpi_powm( t1, pkey->y, a, pkey->p );
-    mpi_powm( t2, a, b, pkey->p );
+    gcry_mpi_powm( t1, pkey->y, a, pkey->p );
+    gcry_mpi_powm( t2, a, b, pkey->p );
     mpi_mulm( t1, t1, t2, pkey->p );
 
     /* t2 = g ^ input mod p */
-    mpi_powm( t2, pkey->g, input, pkey->p );
+    gcry_mpi_powm( t2, pkey->g, input, pkey->p );
 
     rc = !mpi_cmp( t1, t2 );
   #elif 0
@@ -479,7 +474,7 @@ verify(MPI a, MPI b, MPI input, ELG_public_key *pkey )
     mpi_mulpowm( t1, base, exp, pkey->p );
 
     /* t2 = g ^ input mod p */
-    mpi_powm( t2, pkey->g, input, pkey->p );
+    gcry_mpi_powm( t2, pkey->g, input, pkey->p );
 
     rc = !mpi_cmp( t1, t2 );
   #else
@@ -509,7 +504,7 @@ elg_generate( int algo, unsigned nbits, MPI *skey, MPI **retfactors )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
 
     generate( &sk, nbits, retfactors );
     skey[0] = sk.p;
@@ -526,16 +521,16 @@ elg_check_secret_key( int algo, MPI *skey )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
     if( !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return G10ERR_BAD_MPI;
+	return GCRYERR_BAD_MPI;
 
     sk.p = skey[0];
     sk.g = skey[1];
     sk.y = skey[2];
     sk.x = skey[3];
     if( !check_secret_key( &sk ) )
-	return G10ERR_BAD_SECKEY;
+	return GCRYERR_BAD_SECRET_KEY;
 
     return 0;
 }
@@ -548,16 +543,16 @@ elg_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey )
     ELG_public_key pk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
     if( !data || !pkey[0] || !pkey[1] || !pkey[2] )
-	return G10ERR_BAD_MPI;
+	return GCRYERR_BAD_MPI;
 
     pk.p = pkey[0];
     pk.g = pkey[1];
     pk.y = pkey[2];
     resarr[0] = mpi_alloc( mpi_get_nlimbs( pk.p ) );
     resarr[1] = mpi_alloc( mpi_get_nlimbs( pk.p ) );
-    do_encrypt( resarr[0], resarr[1], data, &pk );
+    encrypt( resarr[0], resarr[1], data, &pk );
     return 0;
 }
 
@@ -567,10 +562,10 @@ elg_decrypt( int algo, MPI *result, MPI *data, MPI *skey )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
     if( !data[0] || !data[1]
 	|| !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return G10ERR_BAD_MPI;
+	return GCRYERR_BAD_MPI;
 
     sk.p = skey[0];
     sk.g = skey[1];
@@ -587,9 +582,9 @@ elg_sign( int algo, MPI *resarr, MPI data, MPI *skey )
     ELG_secret_key sk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
     if( !data || !skey[0] || !skey[1] || !skey[2] || !skey[3] )
-	return G10ERR_BAD_MPI;
+	return GCRYERR_BAD_MPI;
 
     sk.p = skey[0];
     sk.g = skey[1];
@@ -608,16 +603,16 @@ elg_verify( int algo, MPI hash, MPI *data, MPI *pkey,
     ELG_public_key pk;
 
     if( !is_ELGAMAL(algo) )
-	return G10ERR_PUBKEY_ALGO;
+	return GCRYERR_INV_PK_ALGO;
     if( !data[0] || !data[1] || !hash
 	|| !pkey[0] || !pkey[1] || !pkey[2] )
-	return G10ERR_BAD_MPI;
+	return GCRYERR_BAD_MPI;
 
     pk.p = pkey[0];
     pk.g = pkey[1];
     pk.y = pkey[2];
     if( !verify( data[0], data[1], hash, &pk ) )
-	return G10ERR_BAD_SIGN;
+	return GCRYERR_BAD_SIGNATURE;
     return 0;
 }
 
@@ -653,11 +648,11 @@ elg_get_info( int algo, int *npkey, int *nskey, int *nenc, int *nsig,
     *nsig = 2;
 
     switch( algo ) {
-      case PUBKEY_ALGO_ELGAMAL:
-	*use = PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC;
+      case GCRY_PK_ELG:
+	*use = GCRY_PK_USAGE_SIGN|GCRY_PK_USAGE_ENCR;
 	return "ELG";
-      case PUBKEY_ALGO_ELGAMAL_E:
-	*use = PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC;
+      case GCRY_PK_ELG_E:
+	*use = GCRY_PK_USAGE_SIGN|GCRY_PK_USAGE_ENCR;
 	return "ELG-E";
       default: *use = 0; return NULL;
     }
