@@ -26,9 +26,7 @@
 #include <assert.h>
 
 #include "g10lib.h"
-#include "util.h"
 #include "cipher.h"
-#include "errors.h"
 #include "dynload.h"
 #include "rmd.h"
 
@@ -43,13 +41,14 @@ struct gcry_md_context {
     FILE  *debug;
     int finalized;
     struct md_digest_list_s *list;
+    byte *macpads;
 };
 #define CTX_MAGIC_NORMAL 0x11071961
 #define CTX_MAGIC_SECURE 0x16917011
 
 static const char * digest_algo_to_string( int algo );
 static int check_digest_algo( int algo );
-static GCRY_MD_HD md_open( int algo, int secure );
+static GCRY_MD_HD md_open( int algo, int secure, int hmac );
 static int  md_enable( GCRY_MD_HD hd, int algo );
 static GCRY_MD_HD md_copy( GCRY_MD_HD a );
 static void md_close(GCRY_MD_HD a);
@@ -93,13 +92,13 @@ new_list_item( int algo,
 {
     struct md_digest_list_s *r;
 
-    r = m_alloc_clear( sizeof *r );
+    r = g10_xcalloc( 1, sizeof *r );
     r->algo = algo,
     r->name = (*get_info)( algo, &r->contextsize,
 			   &r->asnoid, &r->asnlen, &r->mdlen,
 			   &r->init, &r->write, &r->final, &r->read );
     if( !r->name ) {
-	m_free(r);
+	g10_free(r);
 	r = NULL;
     }
     return r;
@@ -159,7 +158,7 @@ load_digest_module( int req_algo )
 	    continue;
 	}
 	/* put it into the list */
-	if( g10_opt_verbose > 1 )
+	if( g10_log_verbosity( 2 ) )
 	    log_info("loaded digest %d\n", algo);
 	r->next = digest_list;
 	digest_list = r;
@@ -230,7 +229,7 @@ check_digest_algo( int algo )
 	    if( r->algo == algo )
 		return 0;
     } while( !r && load_digest_module(algo) );
-    return G10ERR_DIGEST_ALGO;
+    return GCRYERR_INV_MD_ALGO;
 }
 
 
@@ -241,7 +240,7 @@ check_digest_algo( int algo )
  * may be 0.
  */
 static GCRY_MD_HD
-md_open( int algo, int secure )
+md_open( int algo, int secure, int hmac )
 {
     GCRY_MD_HD hd;
     struct gcry_md_context *ctx;
@@ -267,8 +266,13 @@ md_open( int algo, int secure )
 	 / sizeof(PROPERLY_ALIGNED_TYPE) ) * sizeof(PROPERLY_ALIGNED_TYPE);
 
     /* allocate and set the Context pointer to the private data */
-    hd = secure ? m_alloc_secure( n + sizeof( struct gcry_md_context ) )
-		: m_alloc(	  n + sizeof( struct gcry_md_context ) );
+    hd = secure ? g10_malloc_secure( n + sizeof( struct gcry_md_context ) )
+		: g10_malloc(	     n + sizeof( struct gcry_md_context ) );
+    if( !hd ) {
+	set_lasterr( GCRYERR_NO_MEM );
+	return NULL;
+    }
+
     hd->ctx = ctx = (struct gcry_md_context*)( (char*)hd + n );
     /* setup the globally visible data (bctl in the diagram)*/
     hd->bufsize = n - sizeof( struct gcry_md_handle ) + 1;
@@ -277,6 +281,14 @@ md_open( int algo, int secure )
     memset( hd->ctx, 0, sizeof *hd->ctx );
     ctx->magic = secure ? CTX_MAGIC_SECURE : CTX_MAGIC_NORMAL;
     ctx->secure = secure;
+    if( hmac ) {
+	ctx->macpads = g10_malloc_secure( 128 );
+	if( !ctx->macpads ) {
+	    md_close( hd );
+	    set_lasterr( GCRYERR_NO_MEM );
+	    return NULL;
+	}
+    }
     fast_random_poll(); /* FIXME: should we really do that? */
     if( algo && md_enable( hd, algo ) ) {
 	md_close( hd );
@@ -292,7 +304,8 @@ gcry_md_open( int algo, unsigned int flags )
     GCRY_MD_HD hd;
     /* fixme: check that algo is available and that only valid
      * flag values are used */
-    hd = md_open( algo, (flags & GCRY_MD_FLAG_SECURE) );
+    hd = md_open( algo, (flags & GCRY_MD_FLAG_SECURE),
+			(flags & GCRY_MD_FLAG_HMAC) );
     return hd;
 }
 
@@ -315,13 +328,16 @@ md_enable( GCRY_MD_HD hd, int algo )
     } while( !r && load_digest_module( algo ) );
     if( !r ) {
 	log_debug("md_enable: algorithm %d not available\n", algo );
-	return set_lasterr( GCRYERR_INV_ALGO );
+	return set_lasterr( GCRYERR_INV_MD_ALGO );
     }
     /* and allocate a new list entry */
-    ac = h->secure? m_alloc_secure( sizeof *ac + r->contextsize
+    ac = h->secure? g10_malloc_secure( sizeof *ac + r->contextsize
 					       - sizeof(r->context) )
-		  : m_alloc( sizeof *ac + r->contextsize
+		  : g10_malloc( sizeof *ac + r->contextsize
 					       - sizeof(r->context) );
+    if( !ac )
+	return set_lasterr( GCRYERR_NO_MEM );
+
     *ac = *r;
     ac->next = h->list;
     h->list = ac;
@@ -350,8 +366,13 @@ md_copy( GCRY_MD_HD ahd )
 	md_write( ahd, NULL, 0 );
 
     n = (char*)ahd->ctx - (char*)ahd;
-    bhd = a->secure ? m_alloc_secure( n + sizeof( struct gcry_md_context ) )
-		    : m_alloc(	      n + sizeof( struct gcry_md_context ) );
+    bhd = a->secure ? g10_malloc_secure( n + sizeof( struct gcry_md_context ) )
+		    : g10_malloc(	 n + sizeof( struct gcry_md_context ) );
+    if( !bhd ) {
+	set_lasterr( GCRYERR_NO_MEM );
+	return NULL;
+    }
+
     bhd->ctx = b = (struct gcry_md_context*)( (char*)bhd + n );
     /* no need to copy the buffer due to the write above */
     assert( ahd->bufsize == (n - sizeof( struct gcry_md_handle ) + 1) );
@@ -360,12 +381,16 @@ md_copy( GCRY_MD_HD ahd )
     memcpy( b, a, sizeof *a );
     b->list = NULL;
     b->debug = NULL;
+    if( a->macpads ) {
+	b->macpads = g10_malloc_secure( 128 );
+	memcpy( b->macpads, a->macpads, 128 );
+    }
     /* and now copy the complete list of algorithms */
     /* I know that the copied list is reversed, but that doesn't matter */
     for( ar=a->list; ar; ar = ar->next ) {
-	br = a->secure ? m_alloc_secure( sizeof *br + ar->contextsize
+	br = a->secure ? g10_xmalloc_secure( sizeof *br + ar->contextsize
 					       - sizeof(ar->context) )
-		       : m_alloc( sizeof *br + ar->contextsize
+		       : g10_xmalloc( sizeof *br + ar->contextsize
 					       - sizeof(ar->context) );
 	memcpy( br, ar, sizeof(*br) + ar->contextsize
 				    - sizeof(ar->context) );
@@ -398,6 +423,9 @@ gcry_md_reset( GCRY_MD_HD a )
 	memset( r->context.c, 0, r->contextsize );
 	(*r->init)( &r->context.c );
     }
+    if( a->ctx->macpads ) {
+	md_write( a, a->ctx->macpads, 64 ); /* inner pad */
+    }
 }
 
 
@@ -412,9 +440,10 @@ md_close(GCRY_MD_HD a)
 	md_stop_debug(a);
     for(r=a->ctx->list; r; r = r2 ) {
 	r2 = r->next;
-	m_free(r);
+	g10_free(r);
     }
-    m_free(a);
+    g10_free(a->ctx->macpads);
+    g10_free(a);
 }
 
 
@@ -468,17 +497,80 @@ md_final(GCRY_MD_HD a)
 	(*r->final)( &r->context.c );
     }
     a->ctx->finalized = 1;
+    if( a->ctx->macpads ) {  /* finish the hmac */
+	int algo = md_get_algo( a );
+	byte *p = md_read( a, algo );
+	size_t dlen = md_digest_length(algo);
+
+	GCRY_MD_HD om = md_open( algo, a->ctx->secure, 0 );
+	if( !om )
+	    g10_fatal_error( gcry_errno(), NULL );
+	md_write( om, a->ctx->macpads+64, 64 );
+	md_write( om, p, dlen );
+	md_final( om );
+	/* replace our digest with the mac (they have the same size) */
+	memcpy( p, md_read( om, algo ), dlen );
+	md_close( om );
+    }
 }
 
 
+
+static int
+prepare_macpads( GCRY_MD_HD hd, byte *key, size_t keylen)
+{
+    int i;
+    int algo = md_get_algo( hd );
+    byte *helpkey = NULL;
+    byte *ipad, *opad;
+
+    if( !algo )
+	return GCRYERR_INV_MD_ALGO; /* i.e. no algo enabled */
+
+    if( keylen > 64 ) {
+	helpkey = g10_malloc_secure( md_digest_length( algo ) );
+	if( !helpkey )
+	    return GCRYERR_NO_MEM;
+	gcry_md_hash_buffer( algo, helpkey, key, keylen );
+	key = helpkey;
+	keylen = md_digest_length( algo );
+	assert( keylen <= 64 );
+    }
+
+    memset( hd->ctx->macpads, 0, 128 );
+    ipad = hd->ctx->macpads;
+    opad = hd->ctx->macpads+64;
+    memcpy( ipad, key, keylen );
+    memcpy( opad, key, keylen );
+    for(i=0; i < 64; i++ ) {
+	ipad[i] ^= 0x36;
+	opad[i] ^= 0x5c;
+    }
+    g10_free( helpkey );
+    return 0;
+}
+
 int
 gcry_md_ctl( GCRY_MD_HD hd, int cmd, byte *buffer, size_t buflen)
 {
+    int rc = 0;
     if( cmd == GCRYCTL_FINALIZE )
 	md_final( hd );
+    else if( cmd == GCRYCTL_SET_KEY ) {
+	if( !(hd->ctx->macpads ) )
+	    rc = GCRYERR_CONFLICT;
+	else if ( !(rc = prepare_macpads( hd, buffer, buflen )) )
+	    gcry_md_reset( hd );
+    }
+    else if( cmd == GCRYCTL_START_DUMP ) {
+	md_start_debug( hd, buffer );
+    }
+    else if( cmd == GCRYCTL_STOP_DUMP ) {
+	md_stop_debug( hd );
+    }
     else
-	return GCRYERR_INV_OP;
-    return 0;
+	rc = GCRYERR_INV_OP;
+    return set_lasterr( rc );
 }
 
 
@@ -555,8 +647,8 @@ md_digest( GCRY_MD_HD a, int algo, byte *buffer, int buflen )
 
     /* I don't want to change the interface, so I simply work on a copy
      * of the context (extra overhead - should be fixed)*/
-    context = a->ctx->secure ? m_alloc_secure( r->contextsize )
-			     : m_alloc( r->contextsize );
+    context = a->ctx->secure ? g10_xmalloc_secure( r->contextsize )
+			     : g10_xmalloc( r->contextsize );
     memcpy( context, r->context.c, r->contextsize );
     (*r->final)( context );
     digest = (*r->read)( context );
@@ -565,7 +657,7 @@ md_digest( GCRY_MD_HD a, int algo, byte *buffer, int buflen )
 	buflen = r->mdlen;
     memcpy( buffer, digest, buflen );
 
-    m_free(context);
+    g10_free(context);
     return buflen;
 }
 #endif
@@ -581,6 +673,27 @@ gcry_md_get( GCRY_MD_HD hd, int algo, byte *buffer, int buflen )
 }
 
 
+/****************
+ * Shortcut function to hash a buffer with a given algo. The only supported
+ * algorithm is RIPE-MD. The supplied digest buffer must be large enough
+ * to store the resulting hash.  No error is returned, the function will
+ * abort on an invalid algo.  DISABLED_ALGOS are ignored here.
+ */
+void
+gcry_md_hash_buffer( int algo, char *digest, const char *buffer, size_t length)
+{
+    if( algo == GCRY_MD_RMD160 )
+	rmd160_hash_buffer( digest, buffer, length );
+    else { /* for the others we do not have a fast function, so
+	    * we use the normal functions to do it */
+	GCRY_MD_HD h = md_open( algo, 0, 0 );
+	if( !h )
+	    BUG(); /* algo not available */
+	md_write( h, (byte*)buffer, length );
+	md_final( h );
+	memcpy( digest, md_read( h, algo ), md_digest_length( algo ) );
+    }
+}
 
 static int
 md_get_algo( GCRY_MD_HD a )
@@ -636,7 +749,7 @@ gcry_md_get_algo_dlen( int algo )
       default: {
 	    int len = md_digest_length( algo );
 	    if( !len )
-		set_lasterr( GCRYERR_INV_ALGO );
+		set_lasterr( GCRYERR_INV_MD_ALGO );
 	    return 0;
 	}
     }
@@ -695,7 +808,7 @@ gcry_md_algo_info( int algo, int what, void *buffer, size_t *nbytes)
 	    return -1;
 	}
 	if( check_digest_algo( algo ) ) {
-	    set_lasterr( GCRYERR_INV_ALGO );
+	    set_lasterr( GCRYERR_INV_MD_ALGO );
 	    return -1;
 	}
 	break;
@@ -727,7 +840,7 @@ gcry_md_algo_info( int algo, int what, void *buffer, size_t *nbytes)
 
 
 
-void
+static void
 md_start_debug( GCRY_MD_HD md, const char *suffix )
 {
     static int idx=0;
@@ -744,7 +857,7 @@ md_start_debug( GCRY_MD_HD md, const char *suffix )
 	log_debug("md debug: can't open %s\n", buf );
 }
 
-void
+static void
 md_stop_debug( GCRY_MD_HD md )
 {
     if( md->ctx->debug ) {
