@@ -1,6 +1,8 @@
 /* md5.c - MD5 Message-Digest Algorithm
- * Copyright (C) 1995, 1996, 1998, 1999,
- *               2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1995, 1996, 1998, 1999 Free Software Foundation, Inc.
+ *
+ * according to the definition of MD5 in RFC 1321 from April 1992.
+ * NOTE: This is *not* the same file as the one from glibc.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -15,12 +17,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * According to the definition of MD5 in RFC 1321 from April 1992.
- * NOTE: This is *not* the same file as the one from glibc.
  */
 /* Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, 1995.  */
-/* Heavily modified for GnuPG by <wk@gnupg.org> */
+/* heavily modified for GnuPG by <werner.koch@guug.de> */
 
 /* Test values:
  * ""                  D4 1D 8C D9 8F 00 B2 04  E9 80 09 98 EC F8 42 7E
@@ -34,7 +33,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include "util.h"
+#include "g10lib.h"
 #include "memory.h"
 #include "dynload.h"
 
@@ -73,18 +72,6 @@ md5_init( MD5_CONTEXT *ctx )
 #define FH(b, c, d) (b ^ c ^ d)
 #define FI(b, c, d) (c ^ (b | ~d))
 
-static void
-burn_stack (int bytes)
-{
-    char buf[128];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
-
 
 /****************
  * transform n*64 bytes
@@ -230,7 +217,6 @@ md5_write( MD5_CONTEXT *hd, byte *inbuf, size_t inlen)
 {
     if( hd->count == 64 ) { /* flush the buffer */
 	transform( hd, hd->buf );
-        burn_stack (80+6*sizeof(void*));
 	hd->count = 0;
 	hd->nblocks++;
     }
@@ -251,9 +237,9 @@ md5_write( MD5_CONTEXT *hd, byte *inbuf, size_t inlen)
 	inlen -= 64;
 	inbuf += 64;
     }
-    burn_stack (80+6*sizeof(void*));
     for( ; inlen && hd->count < 64; inlen-- )
 	hd->buf[hd->count++] = *inbuf++;
+
 }
 
 
@@ -272,19 +258,18 @@ md5_final( MD5_CONTEXT *hd )
 
     md5_write(hd, NULL, 0); /* flush */;
 
+    msb = 0;
     t = hd->nblocks;
-    /* multiply by 64 to make a byte count */
-    lsb = t << 6;
-    msb = t >> 26;
-    /* add the count */
-    t = lsb;
-    if( (lsb += hd->count) < t )
+    if( (lsb = t << 6) < t ) /* multiply by 64 to make a byte count */
 	msb++;
-    /* multiply by 8 to make a bit count */
+    msb += t >> 26;
     t = lsb;
-    lsb <<= 3;
-    msb <<= 3;
-    msb |= t >> 29;
+    if( (lsb = t + hd->count) < t ) /* add the count */
+	msb++;
+    t = lsb;
+    if( (lsb = t << 3) < t ) /* multiply by 8 to make a bit count */
+	msb++;
+    msb += t >> 29;
 
     if( hd->count < 56 ) { /* enough room */
 	hd->buf[hd->count++] = 0x80; /* pad */
@@ -308,14 +293,16 @@ md5_final( MD5_CONTEXT *hd )
     hd->buf[62] = msb >> 16;
     hd->buf[63] = msb >> 24;
     transform( hd, hd->buf );
-    burn_stack (80+6*sizeof(void*));
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
-    #define X(a) do { *p++ = hd-> a      ; *p++ = hd-> a >> 8;      \
-		      *p++ = hd-> a >> 16; *p++ = hd-> a >> 24; } while(0)
+    #define X(a) do { *p++ = hd->##a	  ; *p++ = hd->##a >> 8;      \
+		      *p++ = hd->##a >> 16; *p++ = hd->##a >> 24; } while(0)
   #else /* little endian */
-    #define X(a) do { *(u32*)p = hd-> a ; p += 4; } while(0)
+    /*#define X(a) do { *(u32*)p = hd->##a ; p += 4; } while(0)*/
+    /* Unixware's cpp doesn't like the above construct so we do it his way:
+     * (reported by Allan Clark) */
+    #define X(a) do { *(u32*)p = (*hd).a ; p += 4; } while(0)
   #endif
     X(A);
     X(B);
@@ -369,7 +356,7 @@ md5_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "MD5 ($Revision: 1.15.2.6 $)";
+const char * const gnupgext_version = "MD5 ($Revision: 1.17 $)";
 
 static struct {
     int class;
@@ -398,11 +385,7 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 	*vers  = func_table[i].version;
 	switch( *class ) {
 	  case 11: case 21: case 31: ret = &func_table[i].value; break;
-#ifndef __riscos__
 	  default:		     ret = func_table[i].func; break;
-#else /* __riscos__ */
-	  default:		     ret = (void *) func_table[i].func; break;
-#endif /* __riscos__ */
 	}
 	i++;
     } while( what && what != *class );
