@@ -1,5 +1,5 @@
 /* random.c  -	random number generator
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -32,7 +32,6 @@
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
-#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -44,16 +43,17 @@
 #ifdef HAVE_GETTIMEOFDAY
   #include <sys/times.h>
 #endif
+#ifdef HAVE_CLOCK_GETTIME
+  #include <time.h>
+#endif
 #ifdef HAVE_GETRUSAGE
   #include <sys/resource.h>
 #endif
 #ifdef __MINGW32__
   #include <process.h>
 #endif
-#include "util.h"
+#include "g10lib.h"
 #include "rmd.h"
-#include "ttyio.h"
-#include "i18n.h"
 #include "random.h"
 #include "rand-internal.h"
 #include "dynload.h"
@@ -105,6 +105,7 @@ static int quick_test;
 static int faked_rng;
 
 
+static byte *get_random_bytes( size_t nbytes, int level, int secure );
 static void read_pool( byte *buffer, size_t length, int level );
 static void add_randomness( const void *buffer, size_t length, int source );
 static void random_poll(void);
@@ -131,24 +132,14 @@ initialize(void)
     /* The data buffer is allocated somewhat larger, so that
      * we can use this extra space (which is allocated in secure memory)
      * as a temporary hash buffer */
-    rndpool = secure_alloc ? m_alloc_secure_clear(POOLSIZE+BLOCKLEN)
-			   : m_alloc_clear(POOLSIZE+BLOCKLEN);
-    keypool = secure_alloc ? m_alloc_secure_clear(POOLSIZE+BLOCKLEN)
-			   : m_alloc_clear(POOLSIZE+BLOCKLEN);
+    rndpool = secure_alloc ? g10_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
+			   : g10_xcalloc(1,POOLSIZE+BLOCKLEN);
+    keypool = secure_alloc ? g10_xcalloc_secure(1,POOLSIZE+BLOCKLEN)
+			   : g10_xcalloc(1,POOLSIZE+BLOCKLEN);
     is_initialized = 1;
     cipher_modules_constructor();
 }
 
-static void
-burn_stack (int bytes)
-{
-    char buf[128];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
 
 void
 random_dump_stats()
@@ -188,11 +179,11 @@ quick_random_gen( int onoff )
  * for most usage, 2 is good for key generation stuff but may be very slow.
  */
 void
-randomize_buffer( byte *buffer, size_t length, int level )
+gcry_randomize( byte *buffer, size_t length, enum gcry_random_level level )
 {
-    char *p = get_random_bits( length*8, level, 1 );
+    char *p = get_random_bytes( length, level, 1 );
     memcpy( buffer, p, length );
-    m_free(p);
+    g10_free(p);
 }
 
 
@@ -209,11 +200,10 @@ random_is_faked()
  * caller must free the buffer.
  * Note: The returned value is rounded up to bytes.
  */
-byte *
-get_random_bits( size_t nbits, int level, int secure )
+static byte *
+get_random_bytes( size_t nbytes, int level, int secure )
 {
     byte *buf, *p;
-    size_t nbytes = (nbits+7)/8;
 
     if( quick_test && level > 1 )
 	level = 1;
@@ -227,7 +217,8 @@ get_random_bits( size_t nbits, int level, int secure )
 	rndstats.ngetbytes2++;
     }
 
-    buf = secure && secure_alloc ? m_alloc_secure( nbytes ) : m_alloc( nbytes );
+    buf = secure && secure_alloc ? g10_xmalloc_secure( nbytes )
+				 : g10_xmalloc( nbytes );
     for( p = buf; nbytes > 0; ) {
 	size_t n = nbytes > POOLSIZE? POOLSIZE : nbytes;
 	read_pool( p, n, level );
@@ -237,6 +228,18 @@ get_random_bits( size_t nbits, int level, int secure )
     return buf;
 }
 
+void *
+gcry_random_bytes( size_t nbytes, enum gcry_random_level level )
+{
+    return get_random_bytes( nbytes, level, 0 );
+}
+
+void *
+gcry_random_bytes_secure( size_t nbytes, enum gcry_random_level level )
+{
+    return get_random_bytes( nbytes, level, 1 );
+}
+
 
 /****************
  * Mix the pool
@@ -279,16 +282,14 @@ mix_pool(byte *pool)
 	rmd160_mixblock( &md, hashbuf);
 	memcpy(p, hashbuf, 20 );
     }
-    burn_stack (200); /* for the rmd160_mixblock() */
 }
 
-
 void
 set_random_seed_file( const char *name )
 {
     if( seed_file_name )
 	BUG();
-    seed_file_name = m_strdup( name );
+    seed_file_name = g10_xstrdup( name );
 }
 
 /****************
@@ -603,13 +604,8 @@ fast_random_poll()
       #endif
     #else
     {	struct rusage buf;
-        /* QNX/Neutrino does return ENOSYS - so we just ignore it and
-         * add whatever is in buf.  In a chroot environment it might not
-         * work at all (i.e. because /proc/ is not accessible), so we better 
-         * ognore all error codes and hope for the best
-         */
-        getrusage( RUSAGE_SELF, &buf );
-        
+	if( getrusage( RUSAGE_SELF, &buf ) )
+	    BUG();
 	add_randomness( &buf, sizeof buf, 1 );
 	memset( &buf, 0, sizeof buf );
     }
@@ -659,18 +655,24 @@ gather_faked( void (*add)(const void*, size_t, int), int requester,
 
     if( !initialized ) {
 	log_info(_("WARNING: using insecure random number generator!!\n"));
+	/* we can't use tty_printf here - do we need this function at
+	  all - does it really make sense or canit be viewed as a potential
+	  security problem ? wk 17.11.99 */
+	#warning Extended warning disabled
+      #if 0
 	tty_printf(_("The random number generator is only a kludge to let\n"
 		   "it run - it is in no way a strong RNG!\n\n"
 		   "DON'T USE ANY DATA GENERATED BY THIS PROGRAM!!\n\n"));
+      #endif
 	initialized=1;
       #ifdef HAVE_RAND
-	srand(make_timestamp()*getpid());
+	srand( time(NULL)*getpid());
       #else
-	srandom(make_timestamp()*getpid());
+	srandom( time(NULL)*getpid());
       #endif
     }
 
-    p = buffer = m_alloc( length );
+    p = buffer = g10_xmalloc( length );
     n = length;
   #ifdef HAVE_RAND
     while( n-- )
@@ -680,7 +682,7 @@ gather_faked( void (*add)(const void*, size_t, int), int requester,
 	*p++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
   #endif
     add_randomness( buffer, length, requester );
-    m_free(buffer);
+    g10_free(buffer);
     return 0; /* okay */
 }
 
