@@ -1,5 +1,5 @@
 /* rmd160.c  -	RIPE-MD160
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include "util.h"
+#include "g10lib.h"
 #include "memory.h"
 #include "rmd.h"
 #include "cipher.h" /* only used for the rmd160_hash_buffer() prototype */
@@ -141,18 +141,6 @@
  * 1 million times "a"   52783243c1697bdbe16d37f97f68f08325dc1528
  */
 
-static void
-burn_stack (int bytes)
-{
-    char buf[150];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
-
 
 void
 rmd160_init( RMD160_CONTEXT *hd )
@@ -417,7 +405,6 @@ rmd160_write( RMD160_CONTEXT *hd, byte *inbuf, size_t inlen)
 {
     if( hd->count == 64 ) { /* flush the buffer */
 	transform( hd, hd->buf );
-        burn_stack (108+5*sizeof(void*));
 	hd->count = 0;
 	hd->nblocks++;
     }
@@ -438,7 +425,6 @@ rmd160_write( RMD160_CONTEXT *hd, byte *inbuf, size_t inlen)
 	inlen -= 64;
 	inbuf += 64;
     }
-    burn_stack (108+5*sizeof(void*));
     for( ; inlen && hd->count < 64; inlen-- )
 	hd->buf[hd->count++] = *inbuf++;
 }
@@ -475,19 +461,18 @@ rmd160_final( RMD160_CONTEXT *hd )
 
     rmd160_write(hd, NULL, 0); /* flush */;
 
+    msb = 0;
     t = hd->nblocks;
-    /* multiply by 64 to make a byte count */
-    lsb = t << 6;
-    msb = t >> 26;
-    /* add the count */
-    t = lsb;
-    if( (lsb += hd->count) < t )
+    if( (lsb = t << 6) < t ) /* multiply by 64 to make a byte count */
 	msb++;
-    /* multiply by 8 to make a bit count */
+    msb += t >> 26;
     t = lsb;
-    lsb <<= 3;
-    msb <<= 3;
-    msb |= t >> 29;
+    if( (lsb = t + hd->count) < t ) /* add the count */
+	msb++;
+    t = lsb;
+    if( (lsb = t << 3) < t ) /* multiply by 8 to make a bit count */
+	msb++;
+    msb += t >> 29;
 
     if( hd->count < 56 ) { /* enough room */
 	hd->buf[hd->count++] = 0x80; /* pad */
@@ -511,7 +496,6 @@ rmd160_final( RMD160_CONTEXT *hd )
     hd->buf[62] = msb >> 16;
     hd->buf[63] = msb >> 24;
     transform( hd, hd->buf );
-    burn_stack (108+5*sizeof(void*));
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
@@ -590,7 +574,7 @@ rmd160_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RMD160 ($Revision: 1.17.2.5 $)";
+const char * const gnupgext_version = "RMD160 ($Revision: 1.19 $)";
 
 static struct {
     int class;
@@ -625,11 +609,7 @@ gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
 	    ret = &func_table[i].value;
 	    break;
 	  default:
-#ifndef __riscos__
 	    ret = func_table[i].func;
-#else /* __riscos__ */
-	    ret = (void *) func_table[i].func;
-#endif /* __riscos__ */
 	    break;
 	}
 	i++;
