@@ -1,5 +1,5 @@
 /* rndegd.c  -	interface to the EGD
- *	Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -31,21 +31,55 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include "types.h"
-#include "util.h"
+#include "g10lib.h"
+#ifndef IS_MODULE
 #include "ttyio.h"
+#endif
 #include "dynload.h"
 #include "cipher.h"
 
-#ifdef IS_MODULE
-  #define _(a) (a)
-#else
-  #include "i18n.h"
-#endif
-
 #ifndef offsetof
 #define offsetof(type, member) ((size_t) &((type *)0)->member)
 #endif
 
+
+/* FIXME: this is duplicated code from util/fileutil
+ * I don't think that this code should go into libgcrypt anyway.
+ */
+char *
+my_make_filename( const char *first_part, ... )
+{
+    va_list arg_ptr ;
+    size_t n;
+    const char *s;
+    char *name, *home, *p;
+
+    va_start( arg_ptr, first_part ) ;
+    n = strlen(first_part)+1;
+    while( (s=va_arg(arg_ptr, const char *)) )
+	n += strlen(s) + 1;
+    va_end(arg_ptr);
+
+    home = NULL;
+    if( *first_part == '~' && first_part[1] == '/'
+			   && (home = getenv("HOME")) && *home )
+	n += strlen(home);
+
+    name = m_alloc(n);
+    p = home ? stpcpy(stpcpy(name,home), first_part+1)
+	     : stpcpy(name, first_part);
+    va_start( arg_ptr, first_part ) ;
+    while( (s=va_arg(arg_ptr, const char *)) )
+	p = stpcpy(stpcpy(p,"/"), s);
+    va_end(arg_ptr);
+
+    return name;
+}
+
+
+
+
+
 static int
 do_write( int fd, void *buf, size_t nbytes )
 {
@@ -114,28 +148,14 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	}
     }
     if( fd == -1 ) {
-        const char *bname = NULL;
-	char *name;
+	#warning Fixme: make the filename configurable
+	char *name = my_make_filename( "~/.gnupg-test", "entropy", NULL );
 	struct sockaddr_un addr;
 	int addr_len;
-      
-      #ifdef EGD_SOCKET_NAME
-        bname = EGD_SOCKET_NAME;
-      #endif
-        if ( !bname || !*bname )
-            bname = "=entropy";
-
-        if ( *bname == '=' && bname[1] )
-            name = make_filename( g10_opt_homedir, bname+1 , NULL );
-        else
-            name = make_filename( bname , NULL );
-
-        if ( strlen(name)+1 >= sizeof addr.sun_path ) 
-            g10_log_fatal ("EGD socketname is too long\n");
 
 	memset( &addr, 0, sizeof addr );
 	addr.sun_family = AF_UNIX;
-	strcpy( addr.sun_path, name );	  
+	strcpy( addr.sun_path, name );	  /* fixme: check that it is long enough */
 	addr_len = offsetof( struct sockaddr_un, sun_path )
 		   + strlen( addr.sun_path );
 
@@ -146,7 +166,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	if( connect( fd, (struct sockaddr*)&addr, addr_len) == -1 )
 	    g10_log_fatal("can't connect to `%s': %s\n",
 						    name, strerror(errno) );
-	m_free(name);
+	g10_free(name);
     }
     do_restart = 0;
 
@@ -210,14 +230,14 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDEGD ($Revision: 1.7.2.5 $)";
+const char * const gnupgext_version = "RNDEGD ($Revision: 1.12 $)";
 
 static struct {
     int class;
     int version;
-    int (*func)(void);
+    void *func;
 } func_table[] = {
-    { 40, 1, (int (*)(void))gather_random },
+    { 40, 1, gather_random },
 };
 
 
