@@ -1,5 +1,5 @@
 /* rndlinux.c  -  raw random number for OSes with /dev/random
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -41,16 +41,9 @@
   #endif
 #endif
 #include "types.h"
-#include "util.h"
-#include "ttyio.h"
+#include "g10lib.h"
 #include "dynload.h"
 
-#ifdef IS_MODULE
-  #define _(a) (a)
-#else
-  #include "i18n.h"
-#endif
-
 static int open_device( const char *name, int minor );
 static int gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level );
@@ -70,7 +63,7 @@ get_entropy_count( int fd )
 #endif
 
 /****************
- * Used to open the /dev/random devices (Linux, xBSD, Solaris (if it exists), ...)
+ * Used to open the Linux and xBSD /dev/random devices
  */
 static int
 open_device( const char *name, int minor )
@@ -83,17 +76,12 @@ open_device( const char *name, int minor )
 	g10_log_fatal("can't open %s: %s\n", name, strerror(errno) );
     if( fstat( fd, &sb ) )
 	g10_log_fatal("stat() off %s failed: %s\n", name, strerror(errno) );
-    /* Don't check device type for better portability */
-    /*  if( (!S_ISCHR(sb.st_mode)) && (!S_ISFIFO(sb.st_mode)) )
-	  g10_log_fatal("invalid random device!\n" ); */
+    if( !S_ISCHR(sb.st_mode) )
+	g10_log_fatal("invalid random device!\n" );
     return fd;
 }
 
 
-/****************
- * Note:  Using a level of 0 should never block and better add nothing
- * to the pool.  This is easy to accomplish with /dev/urandom.
- */
 static int
 gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level )
@@ -111,8 +99,6 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	fd = fd_random;
     }
     else {
-	/* this will also be used for elve 0 but by using /dev/urandom
-	 * we can be sure that oit will never block. */
 	if( fd_urandom == -1 )
 	    fd_urandom = open_device( NAME_OF_DEV_URANDOM, 9 );
 	fd = fd_urandom;
@@ -133,25 +119,17 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	tv.tv_sec = 3;
 	tv.tv_usec = 0;
 	if( !(rc=select(fd+1, &rfds, NULL, NULL, &tv)) ) {
+	    #warning FIXME: Replace fprintf by a callback
 	    if( !warn )
-	      #ifdef IS_MODULE
 		fprintf(stderr,
-	      #else
-		tty_printf(
-	      #endif
 _("\n"
 "Not enough random bytes available.  Please do some other work to give\n"
-"the OS a chance to collect more entropy! (Need %d more bytes)\n"), (int)length );
+"the OS a chance to collect more entropy! (Need %d more bytes)\n"), length );
 	    warn = 1;
 	    continue;
 	}
 	else if( rc == -1 ) {
-	  #ifdef IS_MODULE
-	    fprintf(stderr,
-	  #else
-	    tty_printf(
-	  #endif
-		       "select() error: %s\n", strerror(errno));
+	    fprintf(stderr, "select() error: %s\n", strerror(errno));
 	    continue;
 	}
 
@@ -178,14 +156,14 @@ _("\n"
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.9.2.5 $)";
+const char * const gnupgext_version = "RNDLINUX ($Revision: 1.12 $)";
 
 static struct {
     int class;
     int version;
-    int (*func)(void);
+    void *func;
 } func_table[] = {
-    { 40, 1, (int (*)(void))gather_random },
+    { 40, 1, gather_random },
 };
 
 
