@@ -44,6 +44,8 @@
   code base to be maintained */
 
 
+/* Fixme: We use plain mallocs here beucase it may be used as a module
+ * should be changed. */
 
 /* General includes */
 
@@ -77,10 +79,9 @@
 #endif				/* __QNX__ */
 #if defined( _AIX ) || defined( __QNX__ )
 #include <sys/select.h>
-#endif				/* _AIX || __QNX__ */
+#endif				/* _AIX */
 #ifndef __QNX__
 #include <sys/shm.h>
-#include <signal.h>
 #include <sys/signal.h>
 #endif				/* __QNX__ */
 #include <sys/stat.h>
@@ -100,7 +101,7 @@
 #ifndef IS_MODULE
 #include "dynload.h"
 #endif
-#include "util.h"
+#include "g10lib.h"
 
 #ifndef EAGAIN
 #define EAGAIN	EWOULDBLOCK
@@ -243,7 +244,6 @@ static struct RI {
     {	"/usr/ucb/ps", "aux", SC(0.3), NULL, 0, 0, 0, 1       },
     {	"/usr/bin/ps", "aux", SC(0.3), NULL, 0, 0, 0, 1       },
     {	"/bin/ps", "aux", SC(0.3), NULL, 0, 0, 0, 0          },
-    {   "/bin/ps", "-A", SC(0.3), NULL, 0, 0, 0, 0           }, /*QNX*/
     {	"/usr/bin/ipcs", "-a", SC(0.5), NULL, 0, 0, 0, 1      },
     {	"/bin/ipcs", "-a", SC(0.5), NULL, 0, 0, 0, 0         },
     /* Unreliable source, depends on system usage */
@@ -292,10 +292,6 @@ static struct RI {
     /* This is a complex and screwball program.  Some systems have things
      * like rX_dmn, x = integer, for RAID systems, but the statistics are
      * pretty dodgy */
-#ifdef __QNXNTO__                                                             
-    { "/bin/pidin", "-F%A%B%c%d%E%I%J%K%m%M%n%N%p%P%S%s%T", SC(0.3),
-             NULL, 0, 0, 0, 0       },
-#endif     
 #if 0
     /* The following aren't enabled since they're somewhat slow and not very
      * unpredictable, however they give an indication of the sort of sources
@@ -320,7 +316,6 @@ typedef struct {
     char data[500];	/* gathered data */
 } GATHER_MSG;
 
-
 #ifndef HAVE_WAITPID
 pid_t
 waitpid(pid_t pid, int *statptr, int options)
@@ -347,7 +342,6 @@ waitpid(pid_t pid, int *statptr, int options)
 }
 #endif
 
-
 /* Under SunOS popen() doesn't record the pid of the child process.  When
  * pclose() is called, instead of calling waitpid() for the correct child, it
  * calls wait() repeatedly until the right child is reaped.  The problem is
@@ -513,6 +507,8 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 		maxFD = dataSources[i].pipeFD;
 	  #ifdef O_NONBLOCK /* Ohhh what a hack (used for Atari) */
 	    fcntl(dataSources[i].pipeFD, F_SETFL, O_NONBLOCK);
+	  #else
+	    #warning O_NONBLOCK is missing
 	  #endif
 	    FD_SET(dataSources[i].pipeFD, &fds);
 	    dataSources[i].length = 0;
@@ -851,15 +847,15 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.14.2.6 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.18 $)";
 
 
 static struct {
     int class;
     int version;
-    int (*func)(void);
+    void *func;
 } func_table[] = {
-    { 40, 1, (int (*)(void))gather_random },
+    { 40, 1, gather_random },
 };
 
 /****************
