@@ -1,5 +1,5 @@
 /* tiger.c  -  The TIGER hash function
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include "util.h"
+#include "g10lib.h"
 #include "memory.h"
 
 
@@ -630,19 +630,6 @@ print_data( const char *text, u64 a, u64 b, u64 c,
 }
 
 
-static void
-burn_stack (int bytes)
-{
-    char buf[256];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
-
-
 static void
 tiger_init( TIGER_CONTEXT *hd )
 {
@@ -781,7 +768,6 @@ tiger_write( TIGER_CONTEXT *hd, byte *inbuf, size_t inlen)
 {
     if( hd->count == 64 ) { /* flush the buffer */
 	transform( hd, hd->buf );
-        burn_stack (21*8+11*sizeof(void*));
 	hd->count = 0;
 	hd->nblocks++;
     }
@@ -802,7 +788,6 @@ tiger_write( TIGER_CONTEXT *hd, byte *inbuf, size_t inlen)
 	inlen -= 64;
 	inbuf += 64;
     }
-    burn_stack (21*8+11*sizeof(void*));
     for( ; inlen && hd->count < 64; inlen-- )
 	hd->buf[hd->count++] = *inbuf++;
 }
@@ -820,19 +805,18 @@ tiger_final( TIGER_CONTEXT *hd )
 
     tiger_write(hd, NULL, 0); /* flush */;
 
+    msb = 0;
     t = hd->nblocks;
-    /* multiply by 64 to make a byte count */
-    lsb = t << 6;
-    msb = t >> 26;
-    /* add the count */
-    t = lsb;
-    if( (lsb += hd->count) < t )
+    if( (lsb = t << 6) < t ) /* multiply by 64 to make a byte count */
 	msb++;
-    /* multiply by 8 to make a bit count */
+    msb += t >> 26;
     t = lsb;
-    lsb <<= 3;
-    msb <<= 3;
-    msb |= t >> 29;
+    if( (lsb = t + hd->count) < t ) /* add the count */
+	msb++;
+    t = lsb;
+    if( (lsb = t << 3) < t ) /* multiply by 8 to make a bit count */
+	msb++;
+    msb += t >> 29;
 
     if( hd->count < 56 ) { /* enough room */
 	hd->buf[hd->count++] = 0x01; /* pad */
@@ -856,16 +840,15 @@ tiger_final( TIGER_CONTEXT *hd )
     hd->buf[62] = msb >> 16;
     hd->buf[63] = msb >> 24;
     transform( hd, hd->buf );
-    burn_stack (21*8+11*sizeof(void*));
 
     p = hd->buf;
   #ifdef BIG_ENDIAN_HOST
-    #define X(a) do { *(u64*)p = hd-> a ; p += 8; } while(0)
+    #define X(a) do { *(u64*)p = hd->##a ; p += 8; } while(0)
   #else /* little endian */
-    #define X(a) do { *p++ = hd-> a >> 56; *p++ = hd-> a >> 48; \
-		      *p++ = hd-> a >> 40; *p++ = hd-> a >> 32; \
-		      *p++ = hd-> a >> 24; *p++ = hd-> a >> 16; \
-		      *p++ = hd-> a >>  8; *p++ = hd-> a; } while(0)
+    #define X(a) do { *p++ = hd->##a >> 56; *p++ = hd->##a >> 48; \
+		      *p++ = hd->##a >> 40; *p++ = hd->##a >> 32; \
+		      *p++ = hd->##a >> 24; *p++ = hd->##a >> 16; \
+		      *p++ = hd->##a >>  8; *p++ = hd->##a; } while(0)
   #endif
     X(a);
     X(b);
@@ -921,7 +904,7 @@ tiger_get_info( int algo, size_t *contextsize,
     *(void  (**)(TIGER_CONTEXT *))r_final		 = tiger_final;
     *(byte *(**)(TIGER_CONTEXT *))r_read		 = tiger_read;
 
-    return "TIGER192";
+    return "TIGER";
 }
 
 
@@ -929,7 +912,7 @@ tiger_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "TIGER ($Revision: 1.10.2.6 $)";
+const char * const gnupgext_version = "TIGER ($Revision: 1.12 $)";
 
 static struct {
     int class;
