@@ -27,8 +27,7 @@
 #include <string.h> /* for memcmp() */
 
 #include "types.h"  /* for byte and u32 typedefs */
-#include "util.h"
-#include "errors.h"
+#include "g10lib.h"
 #include "dynload.h"
 
 
@@ -545,25 +544,11 @@ static byte calc_sb_tbl[512] = {
    x += y; y += x; ctx->a[j] = x; \
    ctx->a[(j) + 1] = (y << 9) + (y >> 23)
 
-
-static void
-burn_stack (int bytes)
-{
-    char buf[64];
-    
-    memset (buf, 0, sizeof buf);
-    bytes -= sizeof buf;
-    if (bytes > 0)
-        burn_stack (bytes);
-}
-
-
-
 /* Perform the key setup.  Note that this works only with 128- and 256-bit
  * keys, despite the API that looks like it might support other sizes. */
 
 static int
-do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
+twofish_setkey (TWOFISH_context *ctx, const byte *key, const unsigned keylen)
 {
     int i, j, k;
 
@@ -584,7 +569,7 @@ do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 
     /* Check key length. */
     if( ( ( keylen - 16 ) | 16 ) != 16 )
-	return G10ERR_WRONG_KEYLEN;
+	return GCRYERR_INV_KEYLEN;
 
     /* Do self-test if necessary. */
     if (!initialized) {
@@ -594,7 +579,7 @@ do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 	 fprintf(stderr, "%s\n", selftest_failed );
     }
     if( selftest_failed )
-       return G10ERR_SELFTEST_FAILED;
+       return GCRYERR_SELFTEST;
 
     /* Compute the first two words of the S vector.  The magic numbers are
      * the entries of the RS matrix, preprocessed through poly_to_exp.	The
@@ -696,16 +681,6 @@ do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 
     return 0;
 }
-
-static int
-twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
-{
-    int rc = do_twofish_setkey (ctx, key, keylen);
-    burn_stack (23+6*sizeof(void*));
-    return rc;
-}
-        
-
 
 /* Macros to compute the g() function in the encryption and decryption
  * rounds.  G1 is the straight g() function; G2 includes the 8-bit
@@ -768,7 +743,7 @@ twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 /* Encrypt one block.  in and out may be the same. */
 
 static void
-do_twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
+twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
 {
    /* The four 32-bit chunks of the text. */
    u32 a, b, c, d;
@@ -798,18 +773,11 @@ do_twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
    OUTUNPACK (2, a, 6);
    OUTUNPACK (3, b, 7);
 }
-
-static void
-twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
-{
-    do_twofish_encrypt (ctx, out, in);
-    burn_stack (24+3*sizeof (void*));
-}
 
 /* Decrypt one block.  in and out may be the same. */
 
 static void
-do_twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
+twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
 {
    /* The four 32-bit chunks of the text. */
    u32 a, b, c, d;
@@ -839,13 +807,6 @@ do_twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
    OUTUNPACK (2, c, 2);
    OUTUNPACK (3, d, 3);
 }
-
-static void
-twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
-{
-    do_twofish_decrypt (ctx, out, in);
-    burn_stack (24+3*sizeof (void*));
-}
 
 /* Test a single encryption and decryption with each key size. */
 
@@ -1042,9 +1003,7 @@ twofish_get_info (int algo, size_t *keylen,
 }
 
 
-#ifdef IS_MODULE
-static
-const char * const gnupgext_version = "TWOFISH ($Revision: 1.8.2.5 $)";
+const char * const gnupgext_version = "TWOFISH ($Revision: 1.11 $)";
 
 static struct {
     int class;
@@ -1074,7 +1033,7 @@ static struct {
  *		  version = interface version of the function/pointer
  *			    (currently this is 1 for all functions)
  */
-static void *
+void *
 gnupgext_enum_func ( int what, int *sequence, int *class, int *vers )
 {
     void *ret;
@@ -1102,4 +1061,4 @@ gnupgext_enum_func ( int what, int *sequence, int *class, int *vers )
     *sequence = i;
     return ret;
 }
-#endif
+
