@@ -1,5 +1,5 @@
 /* build-packet.c - assemble packets and write them
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,14 +24,13 @@
 #include <string.h>
 #include <assert.h>
 
+#include <gcrypt.h>
 #include "packet.h"
 #include "errors.h"
 #include "iobuf.h"
-#include "mpi.h"
 #include "util.h"
-#include "cipher.h"
-#include "memory.h"
 #include "options.h"
+#include "main.h"
 
 
 static int do_comment( IOBUF out, int ctb, PKT_comment *rem );
@@ -44,6 +43,7 @@ static u32 calc_plaintext( PKT_plaintext *pt );
 static int do_plaintext( IOBUF out, int ctb, PKT_plaintext *pt );
 static int do_encrypted( IOBUF out, int ctb, PKT_encrypted *ed );
 static int do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed );
+static int do_mdc( IOBUF out, PKT_mdc *mdc );
 static int do_compressed( IOBUF out, int ctb, PKT_compressed *cd );
 static int do_signature( IOBUF out, int ctb, PKT_signature *sig );
 static int do_onepass_sig( IOBUF out, int ctb, PKT_onepass_sig *ops );
@@ -67,25 +67,31 @@ int
 build_packet( IOBUF out, PACKET *pkt )
 {
     int new_ctb=0, rc=0, ctb;
+    int pkttype;
 
     if( DBG_PACKET )
 	log_debug("build_packet() type=%d\n", pkt->pkttype );
     assert( pkt->pkt.generic );
 
-    switch( pkt->pkttype ) {
-      case PKT_OLD_COMMENT: pkt->pkttype = PKT_COMMENT; break;
+    switch( (pkttype = pkt->pkttype) ) {
+      case PKT_OLD_COMMENT: pkttype = pkt->pkttype = PKT_COMMENT; break;
       case PKT_PLAINTEXT: new_ctb = pkt->pkt.plaintext->new_ctb; break;
       case PKT_ENCRYPTED:
       case PKT_ENCRYPTED_MDC: new_ctb = pkt->pkt.encrypted->new_ctb; break;
       case PKT_COMPRESSED:new_ctb = pkt->pkt.compressed->new_ctb; break;
+      case PKT_USER_ID:
+	    if( pkt->pkt.user_id->photo )
+		pkttype = PKT_PHOTO_ID;
+	    break;
       default: break;
     }
 
-    if( new_ctb || pkt->pkttype > 15 ) /* new format */
-	ctb = 0xc0 | (pkt->pkttype & 0x3f);
+    if( new_ctb || pkttype > 15 ) /* new format */
+	ctb = 0xc0 | (pkttype & 0x3f);
     else
-	ctb = 0x80 | ((pkt->pkttype & 15)<<2);
-    switch( pkt->pkttype ) {
+	ctb = 0x80 | ((pkttype & 15)<<2);
+    switch( pkttype ) {
+      case PKT_PHOTO_ID:
       case PKT_USER_ID:
 	rc = do_user_id( out, ctb, pkt->pkt.user_id );
 	break;
@@ -115,6 +121,9 @@ build_packet( IOBUF out, PACKET *pkt )
       case PKT_ENCRYPTED_MDC:
 	rc = do_encrypted_mdc( out, ctb, pkt->pkt.encrypted );
 	break;
+      case PKT_MDC:
+	rc = do_mdc( out, pkt->pkt.mdc );
+	break;
       case PKT_COMPRESSED:
 	rc = do_compressed( out, ctb, pkt->pkt.compressed );
 	break;
@@ -149,6 +158,7 @@ calc_packet_length( PACKET *pkt )
 	n = calc_plaintext( pkt->pkt.plaintext );
 	new_ctb = pkt->pkt.plaintext->new_ctb;
 	break;
+      case PKT_PHOTO_ID:
       case PKT_USER_ID:
       case PKT_COMMENT:
       case PKT_PUBLIC_KEY:
@@ -188,7 +198,7 @@ do_comment( IOBUF out, int ctb, PKT_comment *rem )
     if( !opt.no_comment ) {
 	write_header(out, ctb, rem->len);
 	if( iobuf_write( out, rem->data, rem->len ) )
-	    return G10ERR_WRITE_FILE;
+	    return GPGERR_WRITE_FILE;
     }
     return 0;
 }
@@ -196,9 +206,20 @@ do_comment( IOBUF out, int ctb, PKT_comment *rem )
 static int
 do_user_id( IOBUF out, int ctb, PKT_user_id *uid )
 {
-    write_header(out, ctb, uid->len);
-    if( iobuf_write( out, uid->name, uid->len ) )
-	return G10ERR_WRITE_FILE;
+    if( uid->photo ) {
+	write_header(out, ctb, uid->photolen);
+	uid->stored_at = iobuf_get_temp_length ( out ); /* what a hack ... */
+		  /* ... and it does only work when used with a temp iobuf */
+	if( iobuf_write( out, uid->photo, uid->photolen ) )
+	    return GPGERR_WRITE_FILE;
+    }
+    else {
+	write_header(out, ctb, uid->len);
+	uid->stored_at = iobuf_get_temp_length ( out ); /* what a hack ... */
+		  /* ... and it does only work when used with a temp iobuf */
+	if( iobuf_write( out, uid->name, uid->len ) )
+	    return GPGERR_WRITE_FILE;
+    }
     return 0;
 }
 
@@ -231,7 +252,7 @@ do_public_key( IOBUF out, int ctb, PKT_public_key *pk )
 
     write_header2(out, ctb, iobuf_get_temp_length(a), pk->hdrbytes, 1 );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
@@ -262,7 +283,7 @@ hash_public_key( GCRY_MD_HD md, PKT_public_key *pk )
     pkt.pkttype = PKT_PUBLIC_KEY;
     pkt.pkt.public_key = pk;
     if( (rc = build_packet( a, &pkt )) )
-	log_fatal("build public_key for hashing failed: %s\n", g10_errstr(rc));
+	log_fatal("build public_key for hashing failed: %s\n", gpg_errstr(rc));
 
     if( !(pk->version == 3 && pk->pubkey_algo == 16) ) {
 	/* skip the constructed header but don't do this for our very old
@@ -357,23 +378,35 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
 	else {
 	    iobuf_put(a, 0xff );
 	    iobuf_put(a, sk->protect.algo );
-	    iobuf_put(a, sk->protect.s2k.mode );
-	    iobuf_put(a, sk->protect.s2k.hash_algo );
+	    if( sk->protect.s2k.mode >= 1000 ) {
+		iobuf_put(a, 101 );
+		iobuf_put(a, sk->protect.s2k.hash_algo );
+		iobuf_write(a, "GNU", 3 );
+		iobuf_put(a, sk->protect.s2k.mode - 1000 );
+	    }
+	    else {
+		iobuf_put(a, sk->protect.s2k.mode );
+		iobuf_put(a, sk->protect.s2k.hash_algo );
+	    }
 	    if( sk->protect.s2k.mode == 1
 		|| sk->protect.s2k.mode == 3 )
 		iobuf_write(a, sk->protect.s2k.salt, 8 );
 	    if( sk->protect.s2k.mode == 3 )
 		iobuf_put(a, sk->protect.s2k.count );
-	    iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
+	    if( sk->protect.s2k.mode != 1001 )
+		iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
 	}
     }
     else
 	iobuf_put(a, 0 );
-    if( sk->is_protected && sk->version >= 4 ) {
+    if( sk->protect.s2k.mode == 1001 )
+	;
+    else if( sk->is_protected && sk->version >= 4 ) {
 	byte *p;
-	assert( mpi_is_opaque( sk->skey[npkey] ) );
-	p = mpi_get_opaque( sk->skey[npkey], &i );
-	iobuf_write(a, p, i );
+	size_t n;
+	assert( gcry_mpi_get_flag( sk->skey[i], GCRYMPI_FLAG_OPAQUE ) );
+	p = gcry_mpi_get_opaque( sk->skey[i], &n );
+	iobuf_write(a, p, (n+7)/8 );
     }
     else {
 	for(   ; i < nskey; i++ )
@@ -384,7 +417,7 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
   leave:
     write_header2(out, ctb, iobuf_get_temp_length(a), sk->hdrbytes, 1 );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
@@ -415,7 +448,7 @@ do_symkey_enc( IOBUF out, int ctb, PKT_symkey_enc *enc )
 
     write_header(out, ctb, iobuf_get_temp_length(a) );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
@@ -449,7 +482,7 @@ do_pubkey_enc( IOBUF out, int ctb, PKT_pubkey_enc *enc )
 
     write_header(out, ctb, iobuf_get_temp_length(a) );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
@@ -478,12 +511,12 @@ do_plaintext( IOBUF out, int ctb, PKT_plaintext *pt )
     for(i=0; i < pt->namelen; i++ )
 	iobuf_put(out, pt->name[i] );
     if( write_32(out, pt->timestamp ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     n = 0;
     while( (nbytes=iobuf_read(pt->buf, buf, 1000)) != -1 ) {
 	if( iobuf_write(out, buf, nbytes) == -1 ) {
-	    rc = G10ERR_WRITE_FILE;
+	    rc = GPGERR_WRITE_FILE;
 	    break;
 	}
 	n += nbytes;
@@ -525,13 +558,24 @@ do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed )
     n = ed->len ? (ed->len + 10) : 0;
     write_header(out, ctb, n );
     iobuf_put(out, 1 );  /* version */
-    iobuf_put(out, ed->mdc_method );
 
     /* This is all. The caller has to write the real data */
 
     return rc;
 }
 
+
+static int
+do_mdc( IOBUF out, PKT_mdc *mdc )
+{
+    /* This packet requires a fixed header encoding */
+    iobuf_put( out, 0xd3 ); /* packet ID and 1 byte length */
+    iobuf_put( out, 0x14 ); /* length = 20 */
+    if( iobuf_write( out, mdc->hash, sizeof(mdc->hash) ) )
+	return GPGERR_WRITE_FILE;
+    return 0;
+}
+
 static int
 do_compressed( IOBUF out, int ctb, PKT_compressed *cd )
 {
@@ -621,7 +665,6 @@ void
 build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 		  const byte *buffer, size_t buflen )
 {
-
     byte *data;
     size_t hlen, dlen, nlen;
     int found=0;
@@ -657,6 +700,7 @@ build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
       case SIGSUBPKT_KEY_EXPIRE:
       case SIGSUBPKT_NOTATION:
       case SIGSUBPKT_POLICY:
+      case SIGSUBPKT_REVOC_REASON:
 	       hashed = 1; break;
       default: hashed = 0; break;
     }
@@ -666,16 +710,16 @@ build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 				    | sig->hashed_data[1]) : 0;
 	n = n0 + nlen + 1 + buflen; /* length, type, buffer */
 	realloced = !!sig->hashed_data;
-	data = sig->hashed_data ? m_realloc( sig->hashed_data, n+2 )
-				: m_alloc( n+2 );
+	data = sig->hashed_data ? gcry_xrealloc( sig->hashed_data, n+2 )
+				: gcry_xmalloc( n+2 );
     }
     else {
 	n0 = sig->unhashed_data ? ((*sig->unhashed_data << 8)
 				      | sig->unhashed_data[1]) : 0;
 	n = n0 + nlen + 1 + buflen; /* length, type, buffer */
 	realloced = !!sig->unhashed_data;
-	data = sig->unhashed_data ? m_realloc( sig->unhashed_data, n+2 )
-				  : m_alloc( n+2 );
+	data = sig->unhashed_data ? gcry_xrealloc( sig->unhashed_data, n+2 )
+				  : gcry_xmalloc( n+2 );
     }
 
     if( critical )
@@ -706,12 +750,12 @@ build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 
     if( hashed ) {
 	if( !realloced )
-	    m_free(sig->hashed_data);
+	    gcry_free(sig->hashed_data);
 	sig->hashed_data = data;
     }
     else {
 	if( !realloced )
-	    m_free(sig->unhashed_data);
+	    gcry_free(sig->unhashed_data);
 	sig->unhashed_data = data;
     }
 }
@@ -796,7 +840,7 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
     else
 	write_header(out, ctb, iobuf_get_temp_length(a) );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
@@ -819,7 +863,7 @@ do_onepass_sig( IOBUF out, int ctb, PKT_onepass_sig *ops )
 
     write_header(out, ctb, iobuf_get_temp_length(a) );
     if( iobuf_write_temp( out, a ) )
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 
     iobuf_close(a);
     return rc;
