@@ -1,5 +1,5 @@
 /* encode.c - encode data
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -30,7 +30,7 @@
 #include "errors.h"
 #include "iobuf.h"
 #include "keydb.h"
-#include "memory.h"
+#include <gcrypt.h>
 #include "util.h"
 #include "main.h"
 #include "filter.h"
@@ -42,6 +42,57 @@ static int encode_simple( const char *filename, int mode );
 static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out );
 
 
+/****************
+ * Emulate our old PK interface here - sometime in the future we might
+ * change the internal design to directly fit to libgcrypt.
+ */
+static int
+pk_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey )
+{
+    GCRY_SEXP s_ciph, s_data, s_pkey;
+    int rc;
+
+    /* make a sexp from pkey */
+    if( algo == GCRY_PK_ELG || algo == GCRY_PK_ELG_E ) {
+	rc = gcry_sexp_build ( &s_pkey, NULL,
+			      "(public-key(elg(p%m)(g%m)(y%m)))",
+				  pkey[0], pkey[1], pkey[2] );
+    }
+    else
+	return GPGERR_PUBKEY_ALGO;
+
+    if ( rc )
+	BUG ();
+
+    /* put the data into a simple list */
+    if ( gcry_sexp_build( &s_data, NULL, "%m", data ) )
+	BUG ();
+
+    /* pass it to libgcrypt */
+    rc = gcry_pk_encrypt( &s_ciph, s_data, s_pkey );
+    gcry_sexp_release( s_data );
+    gcry_sexp_release( s_pkey );
+
+    if( rc )
+	;
+    else { /* add better error handling or make gnupg use S-Exp directly */
+	GCRY_SEXP list = gcry_sexp_find_token( s_ciph, "a" , 0 );
+	assert( list );
+	resarr[0] = gcry_sexp_nth_mpi( list, 1, 0 );
+	assert( resarr[0] );
+	gcry_sexp_release ( list );
+
+	list = gcry_sexp_find_token( s_ciph, "b" , 0 );
+	assert( list );
+	resarr[1] = gcry_sexp_nth_mpi( list, 1, 0 );
+	assert( resarr[1] );
+	gcry_sexp_release ( list );
+    }
+
+    gcry_sexp_release( s_ciph );
+    return rc;
+}
+
 
 /****************
  * Encode FILENAME with only the symmetric cipher.  Take input from
@@ -90,7 +141,7 @@ encode_simple( const char *filename, int mode )
     if( !(inp = iobuf_open(filename)) ) {
 	log_error(_("%s: can't open: %s\n"), filename? filename: "[stdin]",
 					strerror(errno) );
-	return G10ERR_OPEN_FILE;
+	return GPGERR_OPEN_FILE;
     }
 
     if( opt.textmode )
@@ -98,7 +149,7 @@ encode_simple( const char *filename, int mode )
 
     cfx.dek = NULL;
     if( mode ) {
-	s2k = m_alloc_clear( sizeof *s2k );
+	s2k = gcry_xcalloc( 1, sizeof *s2k );
 	s2k->mode = opt.rfc1991? 0:opt.s2k_mode;
 	s2k->hash_algo = opt.def_digest_algo ? opt.def_digest_algo
 					     : opt.s2k_digest_algo;
@@ -106,19 +157,19 @@ encode_simple( const char *filename, int mode )
 		       opt.def_cipher_algo ? opt.def_cipher_algo
 					   : opt.s2k_cipher_algo , s2k, 2 );
 	if( !cfx.dek || !cfx.dek->keylen ) {
-	    rc = G10ERR_PASSPHRASE;
-	    m_free(cfx.dek);
-	    m_free(s2k);
+	    rc = GPGERR_PASSPHRASE;
+	    gcry_free(cfx.dek);
+	    gcry_free(s2k);
 	    iobuf_close(inp);
-	    log_error(_("error creating passphrase: %s\n"), g10_errstr(rc) );
+	    log_error(_("error creating passphrase: %s\n"), gpg_errstr(rc) );
 	    return rc;
 	}
     }
 
     if( (rc = open_outfile( filename, opt.armor? 1:0, &out )) ) {
-	iobuf_close(inp);
-	m_free(cfx.dek);
-	m_free(s2k);
+	iobuf_cancel(inp);
+	gcry_free(cfx.dek);
+	gcry_free(s2k);
 	return rc;
     }
 
@@ -133,15 +184,15 @@ encode_simple( const char *filename, int mode )
     }
   #endif
     if( s2k && !opt.rfc1991 ) {
-	PKT_symkey_enc *enc = m_alloc_clear( sizeof *enc );
+	PKT_symkey_enc *enc = gcry_xcalloc( 1, sizeof *enc );
 	enc->version = 4;
 	enc->cipher_algo = cfx.dek->algo;
 	enc->s2k = *s2k;
 	pkt.pkttype = PKT_SYMKEY_ENC;
 	pkt.pkt.symkey_enc = enc;
 	if( (rc = build_packet( out, &pkt )) )
-	    log_error("build symkey packet failed: %s\n", g10_errstr(rc) );
-	m_free(enc);
+	    log_error("build symkey packet failed: %s\n", gpg_errstr(rc) );
+	gcry_free(enc);
     }
 
     if (!opt.no_literal) {
@@ -149,13 +200,13 @@ encode_simple( const char *filename, int mode )
 	if( filename || opt.set_filename ) {
 	    char *s = make_basename( opt.set_filename ? opt.set_filename
 						      : filename );
-	    pt = m_alloc( sizeof *pt + strlen(s) - 1 );
+	    pt = gcry_xmalloc( sizeof *pt + strlen(s) - 1 );
 	    pt->namelen = strlen(s);
 	    memcpy(pt->name, s, pt->namelen );
-	    m_free(s);
+	    gcry_free(s);
 	}
 	else { /* no filename */
-	    pt = m_alloc( sizeof *pt - 1 );
+	    pt = gcry_xmalloc( sizeof *pt - 1 );
 	    pt->namelen = 0;
 	}
     }
@@ -196,7 +247,7 @@ encode_simple( const char *filename, int mode )
     /* do the work */
     if (!opt.no_literal) {
 	if( (rc = build_packet( out, &pkt )) )
-	    log_error("build_packet failed: %s\n", g10_errstr(rc) );
+	    log_error("build_packet failed: %s\n", gpg_errstr(rc) );
     }
     else {
 	/* user requested not to create a literal packet,
@@ -205,8 +256,8 @@ encode_simple( const char *filename, int mode )
 	int  bytes_copied;
 	while ((bytes_copied = iobuf_read(inp, copy_buffer, 4096)) != -1)
 	    if (iobuf_write(out, copy_buffer, bytes_copied) == -1) {
-		rc = G10ERR_WRITE_FILE;
-		log_error("copying input to output failed: %s\n", g10_errstr(rc) );
+		rc = GPGERR_WRITE_FILE;
+		log_error("copying input to output failed: %s\n", gpg_errstr(rc) );
 		break;
 	    }
 	memset(copy_buffer, 0, 4096); /* burn buffer */
@@ -221,8 +272,8 @@ encode_simple( const char *filename, int mode )
     if (pt)
 	pt->buf = NULL;
     free_packet(&pkt);
-    m_free(cfx.dek);
-    m_free(s2k);
+    gcry_free(cfx.dek);
+    gcry_free(s2k);
     return rc;
 }
 
@@ -252,14 +303,14 @@ encode_crypt( const char *filename, STRLIST remusr )
     memset( &tfx, 0, sizeof tfx);
     init_packet(&pkt);
 
-    if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC)) )
+    if( (rc=build_pk_list( remusr, &pk_list, GCRY_PK_USAGE_ENCR)) )
 	return rc;
 
     /* prepare iobufs */
     if( !(inp = iobuf_open(filename)) ) {
 	log_error(_("can't open %s: %s\n"), filename? filename: "[stdin]",
 					strerror(errno) );
-	rc = G10ERR_OPEN_FILE;
+	rc = GPGERR_OPEN_FILE;
 	goto leave;
     }
     else if( opt.verbose )
@@ -283,7 +334,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     }
   #endif
     /* create a session key */
-    cfx.dek = m_alloc_secure( sizeof *cfx.dek );
+    cfx.dek = gcry_xmalloc_secure( sizeof *cfx.dek );
     if( !opt.def_cipher_algo ) { /* try to get it from the prefs */
 	cfx.dek->algo = select_algo_from_prefs( pk_list, PREFTYPE_SYM );
 	if( cfx.dek->algo == -1 )
@@ -303,13 +354,13 @@ encode_crypt( const char *filename, STRLIST remusr )
 	/* setup the inner packet */
 	if( filename || opt.set_filename ) {
 	    char *s = make_basename( opt.set_filename ? opt.set_filename : filename );
-	    pt = m_alloc( sizeof *pt + strlen(s) - 1 );
+	    pt = gcry_xmalloc( sizeof *pt + strlen(s) - 1 );
 	    pt->namelen = strlen(s);
 	    memcpy(pt->name, s, pt->namelen );
-	    m_free(s);
+	    gcry_free(s);
 	}
 	else { /* no filename */
-	    pt = m_alloc( sizeof *pt - 1 );
+	    pt = gcry_xmalloc( sizeof *pt - 1 );
 	    pt->namelen = 0;
 	}
     }
@@ -352,7 +403,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     /* do the work */
     if (!opt.no_literal) {
 	if( (rc = build_packet( out, &pkt )) )
-	    log_error("build_packet failed: %s\n", g10_errstr(rc) );
+	    log_error("build_packet failed: %s\n", gpg_errstr(rc) );
     }
     else {
 	/* user requested not to create a literal packet, so we copy the plain data */
@@ -360,8 +411,8 @@ encode_crypt( const char *filename, STRLIST remusr )
 	int  bytes_copied;
 	while ((bytes_copied = iobuf_read(inp, copy_buffer, 4096)) != -1)
 	    if (iobuf_write(out, copy_buffer, bytes_copied) == -1) {
-		rc = G10ERR_WRITE_FILE;
-		log_error("copying input to output failed: %s\n", g10_errstr(rc) );
+		rc = GPGERR_WRITE_FILE;
+		log_error("copying input to output failed: %s\n", gpg_errstr(rc) );
 		break;
 	    }
 	memset(copy_buffer, 0, 4096); /* burn buffer */
@@ -377,7 +428,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     if( pt )
 	pt->buf = NULL;
     free_packet(&pkt);
-    m_free(cfx.dek);
+    gcry_free(cfx.dek);
     release_pk_list( pk_list );
     return rc;
 }
@@ -401,7 +452,7 @@ encrypt_filter( void *opaque, int control,
     }
     else if( control == IOBUFCTRL_FLUSH ) { /* encrypt */
 	if( !efx->header_okay ) {
-	    efx->cfx.dek = m_alloc_secure( sizeof *efx->cfx.dek );
+	    efx->cfx.dek = gcry_xmalloc_secure( sizeof *efx->cfx.dek );
 
 	    if( !opt.def_cipher_algo  ) { /* try to get it from the prefs */
 		efx->cfx.dek->algo =
@@ -453,7 +504,7 @@ write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out )
 	pk = pk_list->pk;
 
 	print_pubkey_algo_note( pk->pubkey_algo );
-	enc = m_alloc_clear( sizeof *enc );
+	enc = gcry_xcalloc( 1, sizeof *enc );
 	enc->pubkey_algo = pk->pubkey_algo;
 	keyid_from_pk( pk, enc->keyid );
 	enc->throw_keyid = opt.throw_keyid;
@@ -473,17 +524,17 @@ write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out )
 	 */
 	frame = encode_session_key( dek, pubkey_nbits( pk->pubkey_algo,
 							  pk->pkey ) );
-	rc = pubkey_encrypt( pk->pubkey_algo, enc->data, frame, pk->pkey );
-	mpi_free( frame );
+	rc = pk_encrypt( pk->pubkey_algo, enc->data, frame, pk->pkey );
+	mpi_release( frame );
 	if( rc )
-	    log_error("pubkey_encrypt failed: %s\n", g10_errstr(rc) );
+	    log_error("pubkey_encrypt failed: %s\n", gpg_errstr(rc) );
 	else {
 	    if( opt.verbose ) {
-		char *ustr = get_user_id_string( enc->keyid );
+		char *ustr = get_user_id_string_native( enc->keyid );
 		log_info(_("%s/%s encrypted for: %s\n"),
 		    gcry_pk_algo_name(enc->pubkey_algo),
 		    gcry_cipher_algo_name(dek->algo), ustr );
-		m_free(ustr);
+		gcry_free(ustr);
 	    }
 	    /* and write it */
 	    init_packet(&pkt);
@@ -491,7 +542,7 @@ write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out )
 	    pkt.pkt.pubkey_enc = enc;
 	    rc = build_packet( out, &pkt );
 	    if( rc )
-	       log_error("build_packet(pubkey_enc) failed: %s\n", g10_errstr(rc));
+	       log_error("build_packet(pubkey_enc) failed: %s\n", gpg_errstr(rc));
 	}
 	free_pubkey_enc(enc);
 	if( rc )
