@@ -1,5 +1,5 @@
 /* export.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,11 +25,11 @@
 #include <errno.h>
 #include <assert.h>
 
+#include <gcrypt.h>
 #include "options.h"
 #include "packet.h"
 #include "errors.h"
 #include "keydb.h"
-#include "memory.h"
 #include "util.h"
 #include "main.h"
 #include "i18n.h"
@@ -83,10 +83,8 @@ do_export( STRLIST users, int secret, int onlyrfc )
     IOBUF out = NULL;
     int any, rc;
     armor_filter_context_t afx;
-    compress_filter_context_t zfx;
 
     memset( &afx, 0, sizeof afx);
-    memset( &zfx, 0, sizeof zfx);
 
     rc = open_outfile( NULL, 0, &out );
     if( rc )
@@ -96,8 +94,6 @@ do_export( STRLIST users, int secret, int onlyrfc )
 	afx.what = secret?5:1;
 	iobuf_push_filter( out, armor_filter, &afx );
     }
-    if( opt.compress_keys && opt.compress )
-	iobuf_push_filter( out, compress_filter, &zfx );
     rc = do_export_stream( out, users, secret, onlyrfc, &any );
 
     if( rc || !any )
@@ -112,55 +108,61 @@ static int
 do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 {
     int rc = 0;
+    compress_filter_context_t zfx;
     PACKET pkt;
     KBNODE keyblock = NULL;
     KBNODE kbctx, node;
-    int ndesc;
-    KEYDB_SEARCH_DESC *desc = NULL;
-    KEYDB_HANDLE kdbhd;
+    KBPOS kbpos;
     STRLIST sl;
+    int all = !users;
 
     *any = 0;
+    memset( &zfx, 0, sizeof zfx);
     init_packet( &pkt );
-    kdbhd = keydb_new (secret);
 
-    if (!users) {
-        ndesc = 1;
-        desc = m_alloc_clear ( ndesc * sizeof *desc);
-        desc[0].mode = KEYDB_SEARCH_MODE_FIRST;
-    }
-    else {
-        for (ndesc=0, sl=users; sl; sl = sl->next, ndesc++) 
-            ;
-        desc = m_alloc ( ndesc * sizeof *desc);
-        
-        for (ndesc=0, sl=users; sl; sl = sl->next) {
-	    if (classify_user_id (sl->d, desc+ndesc))
-                ndesc++;
-            else
-                log_error (_("key `%s' not found: %s\n"),
-                           sl->d, g10_errstr (G10ERR_INV_USER_ID));
-        }
+    if( opt.compress_keys && opt.compress )
+	iobuf_push_filter( out, compress_filter, &zfx );
 
-        /* it would be nice to see which of the given users did
-           actually match one in the keyring.  To implement this we
-           need to have a found flag for each entry in desc and to set
-           this we must check all those entries after a match to mark
-           all matched one - currently we stop at the first match.  To
-           do this we need an extra flag to enable this feature so */
-    }
-
-
-    while (!(rc = keydb_search (kdbhd, desc, ndesc))) {
-	if (!users) 
-            desc[0].mode = KEYDB_SEARCH_MODE_NEXT;
-
-        /* read the keyblock */
-        rc = keydb_get_keyblock (kdbhd, &keyblock );
+    if( all ) {
+	rc = enum_keyblocks( secret?5:0, &kbpos, &keyblock );
 	if( rc ) {
-            log_error (_("error reading keyblock: %s\n"), g10_errstr(rc) );
+	    if( rc != -1 )
+		log_error("enum_keyblocks(open) failed: %s\n", gpg_errstr(rc) );
 	    goto leave;
 	}
+	all = 2;
+    }
+
+    /* use the correct sequence. strlist_last,prev do work correctly with
+     * NULL pointers :-) */
+    for( sl=strlist_last(users); sl || all ; sl=strlist_prev( users, sl )) {
+	if( all ) { /* get the next user */
+	    rc = enum_keyblocks( 1, &kbpos, &keyblock );
+	    if( rc == -1 )  /* EOF */
+		break;
+	    if( rc ) {
+		log_error("enum_keyblocks(read) failed: %s\n", gpg_errstr(rc));
+		break;
+	    }
+	}
+	else {
+	    /* search the userid */
+	    rc = secret? find_secret_keyblock_byname( &kbpos, sl->d )
+		       : find_keyblock_byname( &kbpos, sl->d );
+	    if( rc ) {
+		log_error(_("%s: user not found: %s\n"), sl->d, gpg_errstr(rc));
+		rc = 0;
+		continue;
+	    }
+	    /* read the keyblock */
+	    rc = read_keyblock( &kbpos, &keyblock );
+	}
+
+	if( rc ) {
+	    log_error(_("certificate read problem: %s\n"), gpg_errstr(rc));
+	    goto leave;
+	}
+
 
 	/* do not export keys which are incompatible with rfc2440 */
 	if( onlyrfc && (node = find_kbnode( keyblock, PKT_PUBLIC_KEY )) ) {
@@ -182,44 +184,19 @@ do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 	    continue;
 	}
 
-	/* no v3 keys with GNU mode 1001 */
-	if( secret == 2 && node->pkt->pkt.secret_key->version == 3 )
-	{
-	    log_info(_("key %08lX: PGP 2.x style key - skipped\n"),
-		  (ulong)keyid_from_sk( node->pkt->pkt.secret_key, NULL) );
-	    continue;
-	}
-
 	/* and write it */
 	for( kbctx=NULL; (node = walk_kbnode( keyblock, &kbctx, 0 )); ) {
 	    /* don't export any comment packets but those in the
 	     * secret keyring */
 	    if( !secret && node->pkt->pkttype == PKT_COMMENT )
 		continue;
-            /* make sure that ring_trust packets never get exported */
-            if (node->pkt->pkttype == PKT_RING_TRUST)
-              continue;
-
+	    /* do not export packets which are marked as not exportable */
 	    if( node->pkt->pkttype == PKT_SIGNATURE ) {
-	      /* do not export packets which are marked as not exportable */
-	      if( !node->pkt->pkt.signature->flags.exportable )
-		continue; /* not exportable */
-
-	      /* do not export packets with a "sensitive" revocation
-                 key.  This will need revisiting when we start
-                 supporting creating revocation keys and not just
-                 reading them. */
-	      if( node->pkt->pkt.signature->revkey ) {
-		int i;
-
-		for(i=0;i<node->pkt->pkt.signature->numrevkeys;i++)
-		  if(node->pkt->pkt.signature->revkey[i]->class & 0x40)
-		    continue;
-	      }
-
-	      /* delete our verification cache */
-	      delete_sig_subpkt (node->pkt->pkt.signature->unhashed,
-				 SIGSUBPKT_PRIV_VERIFY_CACHE);
+		const char *p;
+		p = parse_sig_subpkt2( node->pkt->pkt.signature,
+				       SIGSUBPKT_EXPORTABLE, NULL );
+		if( p && !*p )
+		    continue; /* not exportable */
 	    }
 
 	    if( secret == 2 && node->pkt->pkttype == PKT_SECRET_KEY ) {
@@ -237,8 +214,8 @@ do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 
 	    if( rc ) {
 		log_error("build_packet(%d) failed: %s\n",
-			    node->pkt->pkttype, g10_errstr(rc) );
-		rc = G10ERR_WRITE_FILE;
+			    node->pkt->pkttype, gpg_errstr(rc) );
+		rc = GPGERR_WRITE_FILE;
 		goto leave;
 	    }
 	}
@@ -248,8 +225,8 @@ do_export_stream( IOBUF out, STRLIST users, int secret, int onlyrfc, int *any )
 	rc = 0;
 
   leave:
-    m_free(desc);
-    keydb_release (kdbhd);
+    if( all == 2 )
+	enum_keyblocks( 2, &kbpos, &keyblock ); /* close */
     release_kbnode( keyblock );
     if( !*any )
 	log_info(_("WARNING: nothing exported\n"));
