@@ -1,5 +1,5 @@
 /* filter.h
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -17,13 +17,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifndef G10_FILTER_H
-#define G10_FILTER_H
+#ifndef GPG_FILTER_H
+#define GPG_FILTER_H
 
 #include <gcrypt.h>
 
 #include "basicdefs.h"
-
+#include "iobuf.h"
 
 
 typedef struct {
@@ -38,6 +38,9 @@ typedef struct {
     int only_keyblocks;     /* skip all headers but ".... key block" */
     const char *hdrlines;   /* write these headerlines */
 
+    /* these fileds must be initialized to zero */
+    int no_openpgp_data;    /* output flag: "No valid OpenPGP data found" */
+
     /* the following fields must be initialized to zero */
     int inp_checked;	    /* set if the input has been checked */
     int inp_bypass;	    /* set if the input is not armored */
@@ -59,6 +62,7 @@ typedef struct {
     u32 crc;
 
     int status; 	    /* an internal state flag */
+    int cancel;
     int any_data;	    /* any valid armored data seen */
     int pending_lf;	    /* used together with faked */
 } armor_filter_context_t;
@@ -82,6 +86,8 @@ typedef struct {
     GCRY_CIPHER_HD cipher_hd;
     int header;
     GCRY_MD_HD mdc_hash;
+    byte enchash[20];
+    int create_mdc; /* flag will be set by the cipher filter */
 } cipher_filter_context_t;
 
 
@@ -92,7 +98,6 @@ typedef struct {
 } encrypt_filter_context_t;
 
 
-
 typedef struct {
     byte *buffer;	    /* malloced buffer */
     unsigned buffer_size;   /* and size of this buffer */
@@ -107,8 +112,6 @@ typedef struct {
 } text_filter_context_t;
 
 
-/* encrypt_filter_context_t defined in main.h */
-
 /*-- mdfilter.c --*/
 int md_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len);
 void free_md_filter_context( md_filter_context_t *mfx );
@@ -134,4 +137,4 @@ int copy_clearsig_text( IOBUF out, IOBUF inp, GCRY_MD_HD md,
 
 
 
-#endif /*G10_FILTER_H*/
+#endif /*GPG_FILTER_H*/
