@@ -1,5 +1,5 @@
 /* free-packet.c - cleanup stuff for packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,18 +24,17 @@
 #include <string.h>
 #include <assert.h>
 
+#include <gcrypt.h>
 #include "packet.h"
 #include "iobuf.h"
-#include "mpi.h"
 #include "util.h"
-#include "cipher.h"
-#include "memory.h"
 #include "options.h"
+#include "main.h"
 
 void
 free_symkey_enc( PKT_symkey_enc *enc )
 {
-    m_free(enc);
+    gcry_free(enc);
 }
 
 void
@@ -44,10 +43,10 @@ free_pubkey_enc( PKT_pubkey_enc *enc )
     int n, i;
     n = pubkey_get_nenc( enc->pubkey_algo );
     if( !n )
-	mpi_free(enc->data[0]);
+	mpi_release(enc->data[0]);
     for(i=0; i < n; i++ )
-	mpi_free( enc->data[i] );
-    m_free(enc);
+	mpi_release( enc->data[i] );
+    gcry_free(enc);
 }
 
 void
@@ -56,14 +55,12 @@ free_seckey_enc( PKT_signature *sig )
     int n, i;
     n = pubkey_get_nsig( sig->pubkey_algo );
     if( !n )
-	mpi_free(sig->data[0]);
+	mpi_release(sig->data[0]);
     for(i=0; i < n; i++ )
-	mpi_free( sig->data[i] );
-
-    m_free(sig->revkey);
-    m_free(sig->hashed);
-    m_free(sig->unhashed);
-    m_free(sig);
+	mpi_release( sig->data[i] );
+    gcry_free(sig->hashed_data);
+    gcry_free(sig->unhashed_data);
+    gcry_free(sig);
 }
 
 
@@ -74,28 +71,15 @@ release_public_key_parts( PKT_public_key *pk )
     int n, i;
     n = pubkey_get_npkey( pk->pubkey_algo );
     if( !n )
-	mpi_free(pk->pkey[0]);
+	mpi_release(pk->pkey[0]);
     for(i=0; i < n; i++ ) {
-	mpi_free( pk->pkey[i] );
+	mpi_release( pk->pkey[i] );
 	pk->pkey[i] = NULL;
     }
-    if (pk->prefs) {
-        m_free (pk->prefs);
-        pk->prefs = NULL;
-    }
     if( pk->namehash ) {
-	m_free(pk->namehash);
+	gcry_free(pk->namehash);
 	pk->namehash = NULL;
     }
-    if (pk->user_id) {
-        free_user_id (pk->user_id);
-        pk->user_id = NULL;
-    }
-    if (pk->revkey) {
-        m_free(pk->revkey);
-	pk->revkey=NULL;
-	pk->numrevkeys=0;
-    }
 }
 
 
@@ -103,60 +87,42 @@ void
 free_public_key( PKT_public_key *pk )
 {
     release_public_key_parts( pk );
-    m_free(pk);
+    gcry_free(pk);
 }
 
 
-static subpktarea_t *
-cp_subpktarea (subpktarea_t *s )
+static void *
+cp_data_block( byte *s )
 {
-    subpktarea_t *d;
+    byte *d;
+    u16 len;
 
     if( !s )
 	return NULL;
-    d = m_alloc (sizeof (*d) + s->size - 1 );
-    d->size = s->size;
-    d->len = s->len;
-    memcpy (d->data, s->data, s->len);
+    len = (s[0] << 8) | s[1];
+    d = gcry_xmalloc( len+2 );
+    memcpy(d, s, len+2);
     return d;
 }
 
-/*
- * Return a copy of the preferences 
- */
-prefitem_t *
-copy_prefs (const prefitem_t *prefs)
-{
-    size_t n;
-    prefitem_t *new;
-
-    if (!prefs)
-        return NULL;
-    
-    for (n=0; prefs[n].type; n++)
-        ;
-    new = m_alloc ( sizeof (*new) * (n+1));
-    for (n=0; prefs[n].type; n++) {
-        new[n].type = prefs[n].type;
-        new[n].value = prefs[n].value;
-    }
-    new[n].type = PREFTYPE_NONE;
-    new[n].value = 0;
-
-    return new;
-}
-
 
 PKT_public_key *
-copy_public_key ( PKT_public_key *d, PKT_public_key *s)
+copy_public_key_new_namehash( PKT_public_key *d, PKT_public_key *s,
+			      const byte *namehash )
 {
     int n, i;
 
     if( !d )
-	d = m_alloc(sizeof *d);
+	d = gcry_xmalloc(sizeof *d);
     memcpy( d, s, sizeof *d );
-    d->user_id = scopy_user_id (s->user_id);
-    d->prefs = copy_prefs (s->prefs);
+    if( namehash ) {
+	d->namehash = gcry_xmalloc( 20 );
+	memcpy(d->namehash, namehash, 20 );
+    }
+    else if( s->namehash ) {
+	d->namehash = gcry_xmalloc( 20 );
+	memcpy(d->namehash, s->namehash, 20 );
+    }
     n = pubkey_get_npkey( s->pubkey_algo );
     if( !n )
 	d->pkey[0] = mpi_copy(s->pkey[0]);
@@ -164,31 +130,13 @@ copy_public_key ( PKT_public_key *d, PKT_public_key *s)
 	for(i=0; i < n; i++ )
 	    d->pkey[i] = mpi_copy( s->pkey[i] );
     }
-    d->revkey=m_alloc(sizeof(struct revocation_key)*s->numrevkeys);
-    memcpy(d->revkey,s->revkey,sizeof(struct revocation_key)*s->numrevkeys);
     return d;
 }
 
-/****************
- * Replace all common parts of a sk by the one from the public key.
- * This is a hack and a better solution will be to just store the real secret
- * parts somewhere and don't duplicate all the other stuff.
- */
-void
-copy_public_parts_to_secret_key( PKT_public_key *pk, PKT_secret_key *sk )
+PKT_public_key *
+copy_public_key( PKT_public_key *d, PKT_public_key *s )
 {
-    sk->expiredate  = pk->expiredate;     
-    sk->pubkey_algo = pk->pubkey_algo;    
-    sk->pubkey_usage= pk->pubkey_usage;
-    sk->req_usage   = pk->req_usage;
-    sk->req_algo    = pk->req_algo;
-    sk->has_expired = pk->has_expired;    
-    sk->is_revoked  = pk->is_revoked;     
-    sk->is_valid    = pk->is_valid;    
-    sk->main_keyid[0]= pk->main_keyid[0];
-    sk->main_keyid[1]= pk->main_keyid[1];
-    sk->keyid[0]    = pk->keyid[0];
-    sk->keyid[1]    = pk->keyid[1];
+   return copy_public_key_new_namehash( d, s, NULL );
 }
 
 PKT_signature *
@@ -197,7 +145,7 @@ copy_signature( PKT_signature *d, PKT_signature *s )
     int n, i;
 
     if( !d )
-	d = m_alloc(sizeof *d);
+	d = gcry_xmalloc(sizeof *d);
     memcpy( d, s, sizeof *d );
     n = pubkey_get_nsig( s->pubkey_algo );
     if( !n )
@@ -206,21 +154,19 @@ copy_signature( PKT_signature *d, PKT_signature *s )
 	for(i=0; i < n; i++ )
 	    d->data[i] = mpi_copy( s->data[i] );
     }
-    d->hashed = cp_subpktarea (s->hashed);
-    d->unhashed = cp_subpktarea (s->unhashed);
+    d->hashed_data = cp_data_block(s->hashed_data);
+    d->unhashed_data = cp_data_block(s->unhashed_data);
     return d;
 }
 
 
-/*
- * shallow copy of the user ID
- */
 PKT_user_id *
-scopy_user_id (PKT_user_id *s)
+copy_user_id( PKT_user_id *d, PKT_user_id *s )
 {
-    if (s)
-        s->ref++;
-    return s;
+    if( !d )
+	d = gcry_xmalloc(sizeof *d + s->len - 1 );
+    memcpy( d, s, sizeof *d + s->len - 1 );
+    return d;
 }
 
 
@@ -232,9 +178,9 @@ release_secret_key_parts( PKT_secret_key *sk )
 
     n = pubkey_get_nskey( sk->pubkey_algo );
     if( !n )
-	mpi_free(sk->skey[0]);
+	mpi_release(sk->skey[0]);
     for(i=0; i < n; i++ ) {
-	mpi_free( sk->skey[i] );
+	mpi_release( sk->skey[i] );
 	sk->skey[i] = NULL;
     }
 }
@@ -243,7 +189,7 @@ void
 free_secret_key( PKT_secret_key *sk )
 {
     release_secret_key_parts( sk );
-    m_free(sk);
+    gcry_free(sk);
 }
 
 PKT_secret_key *
@@ -252,7 +198,7 @@ copy_secret_key( PKT_secret_key *d, PKT_secret_key *s )
     int n, i;
 
     if( !d )
-	d = m_alloc(sizeof *d);
+	d = gcry_xmalloc(sizeof *d);
     memcpy( d, s, sizeof *d );
     n = pubkey_get_nskey( s->pubkey_algo );
     if( !n )
@@ -267,32 +213,15 @@ copy_secret_key( PKT_secret_key *d, PKT_secret_key *s )
 void
 free_comment( PKT_comment *rem )
 {
-    m_free(rem);
+    gcry_free(rem);
 }
 
 void
-free_attributes(PKT_user_id *uid)
+free_user_id( PKT_user_id *uid )
 {
-  m_free(uid->attribs);
-  m_free(uid->attrib_data);
-
-  uid->attribs=NULL;
-  uid->attrib_data=NULL;
-  uid->attrib_len=0;
-}
-
-void
-free_user_id (PKT_user_id *uid)
-{
-    assert (uid->ref > 0);
-    if (--uid->ref)
-        return;
-
-    free_attributes(uid);
-
-    if (uid->prefs)
-        m_free (uid->prefs);
-    m_free (uid);
+    if( uid->photo )
+	gcry_free( uid->photo );
+    gcry_free(uid);
 }
 
 void
@@ -304,7 +233,7 @@ free_compressed( PKT_compressed *zd )
 	while( iobuf_read( zd->buf, NULL, 1<<30 ) != -1 )
 	    ;
     }
-    m_free(zd);
+    gcry_free(zd);
 }
 
 void
@@ -325,7 +254,7 @@ free_encrypted( PKT_encrypted *ed )
 	   }
 	}
     }
-    m_free(ed);
+    gcry_free(ed);
 }
 
 
@@ -347,7 +276,7 @@ free_plaintext( PKT_plaintext *pt )
 	   }
 	}
     }
-    m_free(pt);
+    gcry_free(pt);
 }
 
 /****************
@@ -390,14 +319,13 @@ free_packet( PACKET *pkt )
 	free_compressed( pkt->pkt.compressed);
 	break;
       case PKT_ENCRYPTED:
-      case PKT_ENCRYPTED_MDC:
 	free_encrypted( pkt->pkt.encrypted );
 	break;
       case PKT_PLAINTEXT:
 	free_plaintext( pkt->pkt.plaintext );
 	break;
       default:
-	m_free( pkt->pkt.generic );
+	gcry_free( pkt->pkt.generic );
 	break;
     }
     pkt->pkt.generic = NULL;
@@ -515,9 +443,6 @@ cmp_user_ids( PKT_user_id *a, PKT_user_id *b )
 {
     int res;
 
-    if ( a == b )
-        return 0;
-
     res = a->len - b->len;
     if( !res )
 	res = memcmp( a->name, b->name, a->len );
