@@ -1,5 +1,5 @@
-/* hkp.h  -  Horowitz Keyserver Protocol
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/* hkp.h  -  Horrowitz Keyserver Protocol
+ *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -18,14 +18,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef G10_HKP_H
-#define G10_HKP_H 1
+#ifndef GPG_HKP_H
+#define GPG_HKP_H 1
 
 
-int hkp_ask_import( u32 *keyid, void *stats_handle);
+int hkp_ask_import( u32 *keyid );
 int hkp_import( STRLIST users );
 int hkp_export( STRLIST users );
-int hkp_refresh_keys (void);
 
 
-#endif /*G10_HKP_H*/
+#endif /*GPG_HKP_H*/
