@@ -1,5 +1,5 @@
 /* import.c
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -29,7 +29,7 @@
 #include "packet.h"
 #include "errors.h"
 #include "keydb.h"
-#include "memory.h"
+#include <gcrypt.h>
 #include "util.h"
 #include "trustdb.h"
 #include "main.h"
@@ -38,6 +38,7 @@
 
 
 static struct {
+    ulong count;
     ulong no_user_id;
     ulong imported;
     ulong imported_rsa;
@@ -53,6 +54,7 @@ static struct {
 
 
 static int import( IOBUF inp, int fast, const char* fname );
+static void print_stats(void);
 static int read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root );
 static int import_one( const char *fname, KBNODE keyblock, int fast );
 static int import_secret_one( const char *fname, KBNODE keyblock );
@@ -105,30 +107,51 @@ static int merge_keysigs( KBNODE dst, KBNODE src, int *n_sigs,
  *  Key revocation certificates have special handling.
  *
  */
-int
-import_keys( const char *fname, int fast )
+void
+import_keys( char **fnames, int nnames, int fast )
 {
-    IOBUF inp = NULL;
-    int rc;
+    int i;
 
-    inp = iobuf_open(fname);
-    if( !fname )
-	fname = "[stdin]";
-    if( !inp ) {
-	log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
-	return G10ERR_OPEN_FILE;
+    /* fixme: don't use static variables */
+    memset( &stats, 0, sizeof( stats ) );
+
+    if( !fnames && !nnames )
+	nnames = 1;  /* Ohh what a ugly hack to jump into the loop */
+
+    for(i=0; i < nnames; i++ ) {
+	const char *fname = fnames? fnames[i] : NULL;
+	IOBUF inp = iobuf_open(fname);
+	if( !fname )
+	    fname = "[stdin]";
+	if( !inp )
+	    log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
+	else {
+	    int rc = import( inp, fast, fname );
+	    iobuf_close(inp);
+	    if( rc )
+		log_error("import from `%s' failed: %s\n", fname,
+							   gpg_errstr(rc) );
+	}
+	if( !fname )
+	    break;
     }
-
-    rc = import( inp, fast, fname );
-
-    iobuf_close(inp);
-    return rc;
+    print_stats();
+    if( !fast )
+	sync_trustdb();
 }
 
 int
 import_keys_stream( IOBUF inp, int fast )
 {
-    return import( inp, fast, "[stream]" );
+    int rc = 0;
+
+    /* fixme: don't use static variables */
+    memset( &stats, 0, sizeof( stats ) );
+    rc = import( inp, fast, "[stream]" );
+    print_stats();
+    if( !fast )
+	sync_trustdb();
+    return rc;
 }
 
 static int
@@ -137,15 +160,11 @@ import( IOBUF inp, int fast, const char* fname )
     PACKET *pending_pkt = NULL;
     KBNODE keyblock;
     int rc = 0;
-    ulong count=0;
-
-    /* fixme: don't use static variables */
-    memset( &stats, 0, sizeof( stats ) );
 
     getkey_disable_caches();
 
     if( !opt.no_armor ) { /* armored reading is not disabled */
-	armor_filter_context_t *afx = m_alloc_clear( sizeof *afx );
+	armor_filter_context_t *afx = gcry_xcalloc( 1, sizeof *afx );
 	afx->only_keyblocks = 1;
 	iobuf_push_filter2( inp, armor_filter, afx, 1 );
     }
@@ -165,16 +184,23 @@ import( IOBUF inp, int fast, const char* fname )
 	release_kbnode(keyblock);
 	if( rc )
 	    break;
-	if( !(++count % 100) && !opt.quiet )
-	    log_info(_("%lu keys so far processed\n"), count );
+	if( !(++stats.count % 100) && !opt.quiet )
+	    log_info(_("%lu keys so far processed\n"), stats.count );
     }
     if( rc == -1 )
 	rc = 0;
-    else if( rc && rc != G10ERR_INV_KEYRING )
-	log_error( _("error reading `%s': %s\n"), fname, g10_errstr(rc));
+    else if( rc && rc != GPGERR_INV_KEYRING )
+	log_error( _("error reading `%s': %s\n"), fname, gpg_errstr(rc));
 
+    return rc;
+}
+
+
+static void
+print_stats()
+{
     if( !opt.quiet ) {
-	log_info(_("Total number processed: %lu\n"), count );
+	log_info(_("Total number processed: %lu\n"), stats.count );
 	if( stats.no_user_id )
 	    log_info(_("          w/o user IDs: %lu\n"), stats.no_user_id );
 	if( stats.imported || stats.imported_rsa ) {
@@ -202,9 +228,9 @@ import( IOBUF inp, int fast, const char* fname )
     }
 
     if( is_status_enabled() ) {
-	char buf[12*16];
+	char buf[12*20];
 	sprintf(buf, "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
-		count,
+		stats.count,
 		stats.no_user_id,
 		stats.imported,
 		stats.imported_rsa,
@@ -218,8 +244,6 @@ import( IOBUF inp, int fast, const char* fname )
 		stats.secret_dups);
 	write_status_text( STATUS_IMPORT_RES, buf );
     }
-
-    return rc;
 }
 
 
@@ -244,13 +268,13 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
     }
     else
 	in_cert = 0;
-    pkt = m_alloc( sizeof *pkt );
+    pkt = gcry_xmalloc( sizeof *pkt );
     init_packet(pkt);
     while( (rc=parse_packet(a, pkt)) != -1 ) {
 	if( rc ) {  /* ignore errors */
-	    if( rc != G10ERR_UNKNOWN_PACKET ) {
-		log_error("read_block: read error: %s\n", g10_errstr(rc) );
-		rc = G10ERR_INV_KEYRING;
+	    if( rc != GPGERR_UNKNOWN_PACKET ) {
+		log_error("read_block: read error: %s\n", gpg_errstr(rc) );
+		rc = GPGERR_INV_KEYRING;
 		goto ready;
 	    }
 	    free_packet( pkt );
@@ -272,11 +296,11 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
 	  case PKT_COMPRESSED:
 	    if( pkt->pkt.compressed->algorithm < 1
 		|| pkt->pkt.compressed->algorithm > 2 ) {
-		rc = G10ERR_COMPR_ALGO;
+		rc = GPGERR_COMPR_ALGO;
 		goto ready;
 	    }
 	    {
-		compress_filter_context_t *cfx = m_alloc_clear( sizeof *cfx );
+		compress_filter_context_t *cfx = gcry_xcalloc( 1, sizeof *cfx );
 		cfx->algo = pkt->pkt.compressed->algorithm;
 		pkt->pkt.compressed->buf = NULL;
 		iobuf_push_filter2( a, compress_filter, cfx, 1 );
@@ -300,7 +324,7 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
 		    root = new_kbnode( pkt );
 		else
 		    add_kbnode( root, new_kbnode( pkt ) );
-		pkt = m_alloc( sizeof *pkt );
+		pkt = gcry_xmalloc( sizeof *pkt );
 	    }
 	    init_packet(pkt);
 	    break;
@@ -315,7 +339,7 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
     else
 	*ret_root = root;
     free_packet( pkt );
-    m_free( pkt );
+    gcry_free( pkt );
     return rc;
 }
 
@@ -324,7 +348,7 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
  * Try to import one keyblock.	Return an error only in serious cases, but
  * never for an invalid keyblock.  It uses log_error to increase the
  * internal errorcount, so that invalid input can be detected by programs
- * which called g10.
+ * which called gpg.
  */
 static int
 import_one( const char *fname, KBNODE keyblock, int fast )
@@ -354,8 +378,8 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 		  pubkey_letter( pk->pubkey_algo ),
 		  (ulong)keyid[1], datestr_from_pk(pk) );
 	if( uidnode )
-	    print_string( stderr, uidnode->pkt->pkt.user_id->name,
-				  uidnode->pkt->pkt.user_id->len, 0 );
+	    print_utf8_string( stderr, uidnode->pkt->pkt.user_id->name,
+				       uidnode->pkt->pkt.user_id->len );
 	putc('\n', stderr);
     }
     if( !uidnode ) {
@@ -383,27 +407,27 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 
 
     /* do we have this key already in one of our pubrings ? */
-    pk_orig = m_alloc_clear( sizeof *pk_orig );
+    pk_orig = gcry_xcalloc( 1, sizeof *pk_orig );
     rc = get_pubkey( pk_orig, keyid );
-    if( rc && rc != G10ERR_NO_PUBKEY ) {
+    if( rc && rc != GPGERR_NO_PUBKEY ) {
 	log_error( _("key %08lX: public key not found: %s\n"),
-				(ulong)keyid[1], g10_errstr(rc));
+				(ulong)keyid[1], gpg_errstr(rc));
     }
     else if( rc ) { /* insert this key */
 	/* get default resource */
 	if( get_keyblock_handle( NULL, 0, &kbpos ) ) {
 	    log_error(_("no default public keyring\n"));
-	    return G10ERR_GENERAL;
+	    return GPGERR_GENERAL;
 	}
 	if( opt.verbose > 1 )
 	    log_info( _("writing to `%s'\n"),
 				keyblock_resource_name(&kbpos) );
 	if( (rc=lock_keyblock( &kbpos )) )
 	   log_error(_("can't lock keyring `%s': %s\n"),
-		       keyblock_resource_name(&kbpos), g10_errstr(rc) );
+		       keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	else if( (rc=insert_keyblock( &kbpos, keyblock )) )
 	   log_error( _("error writing keyring `%s': %s\n"),
-		       keyblock_resource_name(&kbpos), g10_errstr(rc) );
+		       keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	unlock_keyblock( &kbpos );
 	/* we are ready */
 	if( !opt.quiet )
@@ -411,7 +435,7 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	if( is_status_enabled() ) {
 	    char *us = get_long_user_id_string( keyid );
 	    write_status_text( STATUS_IMPORTED, us );
-	    m_free(us);
+	    gcry_free(us);
 	}
 	stats.imported++;
 	if( is_RSA( pk->pubkey_algo ) )
@@ -426,7 +450,7 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	if( cmp_public_keys( pk_orig, pk ) ) {
 	    log_error( _("key %08lX: doesn't match our copy\n"),
 							  (ulong)keyid[1]);
-	    rc = G10ERR_GENERAL;
+	    rc = GPGERR_GENERAL;
 	    goto leave;
 	}
 
@@ -434,13 +458,13 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	rc = find_keyblock_bypk( &kbpos, pk_orig );
 	if( rc ) {
 	    log_error( _("key %08lX: can't locate original keyblock: %s\n"),
-				     (ulong)keyid[1], g10_errstr(rc));
+				     (ulong)keyid[1], gpg_errstr(rc));
 	    goto leave;
 	}
 	rc = read_keyblock( &kbpos, &keyblock_orig );
 	if( rc ) {
 	    log_error( _("key %08lX: can't read original keyblock: %s\n"),
-					    (ulong)keyid[1], g10_errstr(rc));
+					    (ulong)keyid[1], gpg_errstr(rc));
 	    goto leave;
 	}
 
@@ -458,10 +482,10 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	    /* keyblock_orig has been updated; write */
 	    if( (rc=lock_keyblock( &kbpos )) )
 	       log_error( _("can't lock keyring `%s': %s\n"),
-			  keyblock_resource_name(&kbpos), g10_errstr(rc) );
+			  keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	    else if( (rc=update_keyblock( &kbpos, keyblock_orig )) )
 		log_error( _("error writing keyring `%s': %s\n"),
-			     keyblock_resource_name(&kbpos), g10_errstr(rc) );
+			     keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	    unlock_keyblock( &kbpos );
 	    /* we are ready */
 	    if( !opt.quiet ) {
@@ -498,12 +522,12 @@ import_one( const char *fname, KBNODE keyblock, int fast )
     if( !rc && !fast ) {
 	rc = query_trust_record( new_key? pk : pk_orig );
 	if( rc && rc != -1 )
-	    log_error("trustdb error: %s\n", g10_errstr(rc) );
+	    log_error("trustdb error: %s\n", gpg_errstr(rc) );
 	else if( rc == -1 ) { /* not found trustdb */
 	    rc = insert_trust_record( new_key? keyblock : keyblock_orig );
 	    if( rc )
 		log_error("key %08lX: trustdb insert failed: %s\n",
-					(ulong)keyid[1], g10_errstr(rc) );
+					(ulong)keyid[1], gpg_errstr(rc) );
 	}
 	else if( mod_key )
 	    rc = update_trust_record( keyblock_orig, 1, NULL );
@@ -545,8 +569,8 @@ import_secret_one( const char *fname, KBNODE keyblock )
 		  pubkey_letter( sk->pubkey_algo ),
 		  (ulong)keyid[1], datestr_from_sk(sk) );
 	if( uidnode )
-	    print_string( stderr, uidnode->pkt->pkt.user_id->name,
-				  uidnode->pkt->pkt.user_id->len, 0 );
+	    print_utf8_string( stderr, uidnode->pkt->pkt.user_id->name,
+				       uidnode->pkt->pkt.user_id->len );
 	putc('\n', stderr);
     }
     stats.secret_read++;
@@ -559,18 +583,18 @@ import_secret_one( const char *fname, KBNODE keyblock )
 
     /* do we have this key already in one of our secrings ? */
     rc = seckey_available( keyid );
-    if( rc == G10ERR_NO_SECKEY ) { /* simply insert this key */
+    if( rc == GPGERR_NO_SECKEY ) { /* simply insert this key */
 	/* get default resource */
 	if( get_keyblock_handle( NULL, 1, &kbpos ) ) {
 	    log_error("no default secret keyring\n");
-	    return G10ERR_GENERAL;
+	    return GPGERR_GENERAL;
 	}
 	if( (rc=lock_keyblock( &kbpos )) )
 	    log_error( _("can't lock keyring `%s': %s\n"),
-			 keyblock_resource_name(&kbpos), g10_errstr(rc) );
+			 keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	else if( (rc=insert_keyblock( &kbpos, keyblock )) )
 	    log_error( _("error writing keyring `%s': %s\n"),
-		      keyblock_resource_name(&kbpos), g10_errstr(rc) );
+		      keyblock_resource_name(&kbpos), gpg_errstr(rc) );
 	unlock_keyblock( &kbpos );
 	/* we are ready */
 	if( !opt.quiet )
@@ -584,7 +608,7 @@ import_secret_one( const char *fname, KBNODE keyblock )
     }
     else
 	log_error( _("key %08lX: secret key not found: %s\n"),
-				(ulong)keyid[1], g10_errstr(rc));
+				(ulong)keyid[1], gpg_errstr(rc));
 
     return rc;
 }
@@ -609,9 +633,9 @@ import_revoke_cert( const char *fname, KBNODE node )
     keyid[0] = node->pkt->pkt.signature->keyid[0];
     keyid[1] = node->pkt->pkt.signature->keyid[1];
 
-    pk = m_alloc_clear( sizeof *pk );
+    pk = gcry_xcalloc( 1, sizeof *pk );
     rc = get_pubkey( pk, keyid );
-    if( rc == G10ERR_NO_PUBKEY ) {
+    if( rc == GPGERR_NO_PUBKEY ) {
 	log_info( _("key %08lX: no public key - "
 		 "can't apply revocation certificate\n"), (ulong)keyid[1]);
 	rc = 0;
@@ -619,7 +643,7 @@ import_revoke_cert( const char *fname, KBNODE node )
     }
     else if( rc ) {
 	log_error( _("key %08lX: public key not found: %s\n"),
-				       (ulong)keyid[1], g10_errstr(rc));
+				       (ulong)keyid[1], gpg_errstr(rc));
 	goto leave;
     }
 
@@ -627,13 +651,13 @@ import_revoke_cert( const char *fname, KBNODE node )
     rc = find_keyblock_bypk( &kbpos, pk );
     if( rc ) {
 	log_error( _("key %08lX: can't locate original keyblock: %s\n"),
-					(ulong)keyid[1], g10_errstr(rc));
+					(ulong)keyid[1], gpg_errstr(rc));
 	goto leave;
     }
     rc = read_keyblock( &kbpos, &keyblock );
     if( rc ) {
 	log_error( _("key %08lX: can't read original keyblock: %s\n"),
-					(ulong)keyid[1], g10_errstr(rc));
+					(ulong)keyid[1], gpg_errstr(rc));
 	goto leave;
     }
 
@@ -644,7 +668,7 @@ import_revoke_cert( const char *fname, KBNODE node )
     rc = check_key_signature( keyblock, node, NULL);
     if( rc ) {
 	log_error( _("key %08lX: invalid revocation certificate"
-		  ": %s - rejected\n"), (ulong)keyid[1], g10_errstr(rc));
+		  ": %s - rejected\n"), (ulong)keyid[1], gpg_errstr(rc));
     }
 
 
@@ -668,16 +692,25 @@ import_revoke_cert( const char *fname, KBNODE node )
     /* and write the keyblock back */
     if( (rc=lock_keyblock( &kbpos )) )
 	log_error( _("can't lock keyring `%s': %s\n"),
-		   keyblock_resource_name(&kbpos), g10_errstr(rc) );
+		   keyblock_resource_name(&kbpos), gpg_errstr(rc) );
     else if( (rc=update_keyblock( &kbpos, keyblock )) )
 	log_error( _("error writing keyring `%s': %s\n"),
-		    keyblock_resource_name(&kbpos), g10_errstr(rc) );
+		    keyblock_resource_name(&kbpos), gpg_errstr(rc) );
     unlock_keyblock( &kbpos );
     /* we are ready */
     if( !opt.quiet )
 	log_info( _("key %08lX: revocation certificate imported\n"),
 					(ulong)keyid[1]);
     stats.n_revoc++;
+    if( clear_trust_checked_flag( pk ) ) {
+	/* seems that we have to insert the record first */
+	rc = insert_trust_record( keyblock );
+	if( rc )
+	    log_error("key %08lX: trustdb insert failed: %s\n",
+					(ulong)keyid[1], gpg_errstr(rc) );
+	else
+	    rc = clear_trust_checked_flag( pk );
+    }
 
   leave:
     release_kbnode( keyblock );
@@ -714,7 +747,7 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		}
 		rc = check_key_signature( keyblock, n, NULL);
 		if( rc ) {
-		    log_info( rc == G10ERR_PUBKEY_ALGO ?
+		    log_info( rc == GPGERR_PUBKEY_ALGO ?
 			 _("key %08lX: unsupported public key algorithm\n"):
 			 _("key %08lX: invalid self-signature\n"),
 				     (ulong)keyid[1]);
@@ -738,7 +771,7 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		else {
 		    rc = check_key_signature( keyblock, n, NULL);
 		    if( rc ) {
-			log_info(  rc == G10ERR_PUBKEY_ALGO ?
+			log_info(  rc == GPGERR_PUBKEY_ALGO ?
 			   _("key %08lX: unsupported public key algorithm\n"):
 			   _("key %08lX: invalid subkey binding\n"),
 					 (ulong)keyid[1]);
@@ -764,7 +797,8 @@ mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid )
     KBNODE node;
     for(node=keyblock->next; node; node = node->next ) {
 	if( node->pkt->pkttype == PKT_USER_ID && !(node->flag & 1) ) {
-	    if( node->next && node->next->pkt->pkttype == PKT_SIGNATURE ) {
+	    if( (node->next && node->next->pkt->pkttype == PKT_SIGNATURE)
+		|| !node->next ) {
 		node->flag |= 1;
 		log_info( _("key %08lX: accepted non self-signed user ID '"),
 							 (ulong)kid[1]);
@@ -797,8 +831,8 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 		if( opt.verbose ) {
 		    log_info( _("key %08lX: skipped user ID '"),
 							 (ulong)keyid[1]);
-		    print_string( stderr, node->pkt->pkt.user_id->name,
-				      node->pkt->pkt.user_id->len, 0 );
+		    print_utf8_string( stderr, node->pkt->pkt.user_id->name,
+				       node->pkt->pkt.user_id->len );
 		    fputs("'\n", stderr );
 		}
 		delete_kbnode( node ); /* the user-id */
@@ -831,8 +865,8 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 	    }
 	}
 	else if( node->pkt->pkttype == PKT_SIGNATURE
-		 && check_pubkey_algo( node->pkt->pkt.signature->pubkey_algo)
-		 && node->pkt->pkt.signature->pubkey_algo != PUBKEY_ALGO_RSA )
+		 && openpgp_pk_test_algo( node->pkt->pkt.signature->pubkey_algo, 0 )
+		 && node->pkt->pkt.signature->pubkey_algo != GCRY_PK_RSA )
 	    delete_kbnode( node ); /* build_packet() can't handle this */
 	else if( node->pkt->pkttype == PKT_SIGNATURE
 		 && (p = parse_sig_subpkt2( node->pkt->pkt.signature,
@@ -862,7 +896,7 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 		if( rc ) {
 		    log_error( _("key %08lX: invalid revocation "
 			      "certificate: %s - skipped\n"),
-			      (ulong)keyid[1], g10_errstr(rc));
+			      (ulong)keyid[1], gpg_errstr(rc));
 		    delete_kbnode( node );
 		}
 	    }
@@ -1127,7 +1161,7 @@ append_uid( KBNODE keyblock, KBNODE node, int *n_sigs,
     if( !node->next || node->next->pkt->pkttype == PKT_USER_ID ) {
 	log_error( _("key %08lX: our copy has no self-signature\n"),
 						  (ulong)keyid[1]);
-	return G10ERR_GENERAL;
+	return GPGERR_GENERAL;
     }
 
     /* find the position */
