@@ -1,5 +1,5 @@
 /* kbnode.c -  keyblock node utility functions
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,8 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+
+#include <gcrypt.h>
 #include "util.h"
-#include "memory.h"
 #include "packet.h"
 #include "keydb.h"
 
@@ -41,7 +42,7 @@ alloc_node(void)
     if( n )
 	unused_nodes = n->next;
     else
-	n = m_alloc( sizeof *n );
+	n = gcry_xmalloc( sizeof *n );
     n->next = NULL;
     n->pkt = NULL;
     n->flag = 0;
@@ -58,7 +59,7 @@ free_node( KBNODE n )
 	n->next = unused_nodes;
 	unused_nodes = n;
       #else
-	m_free( n );
+	gcry_free( n );
       #endif
     }
 }
@@ -94,7 +95,7 @@ release_kbnode( KBNODE n )
 	n2 = n->next;
 	if( !is_cloned_kbnode(n) ) {
 	    free_packet( n->pkt );
-	    m_free( n->pkt );
+	    gcry_free( n->pkt );
 	}
 	free_node( n );
 	n = n2;
@@ -266,7 +267,7 @@ commit_kbnode( KBNODE *root )
 		nl->next = n->next;
 	    if( !is_cloned_kbnode(n) ) {
 		free_packet( n->pkt );
-		m_free( n->pkt );
+		gcry_free( n->pkt );
 	    }
 	    free_node( n );
 	    changed = 1;
@@ -290,7 +291,7 @@ remove_kbnode( KBNODE *root, KBNODE node )
 		nl->next = n->next;
 	    if( !is_cloned_kbnode(n) ) {
 		free_packet( n->pkt );
-		m_free( n->pkt );
+		gcry_free( n->pkt );
 	    }
 	    free_node( n );
 	}
@@ -355,7 +356,6 @@ dump_kbnode( KBNODE node )
 	  case PKT_PLAINTEXT:	s="plaintext"; break;
 	  case PKT_COMPRESSED:	s="compressed"; break;
 	  case PKT_ENCRYPTED:	s="encrypted"; break;
-          case PKT_GPG_CONTROL: s="gpg-control"; break;
 	  default:		s="unknown"; break;
 	}
 	fprintf(stderr, "node %p %02x/%02x type=%s",
@@ -371,11 +371,6 @@ dump_kbnode( KBNODE node )
 		   node->pkt->pkt.signature->sig_class,
 		   (ulong)node->pkt->pkt.signature->keyid[1] );
 	}
-	else if( node->pkt->pkttype == PKT_GPG_CONTROL ) {
-	    fprintf(stderr, " ctrl=%d len=%u\n",
-                    node->pkt->pkt.gpg_control->control,
-                    (unsigned int)node->pkt->pkt.gpg_control->datalen);
-	}
 	else if( node->pkt->pkttype == PKT_PUBLIC_KEY
 		 || node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
 	    fprintf(stderr, "  keyid=%08lX\n", (ulong)
@@ -385,3 +380,4 @@ dump_kbnode( KBNODE node )
 	    fputs("\n", stderr);
     }
 }
+
