@@ -1,5 +1,5 @@
 /* keyid.c - jeyid and fingerprint handling
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,25 +25,26 @@
 #include <errno.h>
 #include <time.h>
 #include <assert.h>
+
 #include <gcrypt.h>
 #include "util.h"
 #include "main.h"
 #include "packet.h"
 #include "options.h"
-#include "mpi.h"
 #include "keydb.h"
+#include "i18n.h"
 
 
 int
 pubkey_letter( int algo )
 {
     switch( algo ) {
-      case PUBKEY_ALGO_RSA:	return 'R' ;
-      case PUBKEY_ALGO_RSA_E:	return 'r' ;
-      case PUBKEY_ALGO_RSA_S:	return 's' ;
-      case PUBKEY_ALGO_ELGAMAL_E: return 'g';
-      case PUBKEY_ALGO_ELGAMAL: return 'G' ;
-      case PUBKEY_ALGO_DSA:	return 'D' ;
+      case GCRY_PK_RSA: return 'R' ;
+      case GCRY_PK_RSA_E:	return 'r' ;
+      case GCRY_PK_RSA_S:	return 's' ;
+      case GCRY_PK_ELG_E: return 'g';
+      case GCRY_PK_ELG: return 'G' ;
+      case GCRY_PK_DSA: return 'D' ;
       default: return '?';
     }
 }
@@ -53,10 +54,9 @@ static GCRY_MD_HD
 do_fingerprint_md( PKT_public_key *pk )
 {
     GCRY_MD_HD md;
-    unsigned n;
-    unsigned nb[PUBKEY_MAX_NPKEY];
-    unsigned nn[PUBKEY_MAX_NPKEY];
-    byte *pp[PUBKEY_MAX_NPKEY];
+    unsigned int n;
+    unsigned int nn[GNUPG_MAX_NPKEY];
+    byte *pp[GNUPG_MAX_NPKEY];
     int i;
     int npkey = pubkey_get_npkey( pk->pubkey_algo );
 
@@ -65,9 +65,17 @@ do_fingerprint_md( PKT_public_key *pk )
 	BUG();
     n = pk->version < 4 ? 8 : 6;
     for(i=0; i < npkey; i++ ) {
-	nb[i] = mpi_get_nbits(pk->pkey[i]);
-	pp[i] = mpi_get_buffer( pk->pkey[i], nn+i, NULL );
-	n += 2 + nn[i];
+	int rc;
+	size_t nbytes;
+
+	rc = gcry_mpi_print( GCRYMPI_FMT_PGP, NULL, &nbytes, pk->pkey[i] );
+	assert( !rc );
+	/* fixme: we should try to allocate a buffer on the stack */
+	pp[i] = gcry_xmalloc(nbytes);
+	rc = gcry_mpi_print( GCRYMPI_FMT_PGP, pp[i], &nbytes, pk->pkey[i] );
+	assert( !rc );
+	nn[i] = nbytes;
+	n += nn[i];
     }
 
     gcry_md_putc( md, 0x99 );	  /* ctb */
@@ -96,10 +104,8 @@ do_fingerprint_md( PKT_public_key *pk )
     }
     gcry_md_putc( md, pk->pubkey_algo );
     for(i=0; i < npkey; i++ ) {
-	gcry_md_putc( md, nb[i]>>8);
-	gcry_md_putc( md, nb[i] );
 	gcry_md_write( md, pp[i], nn[i] );
-	m_free(pp[i]);
+	gcry_free(pp[i]);
     }
     gcry_md_final( md );
 
@@ -124,6 +130,30 @@ do_fingerprint_md_sk( PKT_secret_key *sk )
 }
 
 
+static void
+v3_keyid( MPI a, u32 *ki )
+{
+    int rc;
+    byte *buffer;
+    size_t nbytes;
+
+    rc = gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, a );
+    assert( !rc );
+    /* fixme: allocate it on the stack */
+    buffer = gcry_xmalloc(nbytes);
+    rc = gcry_mpi_print( GCRYMPI_FMT_USG, buffer, &nbytes, a );
+    assert( !rc );
+    if( nbytes < 8 ) { /* oops */
+	ki[0] = ki[1] = 0;
+    }
+    else  {
+	memcpy( ki+0, buffer+nbytes-8, 4);
+	memcpy( ki+1, buffer+nbytes-4, 4);
+    }
+    gcry_free( buffer );
+}
+
+
 /****************
  * Get the keyid from the secret key and put it into keyid
  * if this is not NULL. Return the 32 low bits of the keyid.
@@ -131,15 +161,16 @@ do_fingerprint_md_sk( PKT_secret_key *sk )
 u32
 keyid_from_sk( PKT_secret_key *sk, u32 *keyid )
 {
-    u32 lowbits;
     u32 dummy_keyid[2];
 
     if( !keyid )
 	keyid = dummy_keyid;
 
     if( sk->version < 4 && is_RSA(sk->pubkey_algo) ) {
-	lowbits = pubkey_get_npkey(sk->pubkey_algo) ?
-		     mpi_get_keyid( sk->skey[0], keyid ) : 0; /* take n */
+	if( pubkey_get_npkey(sk->pubkey_algo) )
+	    v3_keyid( sk->skey[0], keyid ); /* take n */
+	else
+	    keyid[0] = keyid[1] = 0;
     }
     else {
 	const byte *dp;
@@ -148,11 +179,10 @@ keyid_from_sk( PKT_secret_key *sk, u32 *keyid )
 	dp = gcry_md_read( md, 0 );
 	keyid[0] = dp[12] << 24 | dp[13] << 16 | dp[14] << 8 | dp[15] ;
 	keyid[1] = dp[16] << 24 | dp[17] << 16 | dp[18] << 8 | dp[19] ;
-	lowbits = keyid[1];
 	gcry_md_close(md);
     }
 
-    return lowbits;
+    return keyid[1];
 }
 
 
@@ -163,7 +193,6 @@ keyid_from_sk( PKT_secret_key *sk, u32 *keyid )
 u32
 keyid_from_pk( PKT_public_key *pk, u32 *keyid )
 {
-    u32 lowbits;
     u32 dummy_keyid[2];
 
     if( !keyid )
@@ -172,11 +201,12 @@ keyid_from_pk( PKT_public_key *pk, u32 *keyid )
     if( pk->keyid[0] || pk->keyid[1] ) {
 	keyid[0] = pk->keyid[0];
 	keyid[1] = pk->keyid[1];
-	lowbits = keyid[1];
     }
     else if( pk->version < 4 && is_RSA(pk->pubkey_algo) ) {
-	lowbits = pubkey_get_npkey(pk->pubkey_algo) ?
-		     mpi_get_keyid( pk->pkey[0], keyid ) : 0 ; /* from n */
+	if( pubkey_get_npkey(pk->pubkey_algo) )
+	    v3_keyid( pk->pkey[0], keyid ); /* from n */
+	else
+	    keyid[0] = keyid[1] = 0;
 	pk->keyid[0] = keyid[0];
 	pk->keyid[1] = keyid[1];
     }
@@ -187,13 +217,12 @@ keyid_from_pk( PKT_public_key *pk, u32 *keyid )
 	dp = gcry_md_read( md, 0 );
 	keyid[0] = dp[12] << 24 | dp[13] << 16 | dp[14] << 8 | dp[15] ;
 	keyid[1] = dp[16] << 24 | dp[17] << 16 | dp[18] << 8 | dp[19] ;
-	lowbits = keyid[1];
 	gcry_md_close(md);
 	pk->keyid[0] = keyid[0];
 	pk->keyid[1] = keyid[1];
     }
 
-    return lowbits;
+    return keyid[1];
 }
 
 
@@ -312,7 +341,7 @@ expirestr_from_pk( PKT_public_key *pk )
     time_t atime;
 
     if( !pk->expiredate )
-	return "never     ";
+	return _("never     ");
     atime = pk->expiredate;
     tp = gmtime( &atime );
     sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
@@ -327,7 +356,7 @@ expirestr_from_sk( PKT_secret_key *sk )
     time_t atime;
 
     if( !sk->expiredate )
-	return "never     ";
+	return _("never     ");
     atime = sk->expiredate;
     tp = gmtime( &atime );
     sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
@@ -344,31 +373,43 @@ expirestr_from_sk( PKT_secret_key *sk )
 byte *
 fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 {
-    byte *p, *buf;
+    byte *buf;
     const char *dp;
     size_t len;
-    unsigned n;
 
     if( pk->version < 4 && is_RSA(pk->pubkey_algo) ) {
 	/* RSA in version 3 packets is special */
 	GCRY_MD_HD md;
 
-	md = gcry_md_open( DIGEST_ALGO_MD5, 0);
+	md = gcry_md_open( GCRY_MD_MD5, 0);
 	if( !md )
 	    BUG();
 	if( pubkey_get_npkey( pk->pubkey_algo ) > 1 ) {
-	    p = buf = mpi_get_buffer( pk->pkey[0], &n, NULL );
-	    gcry_md_write( md, p, n );
-	    m_free(buf);
-	    p = buf = mpi_get_buffer( pk->pkey[1], &n, NULL );
-	    gcry_md_write( md, p, n );
-	    m_free(buf);
+	    int rc;
+	    size_t nbytes;
+
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, pk->pkey[0] );
+	    assert( !rc );
+	    /* fixme: allocate it on the stack */
+	    buf = gcry_xmalloc(nbytes);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, buf, &nbytes, pk->pkey[0] );
+	    assert( !rc );
+	    gcry_md_write( md, buf, nbytes );
+	    gcry_free(buf);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, pk->pkey[1] );
+	    assert( !rc );
+	    /* fixme: allocate it on the stack */
+	    buf = gcry_xmalloc(nbytes);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, buf, &nbytes, pk->pkey[1] );
+	    assert( !rc );
+	    gcry_md_write( md, buf, nbytes );
+	    gcry_free(buf);
 	}
 	gcry_md_final(md);
 	if( !array )
-	    array = m_alloc( 16 );
+	    array = gcry_xmalloc( 16 );
 	len = 16;
-	memcpy(array, gcry_md_read(md, DIGEST_ALGO_MD5), 16 );
+	memcpy(array, gcry_md_read(md, GCRY_MD_MD5), 16 );
 	gcry_md_close(md);
     }
     else {
@@ -378,7 +419,7 @@ fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 	len = gcry_md_get_algo_dlen( gcry_md_get_algo( md ) );
 	assert( len <= MAX_FINGERPRINT_LEN );
 	if( !array )
-	    array = m_alloc( len );
+	    array = gcry_xmalloc( len );
 	memcpy(array, dp, len );
 	gcry_md_close(md);
     }
@@ -390,29 +431,42 @@ fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 byte *
 fingerprint_from_sk( PKT_secret_key *sk, byte *array, size_t *ret_len )
 {
-    byte *p, *buf;
+    byte *buf;
     const char *dp;
     size_t len;
-    unsigned n;
 
     if( sk->version < 4 && is_RSA(sk->pubkey_algo) ) {
 	/* RSA in version 3 packets is special */
 	GCRY_MD_HD md;
 
-	md = gcry_md_open( DIGEST_ALGO_MD5, 0);
+	md = gcry_md_open( GCRY_MD_MD5, 0);
 	if( !md )
 	    BUG();
 	if( pubkey_get_npkey( sk->pubkey_algo ) > 1 ) {
-	    p = buf = mpi_get_buffer( sk->skey[1], &n, NULL );
-	    gcry_md_write( md, p, n );
-	    m_free(buf);
-	    p = buf = mpi_get_buffer( sk->skey[0], &n, NULL );
-	    gcry_md_write( md, p, n );
-	    m_free(buf);
+	    int rc;
+	    size_t nbytes;
+
+	    #warning Why is the hash sequence for secret keys different
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, sk->skey[1] );
+	    assert( !rc );
+	    /* fixme: allocate it on the stack */
+	    buf = gcry_xmalloc(nbytes);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, buf, &nbytes, sk->skey[1] );
+	    assert( !rc );
+	    gcry_md_write( md, buf, nbytes );
+	    gcry_free(buf);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, sk->skey[0] );
+	    assert( !rc );
+	    /* fixme: allocate it on the stack */
+	    buf = gcry_xmalloc(nbytes);
+	    rc = gcry_mpi_print( GCRYMPI_FMT_USG, buf, &nbytes, sk->skey[0] );
+	    assert( !rc );
+	    gcry_md_write( md, buf, nbytes );
+	    gcry_free(buf);
 	}
 	gcry_md_final(md);
 	if( !array )
-	    array = m_alloc( 16 );
+	    array = gcry_xmalloc( 16 );
 	len = 16;
 	memcpy(array, gcry_md_read(md, GCRY_MD_MD5), 16 );
 	gcry_md_close(md);
@@ -424,7 +478,7 @@ fingerprint_from_sk( PKT_secret_key *sk, byte *array, size_t *ret_len )
 	len = gcry_md_get_algo_dlen( gcry_md_get_algo( md ) );
 	assert( len <= MAX_FINGERPRINT_LEN );
 	if( !array )
-	    array = m_alloc( len );
+	    array = gcry_xmalloc( len );
 	memcpy(array, dp, len );
 	gcry_md_close(md);
     }
