@@ -1,5 +1,5 @@
-/* maPPPPinproc.c - handle packets
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+/* mainproc.c - handle packets
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -28,7 +28,6 @@
 #include <gcrypt.h>
 #include "packet.h"
 #include "iobuf.h"
-#include "memory.h"
 #include "options.h"
 #include "util.h"
 #include "keydb.h"
@@ -90,7 +89,7 @@ release_list( CTX c )
     release_kbnode( c->list );
     while( c->failed_pkenc ) {
 	struct kidlist_item *tmp = c->failed_pkenc->next;
-	m_free( c->failed_pkenc );
+	gcry_free( c->failed_pkenc );
 	c->failed_pkenc = tmp;
     }
     c->failed_pkenc = NULL;
@@ -161,11 +160,11 @@ add_signature( CTX c, PACKET *pkt )
 
     if( pkt->pkttype == PKT_SIGNATURE && !c->list ) {
 	/* This is the first signature for the following datafile.
-	 * G10 does not write such packets; instead it always uses
+	 * GPG does not write such packets; instead it always uses
 	 * onepass-sig packets.  The drawback of PGP's method
 	 * of prepending the signature to the data is
 	 * that it is not possible to make a signature from data read
-	 * from stdin.	(G10 is able to read PGP stuff anyway.) */
+	 * from stdin.	(GPG is able to read PGP stuff anyway.) */
 	node = new_kbnode( pkt );
 	c->list = node;
 	return 1;
@@ -221,21 +220,25 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 
 
     if( is_ELGAMAL(enc->pubkey_algo)
-	|| enc->pubkey_algo == PUBKEY_ALGO_DSA
+	|| enc->pubkey_algo == GCRY_PK_DSA
 	|| is_RSA(enc->pubkey_algo)  ) {
 	if ( !c->dek && ((!enc->keyid[0] && !enc->keyid[1])
 			  || !seckey_available( enc->keyid )) ) {
-	    c->dek = m_alloc_secure( sizeof *c->dek );
-	    if( (result = get_session_key( enc, c->dek )) ) {
-		/* error: delete the DEK */
-		m_free(c->dek); c->dek = NULL;
+	    if( opt.list_only )
+		result = -1;
+	    else {
+		c->dek = gcry_xmalloc_secure( sizeof *c->dek );
+		if( (result = get_session_key( enc, c->dek )) ) {
+		    /* error: delete the DEK */
+		    gcry_free(c->dek); c->dek = NULL;
+		}
 	    }
 	}
 	else
-	    result = G10ERR_NO_SECKEY;
+	    result = GPGERR_NO_SECKEY;
     }
     else
-	result = G10ERR_PUBKEY_ALGO;
+	result = GPGERR_PUBKEY_ALGO;
 
     if( result == -1 )
 	;
@@ -244,7 +247,7 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 	    log_info( _("public key encrypted data: good DEK\n") );
     }
     else { /* store it for later display */
-	struct kidlist_item *x = m_alloc( sizeof *x );
+	struct kidlist_item *x = gcry_xmalloc( sizeof *x );
 	x->kid[0] = enc->keyid[0];
 	x->kid[1] = enc->keyid[1];
 	x->pubkey_algo = enc->pubkey_algo;
@@ -265,9 +268,11 @@ static void
 print_failed_pkenc( struct kidlist_item *list )
 {
     for( ; list; list = list->next ) {
-	PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+	PKT_public_key *pk = gcry_xcalloc( 1, sizeof *pk );
 	const char *algstr = gcry_pk_algo_name( list->pubkey_algo );
 
+	if( !algstr )
+	    algstr = "[?]";
 	pk->pubkey_algo = list->pubkey_algo;
 	if( !get_pubkey( pk, list->kid ) ) {
 	    size_t n;
@@ -278,7 +283,7 @@ print_failed_pkenc( struct kidlist_item *list )
 	    fputs("      \"", log_stream() );
 	    p = get_user_id( list->kid, &n );
 	    print_string( log_stream(), p, n, '"' );
-	    m_free(p);
+	    gcry_free(p);
 	    fputs("\"\n", log_stream() );
 	}
 	else {
@@ -287,7 +292,7 @@ print_failed_pkenc( struct kidlist_item *list )
 	}
 	free_public_key( pk );
 
-	if( list->reason == G10ERR_NO_SECKEY ) {
+	if( list->reason == GPGERR_NO_SECKEY ) {
 	    log_info(_("no secret key for decryption available\n"));
 	    if( is_status_enabled() ) {
 		char buf[20];
@@ -298,7 +303,7 @@ print_failed_pkenc( struct kidlist_item *list )
 	}
 	else
 	    log_error(_("public key decryption failed: %s\n"),
-						g10_errstr(list->reason));
+						gpg_errstr(list->reason));
     }
 }
 
@@ -310,20 +315,25 @@ proc_encrypted( CTX c, PACKET *pkt )
 
     print_failed_pkenc( c->failed_pkenc );
 
+    write_status( STATUS_BEGIN_DECRYPTION );
+
     /*log_debug("dat: %sencrypted data\n", c->dek?"":"conventional ");*/
-    if( !c->dek && !c->last_was_session_key ) {
+    if( opt.list_only )
+	result = -1;
+    else if( !c->dek && !c->last_was_session_key ) {
 	/* assume this is old conventional encrypted data
-	 * Actually we should use IDEA and MD5 in this case, but becuase
+	 * Actually we should use IDEA and MD5 in this case, but because
 	 * IDEA is patented we can't do so */
 	c->dek = passphrase_to_dek( NULL, 0,
 		    opt.def_cipher_algo ? opt.def_cipher_algo
 					: DEFAULT_CIPHER_ALGO, NULL, 0 );
     }
     else if( !c->dek )
-	result = G10ERR_NO_SECKEY;
+	result = GPGERR_NO_SECKEY;
     if( !result )
 	result = decrypt_data( c, pkt->pkt.encrypted, c->dek );
-    m_free(c->dek); c->dek = NULL;
+
+    gcry_free(c->dek); c->dek = NULL;
     if( result == -1 )
 	;
     else if( !result ) {
@@ -333,18 +343,19 @@ proc_encrypted( CTX c, PACKET *pkt )
 	if( pkt->pkt.encrypted->mdc_method )
 	    write_status( STATUS_GOODMDC );
     }
-    else if( result == G10ERR_BAD_SIGN ) {
+    else if( result == GPGERR_BAD_SIGN ) {
 	log_error(_("WARNING: encrypted message has been manipulated!\n"));
 	write_status( STATUS_BADMDC );
     }
     else {
 	write_status( STATUS_DECRYPTION_FAILED );
-	log_error(_("decryption failed: %s\n"), g10_errstr(result));
-	/* Hmmm: does this work when we have encrypted using a multiple
+	log_error(_("decryption failed: %s\n"), gpg_errstr(result));
+	/* Hmmm: does this work when we have encrypted using multiple
 	 * ways to specify the session key (symmmetric and PK)*/
     }
     free_packet(pkt);
     c->last_was_session_key = 0;
+    write_status( STATUS_END_DECRYPTION );
 }
 
 
@@ -361,7 +372,8 @@ proc_plaintext( CTX c, PACKET *pkt )
     else if( opt.verbose )
 	log_info(_("original file name='%.*s'\n"), pt->namelen, pt->name);
     free_md_filter_context( &c->mfx );
-    if( !(c->mfx.md = gcry_md_open( 0, 0)) )
+    c->mfx.md = gcry_md_open( 0, 0);
+    if( !c->mfx.md )
 	BUG();
     /* fixme: we may need to push the textfilter if we have sigclass 1
      * and no armoring - Not yet tested
@@ -373,10 +385,9 @@ proc_plaintext( CTX c, PACKET *pkt )
     for(n=c->list; n; n = n->next ) {
 	if( n->pkt->pkttype == PKT_ONEPASS_SIG ) {
 	    if( n->pkt->pkt.onepass_sig->digest_algo ) {
-		gcry_md_enable( c->mfx.md,
-				n->pkt->pkt.onepass_sig->digest_algo );
+		gcry_md_enable( c->mfx.md, n->pkt->pkt.onepass_sig->digest_algo );
 		if( !any && n->pkt->pkt.onepass_sig->digest_algo
-						      == DIGEST_ALGO_MD5 )
+						      == GCRY_MD_MD5 )
 		    only_md5 = 1;
 		else
 		    only_md5 = 0;
@@ -399,32 +410,36 @@ proc_plaintext( CTX c, PACKET *pkt )
 		clearsig = 1;
 	}
     }
-    if( !any ) { /* no onepass sig packet: enable all standard algos */
-	gcry_md_enable( c->mfx.md, DIGEST_ALGO_RMD160 );
-	gcry_md_enable( c->mfx.md, DIGEST_ALGO_SHA1 );
-	gcry_md_enable( c->mfx.md, DIGEST_ALGO_MD5 );
+
+    if( !any && !opt.skip_verify ) {
+	/* no onepass sig packet: enable all standard algos */
+	gcry_md_enable( c->mfx.md, GCRY_MD_RMD160 );
+	gcry_md_enable( c->mfx.md, GCRY_MD_SHA1 );
+	gcry_md_enable( c->mfx.md, GCRY_MD_MD5 );
     }
-    if( only_md5 ) {
+    if( opt.pgp2_workarounds && only_md5 && !opt.skip_verify ) {
 	/* This is a kludge to work around a bug in pgp2.  It does only
 	 * catch those mails which are armored.  To catch the non-armored
 	 * pgp mails we could see whether there is the signature packet
 	 * in front of the plaintext.  If someone needs this, send me a patch.
 	 */
-	if( !(c->mfx.md2 = gcry_md_open( DIGEST_ALGO_MD5, 0)) )
+	c->mfx.md2 = gcry_md_open( GCRY_MD_MD5, 0);
+	if( !c->mfx.md2 )
 	    BUG();
     }
-  #if 0
-    #warning md_start_debug is enabled
-    md_start_debug( c->mfx.md, "verify" );
-  #endif
+    if ( DBG_HASHING ) {
+	gcry_md_start_debug( c->mfx.md, "verify" );
+	if ( c->mfx.md2  )
+	    gcry_md_start_debug( c->mfx.md2, "verify2" );
+    }
     rc = handle_plaintext( pt, &c->mfx, c->sigs_only, clearsig );
-    if( rc == G10ERR_CREATE_FILE && !c->sigs_only) {
+    if( rc == GPGERR_CREATE_FILE && !c->sigs_only) {
 	/* can't write output but we hash it anyway to
 	 * check the signature */
 	rc = handle_plaintext( pt, &c->mfx, 1, clearsig );
     }
     if( rc )
-	log_error( "handle plaintext failed: %s\n", g10_errstr(rc));
+	log_error( "handle plaintext failed: %s\n", gpg_errstr(rc));
     free_packet(pkt);
     c->last_was_session_key = 0;
 }
@@ -457,7 +472,7 @@ proc_compressed( CTX c, PACKET *pkt )
     else
 	rc = handle_compressed( c, zd, NULL, NULL );
     if( rc )
-	log_error("uncompressing failed: %s\n", g10_errstr(rc));
+	log_error("uncompressing failed: %s\n", gpg_errstr(rc));
     free_packet(pkt);
     c->last_was_session_key = 0;
 }
@@ -495,12 +510,17 @@ do_check_sig( CTX c, KBNODE node, int *is_selfsig )
 	 * in canonical mode ??? (calculating both modes???) */
 	if( c->mfx.md ) {
 	    md = gcry_md_copy( c->mfx.md );
-	    if( c->mfx.md2 )
+	    if( !md )
+		BUG();
+	    if( c->mfx.md2 ) {
 		md2 = gcry_md_copy( c->mfx.md2 );
+		if( !md2 )
+		    BUG();
+	    }
 	}
 	else { /* detached signature */
-	  log_debug("Do we really need this here?");
-	    md = gcry_md_open( 0, 0 ); /* signature_check() will enable the md*/
+	    log_debug("Do we really need this here?");
+	    md =  gcry_md_open( 0, 0 ); /* signature_check() will enable the md*/
 	    md2 = gcry_md_open( 0, 0 );
 	    if( !md || !md2 )
 		BUG();
@@ -514,16 +534,21 @@ do_check_sig( CTX c, KBNODE node, int *is_selfsig )
 	    || c->list->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
 	    return check_key_signature( c->list, node, is_selfsig );
 	}
+	else if( sig->sig_class == 0x20 ) {
+	    log_info(_("standalone revocation - "
+		       "use \"gpg --import\" to apply\n"));
+	    return GPGERR_NOT_PROCESSED;
+	}
 	else {
 	    log_error("invalid root packet for sigclass %02x\n",
 							sig->sig_class);
-	    return G10ERR_SIG_CLASS;
+	    return GPGERR_SIG_CLASS;
 	}
     }
     else
-	return G10ERR_SIG_CLASS;
+	return GPGERR_SIG_CLASS;
     rc = signature_check( sig, md );
-    if( rc == G10ERR_BAD_SIGN && md2 )
+    if( rc == GPGERR_BAD_SIGN && md2 )
 	rc = signature_check( sig, md2 );
     gcry_md_close(md);
     gcry_md_close(md2);
@@ -541,8 +566,12 @@ print_userid( PACKET *pkt )
 	printf("ERROR: unexpected packet type %d", pkt->pkttype );
 	return;
     }
-    print_string( stdout,  pkt->pkt.user_id->name, pkt->pkt.user_id->len,
-							opt.with_colons );
+    if( opt.with_colons )
+	print_string( stdout,  pkt->pkt.user_id->name,
+				pkt->pkt.user_id->len, ':');
+    else
+	print_utf8_string( stdout,  pkt->pkt.user_id->name,
+				     pkt->pkt.user_id->len );
 }
 
 
@@ -645,11 +674,13 @@ list_node( CTX c, KBNODE node )
 	    keyid_from_pk( pk, keyid );
 	    if( mainkey ) {
 		c->local_id = pk->local_id;
-		c->trustletter = query_trust_info( pk, NULL );
+		c->trustletter = opt.fast_list_mode?
+					   0 : query_trust_info( pk, NULL );
 	    }
-	    printf("%s:%c:%u:%d:%08lX%08lX:%s:%s:",
-		    mainkey? "pub":"sub",
-		    c->trustletter,
+	    printf("%s:", mainkey? "pub":"sub" );
+	    if( c->trustletter )
+		putchar( c->trustletter );
+	    printf(":%u:%d:%08lX%08lX:%s:%s:",
 		    nbits_from_pk( pk ),
 		    pk->pubkey_algo,
 		    (ulong)keyid[0],(ulong)keyid[1],
@@ -658,7 +689,7 @@ list_node( CTX c, KBNODE node )
 	    if( c->local_id )
 		printf("%lu", c->local_id );
 	    putchar(':');
-	    if( c->local_id )
+	    if( c->local_id && !opt.fast_list_mode )
 		putchar( get_ownertrust_info( c->local_id ) );
 	    putchar(':');
 	    if( node->next && node->next->pkt->pkttype == PKT_RING_TRUST) {
@@ -676,6 +707,7 @@ list_node( CTX c, KBNODE node )
 				      pubkey_letter( pk->pubkey_algo ),
 				      (ulong)keyid_from_pk( pk, NULL ),
 				      datestr_from_pk( pk )	);
+
 	if( mainkey ) {
 	    /* and now list all userids with their signatures */
 	    for( node = node->next; node; node = node->next ) {
@@ -718,6 +750,10 @@ list_node( CTX c, KBNODE node )
 		}
 	    }
 	}
+	else if( pk->expiredate ) { /* of subkey */
+	    printf(_(" [expires: %s]"), expirestr_from_pk( pk ) );
+	}
+
 	if( !any )
 	    putchar('\n');
 	if( !mainkey && opt.fingerprint > 1 )
@@ -807,8 +843,8 @@ list_node( CTX c, KBNODE node )
 	    fflush(stdout);
 	    switch( (rc2=do_check_sig( c, node, &is_selfsig )) ) {
 	      case 0:		       sigrc = '!'; break;
-	      case G10ERR_BAD_SIGN:    sigrc = '-'; break;
-	      case G10ERR_NO_PUBKEY:   sigrc = '?'; break;
+	      case GPGERR_BAD_SIGN:    sigrc = '-'; break;
+	      case GPGERR_NO_PUBKEY:   sigrc = '?'; break;
 	      default:		       sigrc = '%'; break;
 	    }
 	}
@@ -830,14 +866,15 @@ list_node( CTX c, KBNODE node )
 	    putchar(':');
 	    if( sigrc != ' ' )
 		putchar(sigrc);
-	    printf(":::%08lX%08lX:%s::::", (ulong)sig->keyid[0],
+	    printf("::%d:%08lX%08lX:%s::::", sig->pubkey_algo,
+					     (ulong)sig->keyid[0],
 		       (ulong)sig->keyid[1], datestr_from_sig(sig));
 	}
 	else
 	    printf("%c       %08lX %s   ",
 		    sigrc, (ulong)sig->keyid[1], datestr_from_sig(sig));
 	if( sigrc == '%' )
-	    printf("[%s] ", g10_errstr(rc2) );
+	    printf("[%s] ", gpg_errstr(rc2) );
 	else if( sigrc == '?' )
 	    ;
 	else if( is_selfsig ) {
@@ -847,10 +884,10 @@ list_node( CTX c, KBNODE node )
 	    if( opt.with_colons )
 		putchar(':');
 	}
-	else {
+	else if( !opt.fast_list_mode ) {
 	    p = get_user_id( sig->keyid, &n );
 	    print_string( stdout, p, n, opt.with_colons );
-	    m_free(p);
+	    gcry_free(p);
 	}
 	if( opt.with_colons )
 	    printf(":%02x:", sig->sig_class );
@@ -861,23 +898,26 @@ list_node( CTX c, KBNODE node )
 }
 
 
+
 int
 proc_packets( void *anchor, IOBUF a )
 {
     int rc;
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = gcry_xcalloc( 1, sizeof *c );
 
     c->anchor = anchor;
     rc = do_proc_packets( c, a );
-    m_free( c );
+    gcry_free( c );
     return rc;
 }
 
+
+
 int
 proc_signature_packets( void *anchor, IOBUF a,
 			STRLIST signedfiles, const char *sigfilename )
 {
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = gcry_xcalloc( 1, sizeof *c );
     int rc;
 
     c->anchor = anchor;
@@ -885,20 +925,20 @@ proc_signature_packets( void *anchor, IOBUF a,
     c->signed_data = signedfiles;
     c->sigfilename = sigfilename;
     rc = do_proc_packets( c, a );
-    m_free( c );
+    gcry_free( c );
     return rc;
 }
 
 int
 proc_encryption_packets( void *anchor, IOBUF a )
 {
-    CTX c = m_alloc_clear( sizeof *c );
+    CTX c = gcry_xcalloc( 1, sizeof *c );
     int rc;
 
     c->anchor = anchor;
     c->encrypt_only = 1;
     rc = do_proc_packets( c, a );
-    m_free( c );
+    gcry_free( c );
     return rc;
 }
 
@@ -906,7 +946,7 @@ proc_encryption_packets( void *anchor, IOBUF a )
 int
 do_proc_packets( CTX c, IOBUF a )
 {
-    PACKET *pkt = m_alloc( sizeof *pkt );
+    PACKET *pkt = gcry_xmalloc( sizeof *pkt );
     int rc=0;
     int any_data=0;
     int newpkt;
@@ -917,7 +957,7 @@ do_proc_packets( CTX c, IOBUF a )
 	any_data = 1;
 	if( rc ) {
 	    free_packet(pkt);
-	    if( rc == G10ERR_INVALID_PACKET )
+	    if( rc == GPGERR_INVALID_PACKET )
 		break;
 	    continue;
 	}
@@ -941,7 +981,7 @@ do_proc_packets( CTX c, IOBUF a )
 	      case PKT_PUBKEY_ENC:
 	      case PKT_ENCRYPTED:
 	      case PKT_ENCRYPTED_MDC:
-		rc = G10ERR_UNEXPECTED;
+		rc = GPGERR_UNEXPECTED;
 		goto leave;
 	      case PKT_SIGNATURE:   newpkt = add_signature( c, pkt ); break;
 	      case PKT_PLAINTEXT:   proc_plaintext( c, pkt ); break;
@@ -955,7 +995,7 @@ do_proc_packets( CTX c, IOBUF a )
 	      case PKT_PUBLIC_KEY:
 	      case PKT_SECRET_KEY:
 	      case PKT_USER_ID:
-		rc = G10ERR_UNEXPECTED;
+		rc = GPGERR_UNEXPECTED;
 		goto leave;
 	      case PKT_SIGNATURE:   newpkt = add_signature( c, pkt ); break;
 	      case PKT_SYMKEY_ENC:  proc_symkey_enc( c, pkt ); break;
@@ -999,13 +1039,13 @@ do_proc_packets( CTX c, IOBUF a )
 	if( newpkt == -1 )
 	    ;
 	else if( newpkt ) {
-	    pkt = m_alloc( sizeof *pkt );
+	    pkt = gcry_xmalloc( sizeof *pkt );
 	    init_packet(pkt);
 	}
 	else
 	    free_packet(pkt);
     }
-    if( rc == G10ERR_INVALID_PACKET )
+    if( rc == GPGERR_INVALID_PACKET )
 	write_status_text( STATUS_NODATA, "3" );
     if( any_data )
 	rc = 0;
@@ -1015,9 +1055,9 @@ do_proc_packets( CTX c, IOBUF a )
 
   leave:
     release_list( c );
-    m_free(c->dek);
+    gcry_free(c->dek);
     free_packet( pkt );
-    m_free( pkt );
+    gcry_free( pkt );
     free_md_filter_context( &c->mfx );
     return rc;
 }
@@ -1027,7 +1067,7 @@ static int
 check_sig_and_print( CTX c, KBNODE node )
 {
     PKT_signature *sig = node->pkt->pkt.signature;
-    const char *tstr;
+    const char *astr, *tstr;
     int rc;
 
     if( opt.skip_verify ) {
@@ -1036,16 +1076,16 @@ check_sig_and_print( CTX c, KBNODE node )
     }
 
     tstr = asctimestamp(sig->timestamp);
+    astr = gcry_pk_algo_name( sig->pubkey_algo );
     log_info(_("Signature made %.*s using %s key ID %08lX\n"),
-	    (int)strlen(tstr), tstr, gcry_pk_algo_name( sig->pubkey_algo ),
-						      (ulong)sig->keyid[1] );
+	    (int)strlen(tstr), tstr, astr? astr: "?", (ulong)sig->keyid[1] );
 
     rc = do_check_sig(c, node, NULL );
-    if( rc == G10ERR_NO_PUBKEY && opt.keyserver_name ) {
+    if( rc == GPGERR_NO_PUBKEY && opt.keyserver_name && opt.auto_key_retrieve) {
 	if( !hkp_ask_import( sig->keyid ) )
 	    rc = do_check_sig(c, node, NULL );
     }
-    if( !rc || rc == G10ERR_BAD_SIGN ) {
+    if( !rc || rc == GPGERR_BAD_SIGN ) {
 	KBNODE un, keyblock;
 	char *us;
 	int count=0;
@@ -1054,7 +1094,7 @@ check_sig_and_print( CTX c, KBNODE node )
 
 	us = get_long_user_id_string( sig->keyid );
 	write_status_text( rc? STATUS_BADSIG : STATUS_GOODSIG, us );
-	m_free(us);
+	gcry_free(us);
 
 	/* fixme: list only user ids which are valid and add information
 	 *	  about the trustworthiness of each user id, sort them.
@@ -1084,7 +1124,7 @@ check_sig_and_print( CTX c, KBNODE node )
 
 	if( !rc && is_status_enabled() ) {
 	    /* print a status response with the fingerprint */
-	    PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+	    PKT_public_key *pk = gcry_xcalloc( 1, sizeof *pk );
 
 	    if( !get_pubkey( pk, sig->keyid ) ) {
 		byte array[MAX_FINGERPRINT_LEN], *p;
@@ -1106,9 +1146,9 @@ check_sig_and_print( CTX c, KBNODE node )
 	if( !rc )
 	    rc = check_signatures_trust( sig );
 	if( rc )
-	    g10_errors_seen = 1;
+	    gpg_errors_seen = 1;
 	if( opt.batch && rc )
-	    g10_exit(1);
+	    gpg_exit(1);
     }
     else {
 	char buf[50];
@@ -1117,11 +1157,12 @@ check_sig_and_print( CTX c, KBNODE node )
 		     sig->pubkey_algo, sig->digest_algo,
 		     sig->sig_class, (ulong)sig->timestamp, rc );
 	write_status_text( STATUS_ERRSIG, buf );
-	if( rc == G10ERR_NO_PUBKEY ) {
+	if( rc == GPGERR_NO_PUBKEY ) {
 	    buf[16] = 0;
 	    write_status_text( STATUS_NO_PUBKEY, buf );
 	}
-	log_error(_("Can't check signature: %s\n"), g10_errstr(rc) );
+	if( rc != GPGERR_NOT_PROCESSED )
+	    log_error(_("Can't check signature: %s\n"), gpg_errstr(rc) );
     }
     return rc;
 }
@@ -1136,7 +1177,7 @@ proc_tree( CTX c, KBNODE node )
     KBNODE n1;
     int rc;
 
-    if( opt.list_packets )
+    if( opt.list_packets || opt.list_only )
 	return;
 
     c->local_id = 0;
@@ -1155,7 +1196,7 @@ proc_tree( CTX c, KBNODE node )
 	if( !c->have_data ) {
 	    free_md_filter_context( &c->mfx );
 	    /* prepare to create all requested message digests */
-	    if( !(c->mfx.md = gcry_md_open(0, 0)) )
+	    if ( !(c->mfx.md = gcry_md_open(0, 0)) )
 		BUG();
 
 	    /* fixme: why looking for the signature packet and not 1passpacket*/
@@ -1163,15 +1204,18 @@ proc_tree( CTX c, KBNODE node )
 		gcry_md_enable( c->mfx.md, n1->pkt->pkt.signature->digest_algo);
 	    }
 	    /* ask for file and hash it */
-	    if( c->sigs_only )
+	    if( c->sigs_only ) {
 		rc = hash_datafiles( c->mfx.md, NULL,
 				     c->signed_data, c->sigfilename,
 			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
-	    else
+	    }
+	    else {
 		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
-					    iobuf_get_fname(c->iobuf), 0 );
+						iobuf_get_fname(c->iobuf),
+			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
+	    }
 	    if( rc ) {
-		log_error("can't hash datafile: %s\n", g10_errstr(rc));
+		log_error("can't hash datafile: %s\n", gpg_errstr(rc));
 		return;
 	    }
 	}
@@ -1182,25 +1226,34 @@ proc_tree( CTX c, KBNODE node )
     else if( node->pkt->pkttype == PKT_SIGNATURE ) {
 	PKT_signature *sig = node->pkt->pkt.signature;
 
-	if( !c->have_data ) {
+	if( sig->sig_class != 0x00 && sig->sig_class != 0x01 )
+	    log_info(_("standalone signature of class 0x%02x\n"),
+						    sig->sig_class);
+	else if( !c->have_data ) {
 	    /* detached signature */
 	    free_md_filter_context( &c->mfx );
-	    if( !(c->mfx.md = gcry_md_open(sig->digest_algo, 0)) )
+	    c->mfx.md = gcry_md_open(sig->digest_algo, 0);
+	    if ( !c->mfx.md )
 		BUG();
-	    if( sig->digest_algo == DIGEST_ALGO_MD5
-		&& is_RSA( sig->pubkey_algo ) ) {
+	    if( !opt.pgp2_workarounds )
+		;
+	    else if( sig->digest_algo == GCRY_MD_MD5
+		     && is_RSA( sig->pubkey_algo ) ) {
 		/* enable a workaround for a pgp2 bug */
-		if( !(c->mfx.md2 = gcry_md_open( DIGEST_ALGO_MD5, 0 )) )
+		c->mfx.md2 = gcry_md_open( GCRY_MD_MD5, 0 );
+		if ( !c->mfx.md2 )
 		    BUG();
 	    }
-	    else if( sig->digest_algo == DIGEST_ALGO_SHA1
-		     && sig->pubkey_algo == PUBKEY_ALGO_DSA
+	    else if( sig->digest_algo == GCRY_MD_SHA1
+		     && sig->pubkey_algo == GCRY_PK_DSA
 		     && sig->sig_class == 0x01 ) {
 		/* enable the workaround also for pgp5 when the detached
 		 * signature has been created in textmode */
-		if( !(c->mfx.md2 = gcry_md_open( sig->digest_algo, 0 )) )
+		c->mfx.md2 = gcry_md_open( sig->digest_algo, 0 );
+		if ( !c->mfx.md2 )
 		    BUG();
 	    }
+	  #if 0 /* workaround disabled */
 	    /* Here we have another hack to work around a pgp 2 bug
 	     * It works by not using the textmode for detached signatures;
 	     * this will let the first signature check (on md) fail
@@ -1208,16 +1261,20 @@ proc_tree( CTX c, KBNODE node )
 	     * then produce the "correct" hash.  This is very, very ugly
 	     * hack but it may help in some cases (and break others)
 	     */
-	    if( c->sigs_only )
+		    /*	c->mfx.md2? 0 :(sig->sig_class == 0x01) */
+	  #endif
+	    if( c->sigs_only ) {
 		rc = hash_datafiles( c->mfx.md, c->mfx.md2,
 				     c->signed_data, c->sigfilename,
-				     c->mfx.md2? 0 :(sig->sig_class == 0x01) );
-	    else
+				     (sig->sig_class == 0x01) );
+	    }
+	    else {
 		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
-					 iobuf_get_fname(c->iobuf),
-			       c->mfx.md2? 0 :(sig->sig_class == 0x01) );
+						iobuf_get_fname(c->iobuf),
+						(sig->sig_class == 0x01) );
+	    }
 	    if( rc ) {
-		log_error("can't hash datafile: %s\n", g10_errstr(rc));
+		log_error("can't hash datafile: %s\n", gpg_errstr(rc));
 		return;
 	    }
 	}
