@@ -1,5 +1,5 @@
 /* openfile.c
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,9 +24,12 @@
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include "util.h"
-#include "memory.h"
+#include <gcrypt.h>
 #include "ttyio.h"
 #include "options.h"
 #include "main.h"
@@ -39,13 +42,16 @@
   #define SKELEXT ".skel"
 #endif
 
-
 #ifdef HAVE_DRIVE_LETTERS
   #define CMP_FILENAME(a,b) stricmp( (a), (b) )
 #else
   #define CMP_FILENAME(a,b) strcmp( (a), (b) )
 #endif
 
+#ifdef MKDIR_TAKES_ONE_ARG
+# undef mkdir
+# define mkdir(a,b) mkdir(a)
+#endif
 
 /* FIXME:  Implement opt.interactive. */
 
@@ -78,7 +84,6 @@ overwrite_filep( const char *fname )
 }
 
 
-
 /****************
  * Strip know extensions from iname and return a newly allocated
  * filename.  Return NULL if we can't do that.
@@ -89,14 +94,14 @@ make_outfile_name( const char *iname )
     size_t n;
 
     if( (!iname || (*iname=='-' && !iname[1]) ))
-	return m_strdup("-");
+	return gcry_xstrdup("-");
 
     n = strlen(iname);
     if( n > 4 && (    !CMP_FILENAME(iname+n-4,".gpg")
 		   || !CMP_FILENAME(iname+n-4,".pgp")
 		   || !CMP_FILENAME(iname+n-4,".sig")
 		   || !CMP_FILENAME(iname+n-4,".asc") ) ) {
-	char *buf = m_strdup( iname );
+	char *buf = gcry_xstrdup( iname );
 	buf[n-4] = 0;
 	return buf;
     }
@@ -127,19 +132,19 @@ ask_outfile_name( const char *name, size_t namelen )
 
     n = strlen(s) + namelen + 10;
     defname = name && namelen? make_printable_string( name, namelen, 0): NULL;
-    prompt = m_alloc(n);
+    prompt = gcry_xmalloc(n);
     if( defname )
 	sprintf(prompt, "%s [%s]: ", s, defname );
     else
 	sprintf(prompt, "%s: ", s );
     fname = cpr_get("openfile.askoutname", prompt );
     cpr_kill_prompt();
-    m_free(prompt);
+    gcry_free(prompt);
     if( !*fname ) {
-	m_free( fname ); fname = NULL;
+	gcry_free( fname ); fname = NULL;
 	fname = defname; defname = NULL;
     }
-    m_free(defname);
+    gcry_free(defname);
     return fname;
 }
 
@@ -161,7 +166,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
     if( (!iname || (*iname=='-' && !iname[1])) && !opt.outfile ) {
 	if( !(*a = iobuf_create(NULL)) ) {
 	    log_error(_("%s: can't open: %s\n"), "[stdout]", strerror(errno) );
-	    rc = G10ERR_CREATE_FILE;
+	    rc = GPGERR_CREATE_FILE;
 	}
 	else if( opt.verbose )
 	    log_info(_("writing to stdout\n"));
@@ -176,7 +181,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    name = opt.outfile;
 	else {
 	  #ifdef USE_ONLY_8DOT3
-	    /* It is quite common for DOS system to have only one dot in a
+	    /* It is quite common DOS system to have only one dot in a
 	     * a filename So if we have something like this, we simple
 	     * replace the suffix execpt in cases where the suffix is
 	     * larger than 3 characters and not the same as.
@@ -187,7 +192,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    const char *newsfx = mode==1 ? ".asc" :
 				 mode==2 ? ".sig" : ".gpg";
 
-	    buf = m_alloc(strlen(iname)+4+1);
+	    buf = gcry_xmalloc(strlen(iname)+4+1);
 	    strcpy(buf,iname);
 	    dot = strchr(buf, '.' );
 	    if( dot && dot > buf && dot[1] && strlen(dot) <= 4
@@ -195,11 +200,11 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 		strcpy(dot, newsfx );
 	    }
 	    else if( dot && !dot[1] ) /* don't duplicate a dot */
-		strcat( dot, newsfx+1 );
+		strcpy( dot, newsfx+1 );
 	    else
 		strcat( buf, newsfx );
 	  #else
-	    buf = m_alloc(strlen(iname)+4+1);
+	    buf = gcry_xmalloc(strlen(iname)+4+1);
 	    strcpy(stpcpy(buf,iname), mode==1 ? ".asc" :
 				      mode==2 ? ".sig" : ".gpg");
 	  #endif
@@ -209,14 +214,14 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	if( overwrite_filep( name ) ) {
 	    if( !(*a = iobuf_create( name )) ) {
 		log_error(_("%s: can't create: %s\n"), name, strerror(errno) );
-		rc = G10ERR_CREATE_FILE;
+		rc = GPGERR_CREATE_FILE;
 	    }
 	    else if( opt.verbose )
 		log_info(_("writing to `%s'\n"), name );
 	}
 	else
-	    rc = G10ERR_FILE_EXISTS;
-	m_free(buf);
+	    rc = GPGERR_FILE_EXISTS;
+	gcry_free(buf);
     }
     return rc;
 }
@@ -238,12 +243,12 @@ open_sigfile( const char *iname )
 	if( len > 4 && ( !strcmp(iname + len - 4, ".sig")
 			|| !strcmp(iname + len - 4, ".asc")) ) {
 	    char *buf;
-	    buf = m_strdup(iname);
+	    buf = gcry_xstrdup(iname);
 	    buf[len-4] = 0 ;
 	    a = iobuf_open( buf );
 	    if( opt.verbose )
 		log_info(_("assuming signed data in `%s'\n"), buf );
-	    m_free(buf);
+	    gcry_free(buf);
 	}
     }
     return a;
@@ -253,7 +258,7 @@ open_sigfile( const char *iname )
 /****************
  * Copy the option file skeleton to the given directory.
  */
-void
+static void
 copy_options_file( const char *destdir )
 {
     const char *datadir = GNUPG_DATADIR;
@@ -265,12 +270,12 @@ copy_options_file( const char *destdir )
     if( opt.dry_run )
 	return;
 
-    fname = m_alloc( strlen(datadir) + strlen(destdir) + 15 );
+    fname = gcry_xmalloc( strlen(datadir) + strlen(destdir) + 15 );
     strcpy(stpcpy(fname, datadir), "/options" SKELEXT );
     src = fopen( fname, "r" );
     if( !src ) {
 	log_error(_("%s: can't open: %s\n"), fname, strerror(errno) );
-	m_free(fname);
+	gcry_free(fname);
 	return;
     }
     strcpy(stpcpy(fname, destdir), "/options" );
@@ -278,7 +283,7 @@ copy_options_file( const char *destdir )
     if( !dst ) {
 	log_error(_("%s: can't create: %s\n"), fname, strerror(errno) );
 	fclose( src );
-	m_free(fname);
+	gcry_free(fname);
 	return;
     }
 
@@ -293,6 +298,26 @@ copy_options_file( const char *destdir )
     fclose( dst );
     fclose( src );
     log_info(_("%s: new options file created\n"), fname );
-    m_free(fname);
+    gcry_free(fname);
+}
+
+
+void
+try_make_homedir( const char *fname )
+{
+    if( opt.dry_run )
+	return;
+    if( strlen(fname) >= 7
+	&& !strcmp(fname+strlen(fname)-7, "/.gnupg" ) ) {
+	if( mkdir( fname, S_IRUSR|S_IWUSR|S_IXUSR ) )
+	    log_fatal( _("%s: can't create directory: %s\n"),
+					fname,	strerror(errno) );
+	else if( !opt.quiet )
+	    log_info( _("%s: directory created\n"), fname );
+	copy_options_file( fname );
+	log_info(_("you have to start GnuPG again, "
+		   "so it can read the new options file\n") );
+	gpg_exit(1);
+    }
 }
 
