@@ -1,5 +1,5 @@
 /* packet.h - packet read/write stuff
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -18,15 +18,23 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef G10_PACKET_H
-#define G10_PACKET_H
+#ifndef GPG_PACKET_H
+#define GPG_PACKET_H
 
 #include "types.h"
 #include "iobuf.h"
-#include "mpi.h"
-#include "cipher.h"
 #include "filter.h"
 
+#ifndef DID_MPI_TYPEDEF
+  typedef struct gcry_mpi *MPI;
+  #define DID_MPI_TYPEDEF
+#endif
+
+#define GNUPG_MAX_NPKEY  4
+#define GNUPG_MAX_NSKEY  6
+#define GNUPG_MAX_NSIG	 2
+#define GNUPG_MAX_NENC	 2
+
 #define DEBUG_PARSE_PACKET 1
 
 typedef enum {
@@ -46,14 +54,16 @@ typedef enum {
 	PKT_USER_ID	  =13, /* user id packet */
 	PKT_PUBLIC_SUBKEY =14, /* public subkey (OpenPGP) */
 	PKT_OLD_COMMENT   =16, /* comment packet from an OpenPGP draft */
+	PKT_PHOTO_ID	  =17, /* PGP's photo ID */
+	PKT_ENCRYPTED_MDC =18, /* integrity protected encrypted data */
+	PKT_MDC 	  =19, /* manipulaion detection code packet */
 	PKT_COMMENT	  =61, /* new comment packet (private) */
-	PKT_ENCRYPTED_MDC =62, /* test: encrypted data with MDC */
 } pkttype_t;
 
 typedef struct packet_struct PACKET;
 
 typedef struct {
-    byte mode;
+    int  mode;
     byte hash_algo;
     byte salt[8];
     u32  count;
@@ -72,7 +82,7 @@ typedef struct {
     byte    version;
     byte    pubkey_algo;    /* algorithm used for public key scheme */
     byte    throw_keyid;
-    MPI     data[PUBKEY_MAX_NENC];
+    MPI     data[GNUPG_MAX_NENC];
 } PKT_pubkey_enc;
 
 
@@ -97,12 +107,12 @@ typedef struct {
     byte    version;
     byte    sig_class;	    /* sig classification, append for MD calculation*/
     byte    pubkey_algo;    /* algorithm used for public key scheme */
-			    /* (PUBKEY_ALGO_xxx) */
+			    /* (GCRY_PK_xxx) */
     byte digest_algo;	    /* algorithm used for digest (DIGEST_ALGO_xxxx) */
     byte *hashed_data;	    /* all subpackets with hashed  data (v4 only) */
     byte *unhashed_data;    /* ditto for unhashed data */
     byte digest_start[2];   /* first 2 bytes of the digest */
-    MPI  data[PUBKEY_MAX_NSIG];
+    MPI  data[GNUPG_MAX_NSIG];
 } PKT_signature;
 
 
@@ -122,7 +132,7 @@ typedef struct {
     ulong   local_id;	    /* internal use, valid if > 0 */
     u32     keyid[2];	    /* calculated by keyid_from_pk() */
     byte    *namehash;	    /* if != NULL: found by this name */
-    MPI     pkey[PUBKEY_MAX_NPKEY];
+    MPI     pkey[GNUPG_MAX_NPKEY];
 } PKT_public_key;
 
 typedef struct {
@@ -143,7 +153,7 @@ typedef struct {
 	byte ivlen;  /* used length of the iv */
 	byte iv[16]; /* initialization vector for CFB mode */
     } protect;
-    MPI skey[PUBKEY_MAX_NSKEY];
+    MPI skey[GNUPG_MAX_NSKEY];
     u16 csum;		/* checksum */
 } PKT_secret_key;
 
@@ -154,7 +164,11 @@ typedef struct {
 } PKT_comment;
 
 typedef struct {
+    ulong stored_at;	  /* the stream offset where it was stored
+			   * by build-packet */
     int  len;		  /* length of the name */
+    char *photo;	  /* if this is not NULL, the packet is a photo ID */
+    int photolen;	  /* and the length of the photo */
     char name[1];
 } PKT_user_id;
 
@@ -168,10 +182,14 @@ typedef struct {
 typedef struct {
     u32  len;		  /* length of encrypted data */
     byte new_ctb;	  /* uses a new CTB */
-    byte mdc_method;	  /* test: > 0: this is is an encrypted_mdc packet */
+    byte mdc_method;	  /* > 0: integrity protected encrypted data packet */
     IOBUF buf;		  /* IOBUF reference */
 } PKT_encrypted;
 
+typedef struct {
+    byte hash[20];
+} PKT_mdc;
+
 typedef struct {
     unsigned int trustval;
 } PKT_ring_trust;
@@ -202,6 +220,7 @@ struct packet_struct {
 	PKT_user_id	*user_id;	/* PKT_USER_ID */
 	PKT_compressed	*compressed;	/* PKT_COMPRESSED */
 	PKT_encrypted	*encrypted;	/* PKT_ENCRYPTED[_MDC] */
+	PKT_mdc 	*mdc;		/* PKT_MDC */
 	PKT_ring_trust	*ring_trust;	/* PKT_RING_TRUST */
 	PKT_plaintext	*plaintext;	/* PKT_PLAINTEXT */
     } pkt;
@@ -236,6 +255,7 @@ typedef enum {
     SIGSUBPKT_POLICY	   =26, /* policy URL */
     SIGSUBPKT_KEY_FLAGS    =27, /* key flags */
     SIGSUBPKT_SIGNERS_UID  =28, /* signer's user id */
+    SIGSUBPKT_REVOC_REASON =29, /* reason for revocation */
     SIGSUBPKT_PRIV_ADD_SIG =101,/* signatur is also valid for this uid */
 
     SIGSUBPKT_FLAG_CRITICAL=128
@@ -329,7 +349,6 @@ int handle_compressed( void *ctx, PKT_compressed *cd,
 
 /*-- encr-data.c --*/
 int decrypt_data( void *ctx, PKT_encrypted *ed, DEK *dek );
-int encrypt_data( PKT_encrypted *ed, DEK *dek );
 
 /*-- plaintext.c --*/
 int handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
@@ -351,4 +370,4 @@ int make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 /*-- keygen.c --*/
 PKT_user_id *generate_user_id(void);
 
-#endif /*G10_PACKET_H*/
+#endif /*GPG_PACKET_H*/
