@@ -1,5 +1,5 @@
 /* plaintext.c -  process an plaintext packet
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,8 +24,12 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
+#ifdef HAVE_DOSISH_SYSTEM
+  #include <fcntl.h> /* for setmode() */
+#endif
+
 #include "util.h"
-#include "memory.h"
+#include <gcrypt.h>
 #include "options.h"
 #include "packet.h"
 #include "ttyio.h"
@@ -56,7 +60,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     if( nooutput )
 	;
     else if( opt.outfile ) {
-	fname = m_alloc( strlen( opt.outfile ) + 1);
+	fname = gcry_xmalloc( strlen( opt.outfile ) + 1);
 	strcpy(fname, opt.outfile );
     }
     else if( pt->namelen == 8 && !memcmp( pt->name, "_CONSOLE", 8 ) ) {
@@ -68,7 +72,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	if( !fname )
 	    fname = ask_outfile_name( pt->name, pt->namelen );
 	if( !fname ) {
-	    rc = G10ERR_CREATE_FILE;
+	    rc = GPGERR_CREATE_FILE;
 	    goto leave;
 	}
     }
@@ -81,9 +85,12 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     else if( !*fname || (*fname=='-' && !fname[1])) {
 	/* no filename or "-" given; write to stdout */
 	fp = stdout;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
     }
     else if( !overwrite_filep( fname ) ) {
-	rc = G10ERR_CREATE_FILE;
+	rc = GPGERR_CREATE_FILE;
 	goto leave;
     }
 
@@ -91,7 +98,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	;
     else if( !(fp = fopen(fname,"wb")) ) {
 	log_error("Error creating `%s': %s\n", fname, strerror(errno) );
-	rc = G10ERR_CREATE_FILE;
+	rc = GPGERR_CREATE_FILE;
 	goto leave;
     }
 
@@ -102,33 +109,35 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		if( (c = iobuf_get(pt->buf)) == -1 ) {
 		    log_error("Problem reading source (%u bytes remaining)\n",
 			      (unsigned)pt->len);
-		    rc = G10ERR_READ_FILE;
+		    rc = GPGERR_READ_FILE;
 		    goto leave;
 		}
 		if( mfx->md )
 		    gcry_md_putc(mfx->md, c );
-		if( c == '\r' )
-		    continue; /* fixme: this hack might be too simple */
+	      #ifndef HAVE_DOSISH_SYSTEM
+		if( c == '\r' )  /* convert to native line ending */
+		    continue;	 /* fixme: this hack might be too simple */
+	      #endif
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
-			rc = G10ERR_WRITE_FILE;
+			rc = GPGERR_WRITE_FILE;
 			goto leave;
 		    }
 		}
 	    }
 	}
 	else { /* binary mode */
-	    byte *buffer = m_alloc( 32768 );
+	    byte *buffer = gcry_xmalloc( 32768 );
 	    while( pt->len ) {
 		int len = pt->len > 32768 ? 32768 : pt->len;
 		len = iobuf_read( pt->buf, buffer, len );
 		if( len == -1 ) {
 		    log_error("Problem reading source (%u bytes remaining)\n",
 			      (unsigned)pt->len);
-		    rc = G10ERR_READ_FILE;
-		    m_free( buffer );
+		    rc = GPGERR_READ_FILE;
+		    gcry_free( buffer );
 		    goto leave;
 		}
 		if( mfx->md )
@@ -137,14 +146,14 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    if( fwrite( buffer, 1, len, fp ) != len ) {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
-			rc = G10ERR_WRITE_FILE;
-			m_free( buffer );
+			rc = GPGERR_WRITE_FILE;
+			gcry_free( buffer );
 			goto leave;
 		    }
 		}
 		pt->len -= len;
 	    }
-	    m_free( buffer );
+	    gcry_free( buffer );
 	}
     }
     else if( !clearsig ) {
@@ -152,27 +161,29 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    while( (c = iobuf_get(pt->buf)) != -1 ) {
 		if( mfx->md )
 		    gcry_md_putc(mfx->md, c );
+	      #ifndef HAVE_DOSISH_SYSTEM
 		if( convert && c == '\r' )
 		    continue; /* fixme: this hack might be too simple */
+	      #endif
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
-			rc = G10ERR_WRITE_FILE;
+			rc = GPGERR_WRITE_FILE;
 			goto leave;
 		    }
 		}
 	    }
 	}
 	else { /* binary mode */
-	    byte *buffer = m_alloc( 32768 );
+	    byte *buffer = gcry_xmalloc( 32768 );
 	    int eof;
 	    for( eof=0; !eof; ) {
 		/* Why do we check for len < 32768:
-		 * If we won we would practically read 2 EOFS but
+		 * If we won't, we would practically read 2 EOFs but
 		 * the first one has already popped the block_filter
 		 * off and therefore we don't catch the boundary.
-		 * Always assume EOF if iobuf_read returns less bytes
+		 * So, always assume EOF if iobuf_read returns less bytes
 		 * then requested */
 		int len = iobuf_read( pt->buf, buffer, 32768 );
 		if( len == -1 )
@@ -185,13 +196,13 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    if( fwrite( buffer, 1, len, fp ) != len ) {
 			log_error("Error writing to `%s': %s\n",
 				  fname, strerror(errno) );
-			rc = G10ERR_WRITE_FILE;
-			m_free( buffer );
+			rc = GPGERR_WRITE_FILE;
+			gcry_free( buffer );
 			goto leave;
 		    }
 		}
 	    }
-	    m_free( buffer );
+	    gcry_free( buffer );
 	}
 	pt->buf = NULL;
     }
@@ -203,7 +214,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		if( putc( c, fp ) == EOF ) {
 		    log_error("Error writing to `%s': %s\n",
 						fname, strerror(errno) );
-		    rc = G10ERR_WRITE_FILE;
+		    rc = GPGERR_WRITE_FILE;
 		    goto leave;
 		}
 	    }
@@ -217,6 +228,8 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    if( !state ) {
 		if( c == '\r'  )
 		    state = 1;
+		else if( c == '\n'  )
+		    state = 2;
 		else
 		    gcry_md_putc(mfx->md, c );
 	    }
@@ -240,7 +253,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     if( fp && fp != stdout && fclose(fp) ) {
 	log_error("Error closing `%s': %s\n", fname, strerror(errno) );
 	fp = NULL;
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
 	goto leave;
     }
     fp = NULL;
@@ -248,7 +261,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
   leave:
     if( fp && fp != stdout )
 	fclose(fp);
-    m_free(fname);
+    gcry_free(fname);
     return rc;
 }
 
@@ -310,12 +323,12 @@ ask_for_detached_datafile( GCRY_MD_HD md, GCRY_MD_HD md2,
 	int any=0;
 	tty_printf(_("Detached signature.\n"));
 	do {
-	    m_free(answer);
+	    gcry_free(answer);
 	    answer = cpr_get("detached_signature.filename",
 			   _("Please enter name of data file: "));
 	    cpr_kill_prompt();
 	    if( any && !*answer ) {
-		rc = G10ERR_READ_FILE;
+		rc = GPGERR_READ_FILE;
 		goto leave;
 	    }
 	    fp = iobuf_open(answer);
@@ -325,7 +338,7 @@ ask_for_detached_datafile( GCRY_MD_HD md, GCRY_MD_HD md2,
 	    }
 	    else if( !fp ) {
 		log_error("can't open `%s': %s\n", answer, strerror(errno) );
-		rc = G10ERR_READ_FILE;
+		rc = GPGERR_READ_FILE;
 		goto leave;
 	    }
 	} while( !fp );
@@ -342,7 +355,7 @@ ask_for_detached_datafile( GCRY_MD_HD md, GCRY_MD_HD md2,
 
 
   leave:
-    m_free(answer);
+    gcry_free(answer);
     return rc;
 }
 
@@ -380,7 +393,7 @@ hash_datafiles( GCRY_MD_HD md, GCRY_MD_HD md2, STRLIST files,
 						print_fname_stdin(sl->d));
 	    if( !files )
 		free_strlist(sl);
-	    return G10ERR_OPEN_FILE;
+	    return GPGERR_OPEN_FILE;
 	}
 	do_hash( md, md2, fp, textmode );
 	iobuf_close(fp);
