@@ -1,5 +1,5 @@
 /* seskey.c -  make sesssion keys etc.
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,8 +26,6 @@
 
 #include <gcrypt.h>
 #include "util.h"
-#include "cipher.h"
-#include "mpi.h"
 #include "main.h"
 #include "i18n.h"
 
@@ -51,7 +49,7 @@ make_session_key( DEK *dek )
 	BUG();
     }
 
-    randomize_buffer( dek->key, dek->keylen, 1 );
+    gcry_randomize( dek->key, dek->keylen, GCRY_STRONG_RANDOM );
     for(i=0; i < 16; i++ ) {
 	rc = gcry_cipher_setkey( chd, dek->key, dek->keylen );
 	if( !rc ) {
@@ -62,7 +60,7 @@ make_session_key( DEK *dek )
 	    BUG();
 	log_info(_("weak key created - retrying\n") );
 	/* Renew the session key until we get a non-weak key. */
-	randomize_buffer( dek->key, dek->keylen, 1 );
+	gcry_randomize( dek->key, dek->keylen, GCRY_STRONG_RANDOM );
     }
     log_fatal(_(
 	    "cannot avoid weak key for symmetric cipher; tried %d times!\n"),
@@ -110,13 +108,13 @@ encode_session_key( DEK *dek, unsigned nbits )
     for( p = dek->key, i=0; i < dek->keylen; i++ )
 	csum += *p++;
 
-    frame = m_alloc_secure( nframe );
+    frame = gcry_xmalloc_secure( nframe );
     n = 0;
     frame[n++] = 0;
     frame[n++] = 2;
     i = nframe - 6 - dek->keylen;
     assert( i > 0 );
-    p = get_random_bits( i*8, 1, 1 );
+    p = gcry_random_bytes_secure( i, GCRY_STRONG_RANDOM );
     /* replace zero bytes by new values */
     for(;;) {
 	int j, k;
@@ -129,14 +127,14 @@ encode_session_key( DEK *dek, unsigned nbits )
 	if( !k )
 	    break; /* okay: no zero bytes */
 	k += k/128; /* better get some more */
-	pp = get_random_bits( k*8, 1, 1);
+	pp = gcry_random_bytes_secure( k, GCRY_STRONG_RANDOM);
 	for(j=0; j < i && k ; j++ )
 	    if( !p[j] )
 		p[j] = pp[--k];
-	m_free(pp);
+	gcry_free(pp);
     }
     memcpy( frame+n, p, i );
-    m_free(p);
+    gcry_free(p);
     n += i;
     frame[n++] = 0;
     frame[n++] = dek->algo;
@@ -144,16 +142,17 @@ encode_session_key( DEK *dek, unsigned nbits )
     frame[n++] = csum >>8;
     frame[n++] = csum;
     assert( n == nframe );
-    a = mpi_alloc_secure( (nframe+BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB );
-    mpi_set_buffer( a, frame, nframe, 0 );
-    m_free(frame);
+    if( gcry_mpi_scan( &a, GCRYMPI_FMT_USG, frame, &nframe ) )
+	BUG();
+    gcry_free(frame);
+
     return a;
 }
 
 
 static MPI
 do_encode_md( GCRY_MD_HD md, int algo, size_t len, unsigned nbits,
-				       const byte *asn, size_t asnlen )
+	      const byte *asn, size_t asnlen, int v3compathack )
 {
     int nframe = (nbits+7) / 8;
     byte *frame;
@@ -166,15 +165,15 @@ do_encode_md( GCRY_MD_HD md, int algo, size_t len, unsigned nbits,
 
     /* We encode the MD in this way:
      *
-     *	   0  A PAD(n bytes)   0  ASN(asnlen bytes)  MD(len bytes)
+     *	   0  1 PAD(n bytes)   0  ASN(asnlen bytes)  MD(len bytes)
      *
      * PAD consists of FF bytes.
      */
-    frame = gcry_md_is_secure(md)? m_alloc_secure( nframe )
-				 : m_alloc( nframe );
+    frame = gcry_md_is_secure(md)? gcry_xmalloc_secure( nframe )
+				 : gcry_xmalloc( nframe );
     n = 0;
     frame[n++] = 0;
-    frame[n++] = algo;
+    frame[n++] = v3compathack? algo : 1; /* block type */
     i = nframe - len - asnlen -3 ;
     assert( i > 1 );
     memset( frame+n, 0xff, i ); n += i;
@@ -182,29 +181,31 @@ do_encode_md( GCRY_MD_HD md, int algo, size_t len, unsigned nbits,
     memcpy( frame+n, asn, asnlen ); n += asnlen;
     memcpy( frame+n, gcry_md_read(md, algo), len ); n += len;
     assert( n == nframe );
-    a = gcry_md_is_secure(md)?
-	 mpi_alloc_secure( (nframe+BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB )
-	 : mpi_alloc( (nframe+BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB );
-    mpi_set_buffer( a, frame, nframe, 0 );
-    m_free(frame);
+    if( gcry_mpi_scan( &a, GCRYMPI_FMT_USG, frame, &nframe ) )
+	BUG();
+    gcry_free(frame);
     return a;
 }
 
 
+/****************
+ * Encode a message digest into an MPI.
+ * v3compathack is used to work around a bug in old GnuPG versions
+ * which did put the algo identifier inseatd of the block type 1 into
+ * the encoded value.  setting this vare force the old behaviour.
+ */
 MPI
-encode_md_value( int pubkey_algo, GCRY_MD_HD md, int hash_algo, unsigned nbits )
+encode_md_value( int pubkey_algo, GCRY_MD_HD md, int hash_algo,
+		 unsigned nbits, int v3compathack )
 {
     int algo = hash_algo? hash_algo : gcry_md_get_algo(md);
     MPI frame;
 
-    if( pubkey_algo == PUBKEY_ALGO_DSA ) {
-	frame = gcry_md_is_secure(md)? mpi_alloc_secure(
-				(gcry_md_get_algo_dlen(hash_algo)
-				 +BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB )
-				: mpi_alloc((gcry_md_get_algo_dlen(hash_algo)
-				 +BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB );
-	mpi_set_buffer( frame, gcry_md_read(md, hash_algo),
-			       gcry_md_get_algo_dlen(hash_algo), 0 );
+    if( pubkey_algo == GCRY_PK_DSA ) {
+	size_t n = gcry_md_get_algo_dlen(hash_algo);
+	if( gcry_mpi_scan( &frame, GCRYMPI_FMT_USG,
+				   gcry_md_read(md, hash_algo), &n ) )
+	    BUG();
     }
     else {
 	byte *asn;
@@ -213,13 +214,14 @@ encode_md_value( int pubkey_algo, GCRY_MD_HD md, int hash_algo, unsigned nbits )
 	if( gcry_md_algo_info( algo, GCRYCTL_GET_ASNOID, NULL, &asnlen ) )
 	    log_fatal("can't get OID of algo %d: %s\n",
 					    algo, gcry_strerror(-1));
-	asn = m_alloc( asnlen );
+	asn = gcry_xmalloc( asnlen );
 	if( gcry_md_algo_info( algo, GCRYCTL_GET_ASNOID, asn, &asnlen ) )
 	    BUG();
 	frame = do_encode_md( md, algo, gcry_md_get_algo_dlen( algo ),
-						     nbits, asn, asnlen );
-	m_free( asn );
+					 nbits, asn, asnlen, v3compathack );
+	gcry_free( asn );
     }
     return frame;
 }
 
+
