@@ -1,5 +1,5 @@
 /* sig-check.c -  Check a signature
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -27,12 +27,11 @@
 #include <gcrypt.h>
 #include "util.h"
 #include "packet.h"
-#include "memory.h"
-#include "mpi.h"
 #include "keydb.h"
 #include "main.h"
 #include "status.h"
 #include "i18n.h"
+#include "options.h"
 
 struct cmp_help_context_s {
     PKT_signature *sig;
@@ -41,9 +40,78 @@ struct cmp_help_context_s {
 
 
 static int do_signature_check( PKT_signature *sig, GCRY_MD_HD digest,
-						      u32 *r_expire );
+					 u32 *r_expiredate, int *r_expired );
 static int do_check( PKT_public_key *pk, PKT_signature *sig,
-						GCRY_MD_HD digest );
+					 GCRY_MD_HD digest, int *r_expired );
+
+
+
+/****************
+ * Emulate our old PK interface here - sometime in the future we might
+ * change the internal design to directly fit to libgcrypt.
+ */
+static int
+pk_verify( int algo, MPI hash, MPI *data, MPI *pkey,
+	   int (*cmp)(void *, MPI), void *opaque )
+{
+    GCRY_SEXP s_sig, s_hash, s_pkey;
+    int rc;
+
+    /* forget about cmp and opaque - we never used it */
+
+    /* make a sexp from pkey */
+    if( algo == GCRY_PK_DSA ) {
+	rc = gcry_sexp_build ( &s_pkey, NULL,
+			      "(public-key(dsa(p%m)(q%m)(g%m)(y%m)))",
+				  pkey[0], pkey[1], pkey[2], pkey[3] );
+    }
+    else if( algo == GCRY_PK_ELG || algo == GCRY_PK_ELG_E ) {
+	rc = gcry_sexp_build ( &s_pkey, NULL,
+			      "(public-key(elg(p%m)(g%m)(y%m)))",
+				  pkey[0], pkey[1], pkey[2] );
+    }
+    else if( algo == GCRY_PK_RSA ) {
+	rc = gcry_sexp_build ( &s_pkey, NULL,
+			      "(public-key(rsa(n%m)(e%m)))",
+				  pkey[0], pkey[1] );
+    }
+    else
+	return GPGERR_PUBKEY_ALGO;
+
+    if ( rc )
+	BUG ();
+
+    /* put hash into a S-Exp s_hash */
+    if ( gcry_sexp_build( &s_hash, NULL, "%m", hash ) )
+	BUG ();
+
+    /* put data into a S-Exp s_sig */
+    if( algo == GCRY_PK_DSA ) {
+	rc = gcry_sexp_build ( &s_sig, NULL,
+			      "(sig-val(dsa(r%m)(s%m)))", data[0], data[1] );
+    }
+    else if( algo == GCRY_PK_ELG || algo == GCRY_PK_ELG_E ) {
+	rc = gcry_sexp_build ( &s_sig, NULL,
+			      "(sig-val(elg(r%m)(s%m)))", data[0], data[1] );
+    }
+    else if( algo == GCRY_PK_RSA ) {
+	rc = gcry_sexp_build ( &s_sig, NULL,
+			      "(sig-val(rsa(s%m)))", data[0] );
+    }
+    else
+	BUG();
+
+    if ( rc )
+	BUG ();
+
+
+    rc = gcry_pk_verify( s_sig, s_hash, s_pkey );
+    gcry_sexp_release( s_sig );
+    gcry_sexp_release( s_hash );
+    gcry_sexp_release( s_pkey );
+    return rc;
+}
+
 
 
 /****************
@@ -55,24 +123,26 @@ int
 signature_check( PKT_signature *sig, GCRY_MD_HD digest )
 {
     u32 dummy;
-    return do_signature_check( sig, digest, &dummy );
+    int dum2;
+    return do_signature_check( sig, digest, &dummy, &dum2 );
 }
 
 static int
-do_signature_check( PKT_signature *sig, GCRY_MD_HD digest, u32 *r_expire )
+do_signature_check( PKT_signature *sig, GCRY_MD_HD digest,
+					u32 *r_expiredate, int *r_expired )
 {
-    PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+    PKT_public_key *pk = gcry_xcalloc( 1, sizeof *pk );
     int rc=0;
 
     if( is_RSA(sig->pubkey_algo) )
 	write_status(STATUS_RSA_OR_IDEA);
 
-    *r_expire = 0;
+    *r_expiredate = 0;
     if( get_pubkey( pk, sig->keyid ) )
-	rc = G10ERR_NO_PUBKEY;
+	rc = GPGERR_NO_PUBKEY;
     else {
-	*r_expire = pk->expiredate;
-	rc = do_check( pk, sig, digest );
+	*r_expiredate = pk->expiredate;
+	rc = do_check( pk, sig, digest, r_expired );
     }
 
     free_public_key( pk );
@@ -90,7 +160,7 @@ do_signature_check( PKT_signature *sig, GCRY_MD_HD digest, u32 *r_expire )
 	int i, nsig = pubkey_get_nsig( sig->pubkey_algo );
 	byte *p, *buffer;
 
-	if( !(md = gcry_md_open( DIGEST_ALGO_RMD160, 0)) )
+	if( !(md = gcry_md_open( GCRY_MD_RMD160, 0)) )
 	    BUG();
 	gcry_md_putc( digest, sig->pubkey_algo );
 	gcry_md_putc( digest, sig->digest_algo );
@@ -99,22 +169,23 @@ do_signature_check( PKT_signature *sig, GCRY_MD_HD digest, u32 *r_expire )
 	gcry_md_putc( digest, (a >>  8) & 0xff );
 	gcry_md_putc( digest,  a	& 0xff );
 	for(i=0; i < nsig; i++ ) {
-	    unsigned n = mpi_get_nbits( sig->data[i]);
+	    size_t n = gcry_mpi_get_nbits( sig->data[i]);
 
 	    gcry_md_putc( md, n>>8);
 	    gcry_md_putc( md, n );
-	    p = mpi_get_buffer( sig->data[i], &n, NULL );
+	    if( gcry_mpi_aprint( GCRYMPI_FMT_USG, &p, &n, sig->data[i] ) )
+		BUG();
 	    gcry_md_write( md, p, n );
-	    m_free(p);
+	    gcry_free(p);
 	}
 	gcry_md_final( md );
 	p = make_radix64_string( gcry_md_read( md, 0 ), 20 );
-	buffer = m_alloc( strlen(p) + 60 );
+	buffer = gcry_xmalloc( strlen(p) + 60 );
 	sprintf( buffer, "%s %s %lu",
 		 p, strtimestamp( sig->timestamp ), (ulong)sig->timestamp );
 	write_status_text( STATUS_SIG_ID, buffer );
-	m_free(buffer);
-	m_free(p);
+	gcry_free(buffer);
+	gcry_free(p);
 	gcry_md_close(md);
     }
 
@@ -122,94 +193,6 @@ do_signature_check( PKT_signature *sig, GCRY_MD_HD digest, u32 *r_expire )
 }
 
 
-#if 0 /* not anymore used */
-/****************
- * Check the MDC which is contained in SIG.
- * The GCRY_MD_HD should be currently open, so that this function
- * is able to append some data, before finalizing the digest.
- */
-int
-mdc_kludge_check( PKT_signature *sig, GCRY_MD_HD digest )
-{
-    int rc=0;
-
-    if( (rc=check_digest_algo(sig->digest_algo)) )
-	return rc;
-
-    /* make sure the digest algo is enabled (in case of a detached mdc??) */
-    md_enable( digest, sig->digest_algo );
-
-    /* complete the digest */
-    if( sig->version >= 4 )
-	gcry_md_putc( digest, sig->version );
-    gcry_md_putc( digest, sig->sig_class );
-    if( sig->version < 4 ) {
-	u32 a = sig->timestamp;
-	gcry_md_putc( digest, (a >> 24) & 0xff );
-	gcry_md_putc( digest, (a >> 16) & 0xff );
-	gcry_md_putc( digest, (a >>  8) & 0xff );
-	gcry_md_putc( digest,  a	& 0xff );
-    }
-    else {
-	byte buf[6];
-	size_t n;
-	gcry_md_putc( digest, sig->pubkey_algo );
-	gcry_md_putc( digest, sig->digest_algo );
-	if( sig->hashed_data ) {
-	    n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
-	    gcry_md_write( digest, sig->hashed_data, n+2 );
-	    n += 6;
-	}
-	else
-	    n = 6;
-	/* add some magic */
-	buf[0] = sig->version;
-	buf[1] = 0xff;
-	buf[2] = n >> 24;
-	buf[3] = n >> 16;
-	buf[4] = n >>  8;
-	buf[5] = n;
-	gcry_md_write( digest, buf, 6 );
-    }
-    md_final( digest );
-
-    rc = G10ERR_BAD_SIGN;
-    {	const byte *s1 = md_read( digest, sig->digest_algo );
-	int s1len = md_digest_length( sig->digest_algo );
-
-	log_hexdump( "MDC calculated", s1, s1len );
-
-	if( !sig->data[0] )
-	    log_debug("sig_data[0] is NULL\n");
-	else {
-	    unsigned s2len;
-	    byte *s2;
-	    s2 = mpi_get_buffer( sig->data[0], &s2len, NULL );
-	    log_hexdump( "MDC stored    ", s2, s2len );
-
-	    if( s2len != s1len )
-		log_debug("MDC check: len differ: %d/%d\n", s1len, s2len);
-	    else if( memcmp( s1, s2, s1len ) )
-		log_debug("MDC check: hashs differ\n");
-	    else
-		rc = 0;
-	    m_free(s2);
-	}
-    }
-
-    if( !rc && sig->flags.unknown_critical ) {
-	log_info(_("assuming bad MDC due to an unknown critical bit\n"));
-	rc = G10ERR_BAD_SIGN;
-    }
-    sig->flags.checked = 1;
-    sig->flags.valid = !rc;
-
-    /* FIXME: check that we are actually in an encrypted packet */
-
-    return rc;
-}
-#endif
-
 /****************
  * This function gets called by pubkey_verify() if the algorithm needs it.
  */
@@ -246,7 +229,7 @@ cmp_help( void *opaque, MPI result )
     }
     if( old_enc ) {
 	log_error("old encoding scheme is not supported\n");
-	return G10ERR_GENERAL;
+	return GPGERR_GENERAL;
     }
 
     if( (rc=check_digest_algo(sig->digest_algo)) )
@@ -258,25 +241,25 @@ cmp_help( void *opaque, MPI result )
 	if( asn[j] != c )
 	    break;
     if( j != -1 || mpi_getbyte(result, i) )
-	return G10ERR_BAD_PUBKEY;  /* ASN is wrong */
+	return GPGERR_BAD_PUBKEY;  /* ASN is wrong */
     for(i++; (c=mpi_getbyte(result, i)) != -1; i++ )
 	if( c != 0xff  )
 	    break;
     i++;
     if( c != sig->digest_algo || mpi_getbyte(result, i) ) {
 	/* Padding or leading bytes in signature is wrong */
-	return G10ERR_BAD_PUBKEY;
+	return GPGERR_BAD_PUBKEY;
     }
     if( mpi_getbyte(result, mdlen-1) != sig->digest_start[0]
 	|| mpi_getbyte(result, mdlen-2) != sig->digest_start[1] ) {
 	/* Wrong key used to check the signature */
-	return G10ERR_BAD_PUBKEY;
+	return GPGERR_BAD_PUBKEY;
     }
 
     dp = md_read( digest, sig->digest_algo );
     for(i=mdlen-1; i >= 0; i--, dp++ ) {
 	if( mpi_getbyte( result, i ) != *dp )
-	    return G10ERR_BAD_SIGN;
+	    return GPGERR_BAD_SIGN;
     }
     return 0;
   #else
@@ -286,17 +269,19 @@ cmp_help( void *opaque, MPI result )
 
 
 static int
-do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest )
+do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest,
+						    int *r_expired )
 {
     MPI result = NULL;
     int rc=0;
     struct cmp_help_context_s ctx;
     u32 cur_time;
 
-    if( pk->version == 4 && pk->pubkey_algo == PUBKEY_ALGO_ELGAMAL_E ) {
+    *r_expired = 0;
+    if( pk->version == 4 && pk->pubkey_algo == GCRY_PK_ELG_E ) {
 	log_info(_("this is a PGP generated "
-		  "ElGamal key which is NOT secure for signatures!\n"));
-	return G10ERR_PUBKEY_ALGO;
+		   "ElGamal key which is NOT secure for signatures!\n"));
+	return GPGERR_PUBKEY_ALGO;
     }
 
     if( pk->timestamp > sig->timestamp ) {
@@ -305,7 +290,8 @@ do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest )
 		  ? _("public key is %lu second newer than the signature\n")
 		  : _("public key is %lu seconds newer than the signature\n"),
 		       d );
-	return G10ERR_TIME_CONFLICT; /* pubkey newer than signature */
+	if( !opt.ignore_time_conflict )
+	    return GPGERR_TIME_CONFLICT; /* pubkey newer than signature */
     }
 
     cur_time = make_timestamp();
@@ -315,19 +301,21 @@ do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest )
 			   "in future (time warp or clock problem)\n")
 		       : _("key has been created %lu seconds "
 			   "in future (time warp or clock problem)\n"), d );
-	return G10ERR_TIME_CONFLICT;
+	if( !opt.ignore_time_conflict )
+	    return GPGERR_TIME_CONFLICT;
     }
 
     if( pk->expiredate && pk->expiredate < cur_time ) {
 	log_info(_("NOTE: signature key expired %s\n"),
 					asctimestamp( pk->expiredate ) );
 	write_status(STATUS_SIGEXPIRED);
+	*r_expired = 1;
     }
 
 
     if( (rc=openpgp_md_test_algo(sig->digest_algo)) )
 	return rc;
-    if( (rc=openpgp_pk_test_algo(sig->pubkey_algo)) )
+    if( (rc=openpgp_pk_test_algo(sig->pubkey_algo, 0)) )
 	return rc;
 
     /* make sure the digest algo is enabled (in case of a detached signature)*/
@@ -368,16 +356,27 @@ do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest )
     gcry_md_final( digest );
 
     result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
-				      mpi_get_nbits(pk->pkey[0]));
-
+			      gcry_mpi_get_nbits(pk->pkey[0]), 0);
     ctx.sig = sig;
     ctx.md = digest;
-    rc = pubkey_verify( pk->pubkey_algo, result, sig->data, pk->pkey,
+    rc = pk_verify( pk->pubkey_algo, result, sig->data, pk->pkey,
 			cmp_help, &ctx );
-    mpi_free( result );
+    mpi_release( result );
+    if( (opt.emulate_bugs & EMUBUG_MDENCODE)
+	&& rc == GPGERR_BAD_SIGN && is_ELGAMAL(pk->pubkey_algo) ) {
+	/* In this case we try again because old GnuPG versions didn't encode
+	 * the hash right. There is no problem with DSA however  */
+	result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
+				  gcry_mpi_get_nbits(pk->pkey[0]), (sig->version < 5) );
+	ctx.sig = sig;
+	ctx.md = digest;
+	rc = pk_verify( pk->pubkey_algo, result, sig->data, pk->pkey,
+			cmp_help, &ctx );
+    }
+
     if( !rc && sig->flags.unknown_critical ) {
 	log_info(_("assuming bad signature due to an unknown critical bit\n"));
-	rc = G10ERR_BAD_SIGN;
+	rc = GPGERR_BAD_SIGN;
     }
     sig->flags.checked = 1;
     sig->flags.valid = !rc;
@@ -392,16 +391,30 @@ hash_uid_node( KBNODE unode, GCRY_MD_HD md, PKT_signature *sig )
     PKT_user_id *uid = unode->pkt->pkt.user_id;
 
     assert( unode->pkt->pkttype == PKT_USER_ID );
-    if( sig->version >=4 ) {
-	byte buf[5];
-	buf[0] = 0xb4; /* indicates a userid packet */
-	buf[1] = uid->len >> 24;  /* always use 4 length bytes */
-	buf[2] = uid->len >> 16;
-	buf[3] = uid->len >>  8;
-	buf[4] = uid->len;
-	gcry_md_write( md, buf, 5 );
+    if( uid->photo ) {
+	if( sig->version >=4 ) {
+	    byte buf[5];
+	    buf[0] = 0xd1;		   /* packet of type 17 */
+	    buf[1] = uid->photolen >> 24;  /* always use 4 length bytes */
+	    buf[2] = uid->photolen >> 16;
+	    buf[3] = uid->photolen >>  8;
+	    buf[4] = uid->photolen;
+	    gcry_md_write( md, buf, 5 );
+	}
+	gcry_md_write( md, uid->photo, uid->photolen );
+    }
+    else {
+	if( sig->version >=4 ) {
+	    byte buf[5];
+	    buf[0] = 0xb4;	      /* indicates a userid packet */
+	    buf[1] = uid->len >> 24;  /* always use 4 length bytes */
+	    buf[2] = uid->len >> 16;
+	    buf[3] = uid->len >>  8;
+	    buf[4] = uid->len;
+	    gcry_md_write( md, buf, 5 );
+	}
+	gcry_md_write( md, uid->name, uid->len );
     }
-    gcry_md_write( md, uid->name, uid->len );
 }
 
 /****************
@@ -413,11 +426,13 @@ int
 check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 {
     u32 dummy;
-    return check_key_signature2(root, node, is_selfsig, &dummy );
+    int dum2;
+    return check_key_signature2(root, node, is_selfsig, &dummy, &dum2 );
 }
 
 int
-check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
+check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
+				       u32 *r_expiredate, int *r_expired )
 {
     GCRY_MD_HD md;
     PKT_public_key *pk;
@@ -427,7 +442,8 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 
     if( is_selfsig )
 	*is_selfsig = 0;
-    *r_expire = 0;
+    *r_expiredate = 0;
+    *r_expired = 0;
     assert( node->pkt->pkttype == PKT_SIGNATURE );
     assert( root->pkt->pkttype == PKT_PUBLIC_KEY );
 
@@ -448,7 +464,7 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 	if( !(md = gcry_md_open( algo, 0 )) )
 	    BUG();
 	hash_public_key( md, pk );
-	rc = do_check( pk, sig, md );
+	rc = do_check( pk, sig, md, r_expired );
 	gcry_md_close(md);
     }
     else if( sig->sig_class == 0x28 ) { /* subkey revocation */
@@ -459,12 +475,12 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 		BUG();
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md );
+	    rc = do_check( pk, sig, md, r_expired );
 	    gcry_md_close(md);
 	}
 	else {
 	    log_error("no subkey for subkey revocation packet\n");
-	    rc = G10ERR_SIG_CLASS;
+	    rc = GPGERR_SIG_CLASS;
 	}
     }
     else if( sig->sig_class == 0x18 ) {
@@ -482,12 +498,12 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 		BUG();
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md );
+	    rc = do_check( pk, sig, md, r_expired );
 	    gcry_md_close(md);
 	}
 	else {
 	    log_error("no subkey for key signature packet\n");
-	    rc = G10ERR_SIG_CLASS;
+	    rc = GPGERR_SIG_CLASS;
 	}
     }
     else {
@@ -504,15 +520,16 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 	    if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] ) {
 		if( is_selfsig )
 		    *is_selfsig = 1;
-		rc = do_check( pk, sig, md );
+		rc = do_check( pk, sig, md, r_expired );
+	    }
+	    else {
+		rc = do_signature_check( sig, md, r_expiredate, r_expired );
 	    }
-	    else
-		rc = do_signature_check( sig, md, r_expire );
 	    gcry_md_close(md);
 	}
 	else {
 	    log_error("no user ID for key signature packet\n");
-	    rc = G10ERR_SIG_CLASS;
+	    rc = GPGERR_SIG_CLASS;
 	}
     }
 
