@@ -1,5 +1,5 @@
 /* sign.c - sign data
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -31,15 +31,76 @@
 #include "errors.h"
 #include "iobuf.h"
 #include "keydb.h"
-#include "memory.h"
 #include "util.h"
 #include "main.h"
 #include "filter.h"
 #include "ttyio.h"
 #include "trustdb.h"
+#include "status.h"
 #include "i18n.h"
 
 
+#ifdef HAVE_DOSISH_SYSTEM
+  #define LF "\r\n"
+#else
+  #define LF "\n"
+#endif
+
+/****************
+ * Emulate our old PK interface here - sometime in the future we might
+ * change the internal design to directly fit to libgcrypt.
+ */
+static int
+pk_sign( int algo, MPI *data, MPI hash, MPI *skey )
+{
+    GCRY_SEXP s_sig, s_hash, s_skey, list;
+    int rc;
+
+    /* make a sexp from skey */
+    if( algo == GCRY_PK_DSA ) {
+	rc = gcry_sexp_build ( &s_skey, NULL,
+			      "(private-key(dsa(p%m)(q%m)(g%m)(y%m)(x%m)))",
+				  skey[0], skey[1], skey[2], skey[3], skey[4] );
+    }
+    else if( algo == GCRY_PK_ELG || algo == GCRY_PK_ELG_E ) {
+	rc = gcry_sexp_build ( &s_skey, NULL,
+			      "(private-key(elg(p%m)(g%m)(y%m)(x%m)))",
+				  skey[0], skey[1], skey[2], skey[3] );
+    }
+    else
+	return GPGERR_PUBKEY_ALGO;
+
+    if ( rc )
+	BUG ();
+
+    /* put hash into a S-Exp s_hash */
+    if ( gcry_sexp_build( &s_hash, NULL, "%m", hash ) )
+	BUG ();
+
+    rc = gcry_pk_sign( &s_sig, s_hash, s_skey );
+    gcry_sexp_release( s_hash );
+    gcry_sexp_release( s_skey );
+
+    if( rc )
+	;
+    else {
+	list = gcry_sexp_find_token( s_sig, "r" , 0 );
+	assert( list );
+	data[0] = gcry_sexp_nth_mpi( list, 1, 0 );
+	assert( data[0] );
+	gcry_sexp_release (list);
+
+	list = gcry_sexp_find_token( s_sig, "s" , 0 );
+	assert( list );
+	data[1] = gcry_sexp_nth_mpi( list, 1, 0 );
+	assert( data[1] );
+	gcry_sexp_release (list);
+    }
+
+
+    gcry_sexp_release( s_sig );
+    return rc;
+}
 
 /****************
  * Create a notation.  It is assumed that the stings in STRLIST
@@ -67,7 +128,7 @@ mk_notation_and_policy( PKT_signature *sig )
 	    n1 = s - string;
 	    s++;
 	    n2 = strlen(s);
-	    buf = m_alloc( 8 + n1 + n2 );
+	    buf = gcry_xmalloc( 8 + n1 + n2 );
 	    buf[0] = 0x80; /* human readable */
 	    buf[1] = buf[2] = buf[3] = 0;
 	    buf[4] = n1 >> 8;
@@ -107,7 +168,8 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 			   "in future (time warp or clock problem)\n")
 		       : _("key has been created %lu seconds "
 			   "in future (time warp or clock problem)\n"), d );
-	return G10ERR_TIME_CONFLICT;
+	if( !opt.ignore_time_conflict )
+	    return GPGERR_TIME_CONFLICT;
     }
 
 
@@ -122,17 +184,17 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
     sig->digest_start[0] = dp[0];
     sig->digest_start[1] = dp[1];
     frame = encode_md_value( sk->pubkey_algo, md,
-			     digest_algo, mpi_get_nbits(sk->skey[0]));
-    rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
-    mpi_free(frame);
+			     digest_algo, gcry_mpi_get_nbits(sk->skey[0]), 0 );
+    rc = pk_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
+    mpi_release(frame);
     if( rc )
-	log_error(_("signing failed: %s\n"), g10_errstr(rc) );
+	log_error(_("signing failed: %s\n"), gpg_errstr(rc) );
     else {
 	if( opt.verbose ) {
 	    char *ustr = get_user_id_string( sig->keyid );
 	    log_info(_("%s signature from: %s\n"),
 		      gcry_pk_algo_name(sk->pubkey_algo), ustr );
-	    m_free(ustr);
+	    gcry_free(ustr);
 	}
     }
     return rc;
@@ -159,10 +221,10 @@ hash_for(int pubkey_algo )
 {
     if( opt.def_digest_algo )
 	return opt.def_digest_algo;
-    if( pubkey_algo == PUBKEY_ALGO_DSA )
-	return DIGEST_ALGO_SHA1;
-    if( pubkey_algo == PUBKEY_ALGO_RSA )
-	return DIGEST_ALGO_MD5;
+    if( pubkey_algo == GCRY_PK_DSA )
+	return GCRY_MD_SHA1;
+    if( pubkey_algo == GCRY_PK_RSA )
+	return GCRY_MD_MD5;
     return DEFAULT_DIGEST_ALGO;
 }
 
@@ -175,7 +237,7 @@ only_old_style( SK_LIST sk_list )
     /* if there are only old style capable key we use the old sytle */
     for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
 	PKT_secret_key *sk = sk_rover->sk;
-	if( sk->pubkey_algo == PUBKEY_ALGO_RSA && sk->version < 4 )
+	if( sk->pubkey_algo == GCRY_PK_RSA && sk->version < 4 )
 	    old_style = 1;
 	else
 	    return 0;
@@ -184,6 +246,25 @@ only_old_style( SK_LIST sk_list )
 }
 
 
+static void
+print_status_sig_created ( PKT_secret_key *sk, PKT_signature *sig, int what )
+{
+    byte array[MAX_FINGERPRINT_LEN], *p;
+    char buf[100+MAX_FINGERPRINT_LEN*2];
+    size_t i, n;
+
+    sprintf(buf, "%c %d %d %02x %lu ",
+	    what, sig->pubkey_algo, sig->digest_algo, sig->sig_class,
+	    (ulong)sig->timestamp );
+
+    fingerprint_from_sk( sk, array, &n );
+    p = buf + strlen(buf);
+    for(i=0; i < n ; i++ )
+	sprintf(p+2*i, "%02X", array[i] );
+
+    write_status_text( STATUS_SIG_CREATED, buf );
+}
+
 
 /****************
  * Sign the files whose names are in FILENAME.
@@ -237,13 +318,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( fname && filenames->next && (!detached || encrypt) )
 	log_bug("multiple files can only be detached signed");
 
-    if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
+    if( (rc=build_sk_list( locusr, &sk_list, 1, GCRY_PK_USAGE_SIGN )) )
 	goto leave;
     if( !old_style )
 	old_style = only_old_style( sk_list );
 
     if( encrypt ) {
-	if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC )) )
+	if( (rc=build_pk_list( remusr, &pk_list, GCRY_PK_USAGE_ENCR )) )
 	    goto leave;
 	if( !old_style )
 	    compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_COMPR );
@@ -255,14 +336,14 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     else if( !(inp = iobuf_open(fname)) ) {
 	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
 					strerror(errno) );
-	rc = G10ERR_OPEN_FILE;
+	rc = GPGERR_OPEN_FILE;
 	goto leave;
     }
 
     if( outfile ) {
 	if( !(out = iobuf_create( outfile )) ) {
 	    log_error(_("can't create %s: %s\n"), outfile, strerror(errno) );
-	    rc = G10ERR_CREATE_FILE;
+	    rc = GPGERR_CREATE_FILE;
 	    goto leave;
 	}
 	else if( opt.verbose )
@@ -334,7 +415,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 		    break;
 
 	    sk = sk_rover->sk;
-	    ops = m_alloc_clear( sizeof *ops );
+	    ops = gcry_xcalloc( 1, sizeof *ops );
 	    ops->sig_class = opt.textmode && !outfile ? 0x01 : 0x00;
 	    ops->digest_algo = hash_for(sk->pubkey_algo);
 	    ops->pubkey_algo = sk->pubkey_algo;
@@ -348,7 +429,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    free_packet( &pkt );
 	    if( rc ) {
 		log_error("build onepass_sig packet failed: %s\n",
-							g10_errstr(rc));
+							gpg_errstr(rc));
 		goto leave;
 	    }
 	}
@@ -367,7 +448,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 		if( !(inp = iobuf_open(sl->d)) ) {
 		    log_error(_("can't open %s: %s\n"),
 					    sl->d, strerror(errno) );
-		    rc = G10ERR_OPEN_FILE;
+		    rc = GPGERR_OPEN_FILE;
 		    goto leave;
 		}
 		if( opt.verbose )
@@ -390,13 +471,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	if (!opt.no_literal) {
 	    if( fname || opt.set_filename ) {
 		char *s = make_basename( opt.set_filename ? opt.set_filename : fname );
-		pt = m_alloc( sizeof *pt + strlen(s) - 1 );
+		pt = gcry_xmalloc( sizeof *pt + strlen(s) - 1 );
 		pt->namelen = strlen(s);
 		memcpy(pt->name, s, pt->namelen );
-		m_free(s);
+		gcry_free(s);
 	    }
 	    else { /* no filename */
-		pt = m_alloc( sizeof *pt - 1 );
+		pt = gcry_xmalloc( sizeof *pt - 1 );
 		pt->namelen = 0;
 	    }
 	}
@@ -426,7 +507,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    pkt.pkt.plaintext = pt;
 	    /*cfx.datalen = filesize? calc_packet_length( &pkt ) : 0;*/
 	    if( (rc = build_packet( out, &pkt )) )
-		log_error("build_packet(PLAINTEXT) failed: %s\n", g10_errstr(rc) );
+		log_error("build_packet(PLAINTEXT) failed: %s\n", gpg_errstr(rc) );
 	    pt->buf = NULL;
 	}
 	else {
@@ -434,8 +515,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    int  bytes_copied;
 	    while ((bytes_copied = iobuf_read(inp, copy_buffer, 4096)) != -1)
 		if (iobuf_write(out, copy_buffer, bytes_copied) == -1) {
-		    rc = G10ERR_WRITE_FILE;
-		    log_error("copying input to output failed: %s\n", g10_errstr(rc));
+		    rc = GPGERR_WRITE_FILE;
+		    log_error("copying input to output failed: %s\n", gpg_errstr(rc));
 		    break;
 		}
 	    memset(copy_buffer, 0, 4096); /* burn buffer */
@@ -456,7 +537,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 
 	/* build the signature packet */
 	/* fixme: this code is partly duplicated in make_keysig_packet */
-	sig = m_alloc_clear( sizeof *sig );
+	sig = gcry_xcalloc( 1, sizeof *sig );
 	sig->version = old_style || opt.force_v3_sigs ? 3 : sk->version;
 	keyid_from_sk( sk, sig->keyid );
 	sig->digest_algo = hash_for(sk->pubkey_algo);
@@ -465,6 +546,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	sig->sig_class = opt.textmode && !outfile? 0x01 : 0x00;
 
 	md = gcry_md_copy( mfx.md );
+	if( !md )
+	    BUG();
 
 	if( sig->version >= 4 ) {
 	    build_sig_subpkt_from_sig( sig );
@@ -517,12 +600,16 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    pkt.pkttype = PKT_SIGNATURE;
 	    pkt.pkt.signature = sig;
 	    rc = build_packet( out, &pkt );
+	    if( !rc && is_status_enabled() ) {
+		print_status_sig_created ( sk, sig, detached ? 'D':'S');
+	    }
 	    free_packet( &pkt );
 	    if( rc )
-		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
+		log_error("build signature packet failed: %s\n", gpg_errstr(rc) );
 	}
 	if( rc )
 	    goto leave;
+
     }
 
 
@@ -559,7 +646,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
 
-    if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
+    if( (rc=build_sk_list( locusr, &sk_list, 1, GCRY_PK_USAGE_SIGN )) )
 	goto leave;
     if( !old_style )
 	old_style = only_old_style( sk_list );
@@ -568,14 +655,14 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     if( !(inp = iobuf_open(fname)) ) {
 	log_error("can't open %s: %s\n", fname? fname: "[stdin]",
 					strerror(errno) );
-	rc = G10ERR_OPEN_FILE;
+	rc = GPGERR_OPEN_FILE;
 	goto leave;
     }
 
     if( outfile ) {
 	if( !(out = iobuf_create( outfile )) ) {
 	    log_error(_("can't create %s: %s\n"), outfile, strerror(errno) );
-	    rc = G10ERR_CREATE_FILE;
+	    rc = GPGERR_CREATE_FILE;
 	    goto leave;
 	}
 	else if( opt.verbose )
@@ -584,11 +671,11 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     else if( (rc = open_outfile( fname, 1, &out )) )
 	goto leave;
 
-    iobuf_writestr(out, "-----BEGIN PGP SIGNED MESSAGE-----\n" );
+    iobuf_writestr(out, "-----BEGIN PGP SIGNED MESSAGE-----" LF );
 
     for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
 	PKT_secret_key *sk = sk_rover->sk;
-	if( hash_for(sk->pubkey_algo) == DIGEST_ALGO_MD5 )
+	if( hash_for(sk->pubkey_algo) == GCRY_MD_MD5 )
 	    only_md5 = 1;
 	else {
 	    only_md5 = 0;
@@ -627,13 +714,15 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     }
 
 
-    if( !(textmd = gcry_md_open(0, 0)) )
+    textmd = gcry_md_open(0, 0);
+    if( !textmd )
 	BUG();
     for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
 	PKT_secret_key *sk = sk_rover->sk;
 	gcry_md_enable(textmd, hash_for(sk->pubkey_algo));
     }
-    /*md_start_debug( textmd, "sign" );*/
+    if ( DBG_HASHING )
+	gcry_md_start_debug( textmd, "clearsign" );
     copy_clearsig_text( out, inp, textmd,
 			!opt.not_dash_escaped, opt.escape_from, old_style );
     /* fixme: check for read errors */
@@ -652,7 +741,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 
 	/* build the signature packet */
 	/* fixme: this code is duplicated above */
-	sig = m_alloc_clear( sizeof *sig );
+	sig = gcry_xcalloc( 1, sizeof *sig );
 	sig->version = old_style || opt.force_v3_sigs ? 3 : sk->version;
 	keyid_from_sk( sk, sig->keyid );
 	sig->digest_algo = hash_for(sk->pubkey_algo);
@@ -661,6 +750,8 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	sig->sig_class = 0x01;
 
 	md = gcry_md_copy( textmd );
+	if( !md )
+	    BUG();
 	if( sig->version >= 4 ) {
 	    build_sig_subpkt_from_sig( sig );
 	    gcry_md_putc( md, sig->version );
@@ -712,9 +803,12 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	    pkt.pkttype = PKT_SIGNATURE;
 	    pkt.pkt.signature = sig;
 	    rc = build_packet( out, &pkt );
+	    if( !rc && is_status_enabled() ) {
+		print_status_sig_created ( sk, sig, 'C');
+	    }
 	    free_packet( &pkt );
 	    if( rc )
-		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
+		log_error("build signature packet failed: %s\n", gpg_errstr(rc) );
 	}
 	if( rc )
 	    goto leave;
@@ -756,10 +850,10 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 	    || sigclass == 0x30 || sigclass == 0x28 );
     if( !digest_algo ) {
 	switch( sk->pubkey_algo ) {
-	  case PUBKEY_ALGO_DSA: digest_algo = DIGEST_ALGO_SHA1; break;
-	  case PUBKEY_ALGO_RSA_S:
-	  case PUBKEY_ALGO_RSA: digest_algo = DIGEST_ALGO_MD5; break;
-	  default:		digest_algo = DIGEST_ALGO_RMD160; break;
+	  case GCRY_PK_DSA: digest_algo = GCRY_MD_SHA1; break;
+	  case GCRY_PK_RSA_S:
+	  case GCRY_PK_RSA: digest_algo = GCRY_MD_MD5; break;
+	  default:		digest_algo = GCRY_MD_RMD160; break;
 	}
     }
     if( !(md = gcry_md_open( digest_algo, 0 )))
@@ -783,7 +877,7 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 	gcry_md_write( md, uid->name, uid->len );
     }
     /* and make the signature packet */
-    sig = m_alloc_clear( sizeof *sig );
+    sig = gcry_xcalloc( 1, sizeof *sig );
     sig->version = sk->version;
     keyid_from_sk( sk, sig->keyid );
     sig->pubkey_algo = sk->pubkey_algo;
