@@ -1,5 +1,5 @@
 /* tdbio.h - Trust database I/O functions
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -18,8 +18,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef G10_TDBIO_H
-#define G10_TDBIO_H
+#ifndef GPG_TDBIO_H
+#define GPG_TDBIO_H
 
 #include "host2net.h"
 
@@ -195,4 +195,4 @@ int tdbio_search_sdir( u32 *keyid, int pubkey_algo, TRUSTREC *rec );
 
 void tdbio_invalid(void);
 
-#endif /*G10_TDBIO_H*/
+#endif /*GPG_TDBIO_H*/
