@@ -1,5 +1,5 @@
 /* types.h - some common typedefs
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -18,8 +18,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef G10_TYPES_H
-#define G10_TYPES_H
+#ifndef GPG_TYPES_H
+#define GPG_TYPES_H
 
 
 /* The AC_CHECK_SIZEOF() in configure fails for some machines.
@@ -121,4 +121,4 @@ typedef struct string_list {
 } *STRLIST;
 
 
-#endif /*G10_TYPES_H*/
+#endif /*GPG_TYPES_H*/
