@@ -1,5 +1,5 @@
 /* mpi-bit.c  -  MPI bit level fucntions
- * Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -55,7 +55,7 @@ __clz_tab[] =
 void
 mpi_normalize( MPI a )
 {
-    if( mpi_is_protected(a) )
+    if( mpi_is_opaque(a) )
 	return;
 
     for( ; a->nlimbs && !a->d[a->nlimbs-1]; a->nlimbs-- )
@@ -67,16 +67,13 @@ mpi_normalize( MPI a )
 /****************
  * Return the number of bits in A.
  */
-unsigned
+unsigned int
 mpi_get_nbits( MPI a )
 {
     unsigned n;
 
-    if( mpi_is_protected(a) ) {
-	n = mpi_get_nbit_info(a);
-	if( !n )
-	    n = a->nlimbs * BITS_PER_MPI_LIMB;
-	return n;
+    if( mpi_is_opaque(a) ) {
+	return a->sign; /* which holds the number of bits */
     }
 
     mpi_normalize( a );
@@ -93,6 +90,11 @@ mpi_get_nbits( MPI a )
     return n;
 }
 
+unsigned int
+gcry_mpi_get_nbits( MPI a )
+{
+    return mpi_get_nbits( a );
+}
 
 /****************
  * Test whether bit N is set.
