@@ -1,6 +1,6 @@
 /* mpi-pow.c  -  MPI functions
  *	Copyright (C) 1998 Free Software Foundation, Inc.
- *	Copyright (C) 1994, 1996 Free Software Foundation, Inc.
+ *	Copyright (C) 1994, 1996, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -30,6 +30,7 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "mpi-internal.h"
 #include "longlong.h"
 #include <assert.h>
@@ -39,7 +40,7 @@
  * RES = BASE ^ EXP mod MOD
  */
 void
-mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
+gcry_mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 {
     mpi_ptr_t  rp, ep, mp, bp;
     mpi_size_t esize, msize, bsize, rsize;
@@ -52,7 +53,8 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
     mpi_ptr_t xp_marker=NULL;
     int assign_rp=0;
     mpi_ptr_t tspace = NULL;
-    mpi_size_t tsize=0;   /* to avoid compiler warning, fixme: check */
+    mpi_size_t tsize=0;   /* to avoid compiler warning */
+			  /* fixme: we should check that the warning is void*/
 
     esize = exp->nlimbs;
     msize = mod->nlimbs;
@@ -158,7 +160,9 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 	int c;
 	mpi_limb_t e;
 	mpi_limb_t carry_limb;
+	struct karatsuba_ctx karactx;
 
+	memset( &karactx, 0, sizeof karactx );
 	negative_result = (ep[0] & 1) && base->sign;
 
 	i = esize - 1;
@@ -176,6 +180,7 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 	 * by RP (==RES->d), and with 50% probability in the area originally
 	 * pointed to by XP.
 	 */
+
 	for(;;) {
 	    while( c ) {
 		mpi_ptr_t tp;
@@ -193,7 +198,6 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 			mpi_free_limb_space( tspace );
 			tsize = 2 * rsize;
 			tspace = mpi_alloc_limb_space( tsize, 0 );
-
 		    }
 		    mpih_sqr_n( xp, rp, rsize, tspace );
 		}
@@ -208,7 +212,15 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 		rsize = xsize;
 
 		if( (mpi_limb_signed_t)e < 0 ) {
-		    mpihelp_mul( xp, rp, rsize, bp, bsize );
+		    /*mpihelp_mul( xp, rp, rsize, bp, bsize );*/
+		    if( bsize < KARATSUBA_THRESHOLD ) {
+			mpihelp_mul( xp, rp, rsize, bp, bsize );
+		    }
+		    else {
+			mpihelp_mul_karatsuba_case(
+				     xp, rp, rsize, bp, bsize, &karactx );
+		    }
+
 		    xsize = rsize + bsize;
 		    if( xsize > msize ) {
 			mpihelp_divrem(xp + msize, 0, xp, xsize, mp, msize);
@@ -257,6 +269,8 @@ mpi_powm( MPI res, MPI base, MPI exp, MPI mod)
 	if( mod_shift_cnt )
 	    mpihelp_rshift( rp, rp, rsize, mod_shift_cnt);
 	MPN_NORMALIZE (rp, rsize);
+
+	mpihelp_release_karatsuba_ctx( &karactx );
     }
 
     if( negative_result && rsize ) {
