@@ -25,10 +25,7 @@
 #include <string.h>
 #include <assert.h>
 #include <unistd.h>
-#ifdef __riscos__
-#include <kernel.h>
-#include <swis.h>
-#endif /* __riscos__ */
+#include <gcrypt.h>
 #include "util.h"
 #include "memory.h"
 #include "ttyio.h"
@@ -43,16 +40,16 @@ make_basename(const char *filepath)
 {
     char *p;
 
-    if ( !(p=strrchr(filepath, DIRSEP_C)) )
+    if ( !(p=strrchr(filepath, '/')) )
       #ifdef HAVE_DRIVE_LETTERS
 	if ( !(p=strrchr(filepath, '\\')) )
 	    if ( !(p=strrchr(filepath, ':')) )
       #endif
 	      {
-		return m_strdup(filepath);
+		return gcry_xstrdup(filepath);
 	      }
 
-    return m_strdup(p+1);
+    return gcry_xstrdup(p+1);
 }
 
 
@@ -70,17 +67,17 @@ make_dirname(const char *filepath)
     int  dirname_length;
     char *p;
 
-    if ( !(p=strrchr(filepath, DIRSEP_C)) )
+    if ( !(p=strrchr(filepath, '/')) )
       #ifdef HAVE_DRIVE_LETTERS
 	if ( !(p=strrchr(filepath, '\\')) )
 	    if ( !(p=strrchr(filepath, ':')) )
       #endif
 	      {
-		return m_strdup(EXTSEP_S);
+		return gcry_xstrdup(".");
 	      }
 
     dirname_length = p-filepath;
-    dirname = m_alloc(dirname_length+1);
+    dirname = gcry_xmalloc(dirname_length+1);
     strncpy(dirname, filepath, dirname_length);
     dirname[dirname_length] = 0;
 
@@ -107,30 +104,20 @@ make_filename( const char *first_part, ... )
 	n += strlen(s) + 1;
     va_end(arg_ptr);
 
-#ifndef __riscos__
     home = NULL;
-    if( *first_part == '~' && first_part[1] == DIRSEP_C
+    if( *first_part == '~' && first_part[1] == '/'
 			   && (home = getenv("HOME")) && *home )
 	n += strlen(home);
-    name = m_alloc(n);
+
+    name = gcry_xmalloc(n);
     p = home ? stpcpy(stpcpy(name,home), first_part+1)
 	     : stpcpy(name, first_part);
-#else /* __riscos__ */
-    name = m_alloc(n);
-    p = stpcpy(name, first_part);
-#endif /* __riscos__ */
     va_start( arg_ptr, first_part ) ;
     while( (s=va_arg(arg_ptr, const char *)) )
-	p = stpcpy(stpcpy(p, DIRSEP_S), s);
+	p = stpcpy(stpcpy(p,"/"), s);
     va_end(arg_ptr);
 
-#ifndef __riscos__
     return name;
-#else /* __riscos__ */
-    p = gstrans(name);
-    m_free(name);
-    return p;
-#endif /* __riscos__ */
 }
 
 
@@ -140,26 +127,11 @@ compare_filenames( const char *a, const char *b )
     /* ? check whether this is an absolute filename and
      * resolve symlinks?
      */
-#ifndef __riscos__
   #ifdef HAVE_DRIVE_LETTERS
-    return ascii_strcasecmp(a,b);
+    return stricmp(a,b);
   #else
     return strcmp(a,b);
   #endif
-#else /* __riscos__ */
-    int c = 0;
-    char *abuf, *bbuf;
-
-    abuf = gstrans(a);
-    bbuf = gstrans(b);
-
-    c = stricmp(abuf, bbuf);
-
-    m_free(abuf);
-    m_free(bbuf);
-
-    return c;
-#endif /* __riscos__ */
 }
 
 
@@ -184,49 +156,4 @@ print_fname_stdin( const char *s )
     return s;
 }
 
-/**
- * Check if the file is compressed.
- **/
-int
-is_file_compressed(const char *s, int *r_status)
-{
-    IOBUF a;
-    int i, rc = 0;
-    byte buf[4];
-
-    const byte sigs[4][4] = {
-    {0x42, 0x5a, 0x68, 0x39}, /* bzip2 */
-    {0x1f, 0x8b, 0x08, 0x08}, /* gzip */
-    {0x50, 0x4b, 0x03, 0x04} /* (pk)zip */
-    };
-    
-    if (!s || *s == '-' || !r_status)
-        return 0; /* We can't check stdin or no file was given */
-    
-    if ( (a = iobuf_open(s)) == NULL ) {
-        *r_status = G10ERR_OPEN_FILE;
-        return 0;
-    }
-    
-    if (iobuf_get_filelength(a) < 4) {        
-        *r_status = 0;
-        goto leave;
-    }
-            
-    iobuf_read(a, buf, 4);
-    
-    for (i=0; i<DIM(sigs); i++) {
-        if (!memcmp(buf, sigs[i], 4)) {
-            *r_status = 0;
-            rc = 1;
-            break;
-        }            
-    }
-
-leave:    
-    iobuf_close(a);
-    return rc;
-}
-
-
 
