@@ -1,5 +1,5 @@
 /* iobuf.c  -  file handling
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -27,6 +27,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <gcrypt.h>
+#ifdef HAVE_DOSISH_SYSTEM
+  #include <fcntl.h> /* for setmode() */
+#endif
 
 #include "memory.h"
 #include "util.h"
@@ -75,11 +79,13 @@ static int underflow(IOBUF a);
  *		    buffer, and should be set to the number of bytes
  *		    which were put into the buffer. The function
  *		    returns 0 to indicate success, -1 on EOF and
- *		    G10ERR_xxxxx for other errors.
+ *		    GPGERR_xxxxx for other errors.
  *
  * IOBUFCTRL_FLUSH: called by iobuf_flush() to write out the collected stuff.
  *		    *RET_LAN is the number of bytes in BUF.
  *
+ * IOBUFCTRL_CANCEL: send to all filters on behalf of iobuf_cancel.  The
+ *		    filter may take appropriate action on this message.
  */
 static int
 file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
@@ -104,7 +110,7 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	    else if( ferror(fp) && errno != EPIPE  ) {
 		log_error("%s: read error: %s\n",
 			  a->fname, strerror(errno));
-		rc = G10ERR_READ_FILE;
+		rc = GPGERR_READ_FILE;
 	    }
 	    *ret_len = nbytes;
 	}
@@ -115,7 +121,7 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	    nbytes = fwrite( buf, 1, size, fp );
 	    if( ferror(fp) ) {
 		log_error("%s: write error: %s\n", a->fname, strerror(errno));
-		rc = G10ERR_WRITE_FILE;
+		rc = GPGERR_WRITE_FILE;
 	    }
 	}
 	*ret_len = nbytes;
@@ -132,7 +138,7 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	    fclose(fp);
 	}
 	fp = NULL;
-	m_free(a); /* we can free our context now */
+	gcry_free(a); /* we can free our context now */
     }
 
     return rc;
@@ -177,7 +183,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    }
 		    else if( (c = iobuf_get(chain)) == -1 ) {
 			log_error("block_filter: 1st length byte missing\n");
-			rc = G10ERR_READ_FILE;
+			rc = GPGERR_READ_FILE;
 			break;
 		    }
 		    if( c < 192 ) {
@@ -194,7 +200,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 			a->size = (c - 192) * 256;
 			if( (c = iobuf_get(chain)) == -1 ) {
 			    log_error("block_filter: 2nd length byte missing\n");
-			    rc = G10ERR_READ_FILE;
+			    rc = GPGERR_READ_FILE;
 			    break;
 			}
 			a->size += c + 192;
@@ -212,7 +218,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 			a->size |= iobuf_get(chain) << 8;
 			if( (c = iobuf_get(chain)) == -1 ) {
 			    log_error("block_filter: invalid 4 byte length\n");
-			    rc = G10ERR_READ_FILE;
+			    rc = GPGERR_READ_FILE;
 			    break;
 			}
 			a->size |= c;
@@ -229,7 +235,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    a->size |= c;
 		    if( c == -1 ) {
 			log_error("block_filter: error reading length info\n");
-			rc = G10ERR_READ_FILE;
+			rc = GPGERR_READ_FILE;
 		    }
 		    if( !a->size ) {
 			a->eof = 1;
@@ -247,7 +253,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    if( c == -1 ) c = 0;
 		    log_error("block_filter %p: read error (size=%lu,a->size=%lu)\n",
 			      a,  (ulong)size+c, (ulong)a->size+c);
-		    rc = G10ERR_READ_FILE;
+		    rc = GPGERR_READ_FILE;
 		}
 		else {
 		    size -= c;
@@ -267,7 +273,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	    if( nbytes < OP_MIN_PARTIAL_CHUNK ) {
 		/* not enough to write a partial block out; so we store it*/
 		if( !a->buffer )
-		    a->buffer = m_alloc( OP_MIN_PARTIAL_CHUNK );
+		    a->buffer = gcry_xmalloc( OP_MIN_PARTIAL_CHUNK );
 		memcpy( a->buffer + a->buflen, buf, size );
 		a->buflen += size;
 	    }
@@ -289,14 +295,14 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    if( (n=a->buflen) ) { /* write stuff from the buffer */
 			assert( n == OP_MIN_PARTIAL_CHUNK);
 			if( iobuf_write(chain, a->buffer, n ) )
-			    rc = G10ERR_WRITE_FILE;
+			    rc = GPGERR_WRITE_FILE;
 			a->buflen = 0;
 			nbytes -= n;
 		    }
 		    if( (n = nbytes) > blen )
 			n = blen;
 		    if( n && iobuf_write(chain, p, n ) )
-			rc = G10ERR_WRITE_FILE;
+			rc = GPGERR_WRITE_FILE;
 		    p += n;
 		    nbytes -= n;
 		} while( !rc && nbytes >= OP_MIN_PARTIAL_CHUNK );
@@ -305,7 +311,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    assert( !a->buflen );
 		    assert( nbytes < OP_MIN_PARTIAL_CHUNK );
 		    if( !a->buffer )
-			a->buffer = m_alloc( OP_MIN_PARTIAL_CHUNK );
+			a->buffer = gcry_xmalloc( OP_MIN_PARTIAL_CHUNK );
 		    memcpy( a->buffer, p, nbytes );
 		    a->buflen = nbytes;
 		}
@@ -334,7 +340,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		if( n > avail )
 		    n = avail;
 		if( iobuf_write(chain, p, n ) )
-		    rc = G10ERR_WRITE_FILE;
+		    rc = GPGERR_WRITE_FILE;
 		a->count += n;
 		p += n;
 		size -= n;
@@ -391,9 +397,9 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    rc = iobuf_write(chain, a->buffer, len );
 		if( rc ) {
 		    log_error("block_filter: write error: %s\n",strerror(errno));
-		    rc = G10ERR_WRITE_FILE;
+		    rc = GPGERR_WRITE_FILE;
 		}
-		m_free( a->buffer ); a->buffer = NULL; a->buflen = 0;
+		gcry_free( a->buffer ); a->buffer = NULL; a->buflen = 0;
 	    }
 	    else {
 		iobuf_writebyte(chain, 0);
@@ -405,7 +411,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	}
 	if( DBG_IOBUF )
 	    log_debug("free block_filter %p\n", a );
-	m_free(a); /* we can free our context now */
+	gcry_free(a); /* we can free our context now */
     }
 
     return rc;
@@ -449,9 +455,9 @@ iobuf_alloc(int use, size_t bufsize)
     IOBUF a;
     static int number=0;
 
-    a = m_alloc_clear(sizeof *a);
+    a = gcry_xcalloc( 1,sizeof *a);
     a->use = use;
-    a->d.buf = m_alloc( bufsize );
+    a->d.buf = gcry_xmalloc( bufsize );
     a->d.size = bufsize;
     a->no = ++number;
     a->subno = 0;
@@ -465,12 +471,12 @@ int
 iobuf_close( IOBUF a )
 {
     IOBUF a2;
-    size_t dummy_len;
+    size_t dummy_len = 0;
     int rc=0;
 
     if( a && a->directfp ) {
 	fclose( a->directfp );
-	m_free( a->real_fname );
+	gcry_free( a->real_fname );
 	if( DBG_IOBUF )
 	    log_debug("iobuf_close -> %p\n", a->directfp );
 	return 0;
@@ -479,33 +485,61 @@ iobuf_close( IOBUF a )
     for( ; a && !rc ; a = a2 ) {
 	a2 = a->chain;
 	if( a->use == 2 && (rc=iobuf_flush(a)) )
-	    log_error("iobuf_flush failed on close: %s\n", g10_errstr(rc));
+	    log_error("iobuf_flush failed on close: %s\n", gpg_errstr(rc));
 
 	if( DBG_IOBUF )
 	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno, a->desc );
 	if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE,
 					 a->chain, NULL, &dummy_len)) )
-	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
-	m_free(a->real_fname);
-	m_free(a->d.buf);
-	m_free(a);
+	    log_error("IOBUFCTRL_FREE failed on close: %s\n", gpg_errstr(rc) );
+	gcry_free(a->real_fname);
+	gcry_free(a->d.buf);
+	gcry_free(a);
     }
     return rc;
 }
 
+
 int
 iobuf_cancel( IOBUF a )
 {
     const char *s;
+    IOBUF a2;
+    int rc;
+  #ifdef HAVE_DOSISH_SYSTEM
+    char *remove_name = NULL;
+  #endif
 
     if( a && a->use == 2 ) {
 	s = iobuf_get_real_fname(a);
-	if( s && *s )
-	    remove(s);	/* remove the file. Fixme: this will fail for MSDOZE*/
-    }			/* because the file is still open */
-    return iobuf_close(a);
-}
+	if( s && *s ) {
+	  #ifdef HAVE_DOSISH_SYSTEM
+	    remove_name = m_strdup ( s );
+	  #else
+	    remove(s);
+	  #endif
+	}
+    }
 
+    /* send a cancel message to all filters */
+    for( a2 = a; a2 ; a2 = a2->chain ) {
+	size_t dummy;
+	if( a2->filter )
+	    a2->filter( a2->filter_ov, IOBUFCTRL_CANCEL, a2->chain,
+							 NULL, &dummy );
+    }
+
+    rc = iobuf_close(a);
+  #ifdef HAVE_DOSISH_SYSTEM
+    if ( remove_name ) {
+	/* Argg, MSDOS does not allow to remove open files.  So
+	 * we have to do it here */
+	remove ( remove_name );
+	m_free ( remove_name );
+    }
+  #endif
+    return rc;
+}
 
 /****************
  * create a temporary iobuf, which can be used to collect stuff
@@ -549,19 +583,22 @@ iobuf_open( const char *fname )
     int print_only = 0;
 
     if( !fname || (*fname=='-' && !fname[1])  ) {
-	fp = stdin; /* fixme: set binary mode for msdoze */
+	fp = stdin;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdin]";
 	print_only = 1;
     }
     else if( !(fp = fopen(fname, "rb")) )
 	return NULL;
     a = iobuf_alloc(1, 8192 );
-    fcx = m_alloc( sizeof *fcx + strlen(fname) );
+    fcx = gcry_xmalloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     fcx->print_only_name = print_only;
     strcpy(fcx->fname, fname );
     if( !print_only )
-	a->real_fname = m_strdup( fname );
+	a->real_fname = gcry_xstrdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -588,7 +625,7 @@ iobuf_fdopen( int fd, const char *mode )
     if( !(fp = fdopen(fd, mode)) )
 	return NULL;
     a = iobuf_alloc( strchr( mode, 'w')? 2:1, 8192 );
-    fcx = m_alloc( sizeof *fcx + 20 );
+    fcx = gcry_xmalloc( sizeof *fcx + 20 );
     fcx->fp = fp;
     fcx->print_only_name = 1;
     sprintf(fcx->fname, "[fd %d]", fd );
@@ -616,18 +653,21 @@ iobuf_create( const char *fname )
 
     if( !fname || (*fname=='-' && !fname[1]) ) {
 	fp = stdout;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdout]";
 	print_only = 1;
     }
     else if( !(fp = fopen(fname, "wb")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
-    fcx = m_alloc( sizeof *fcx + strlen(fname) );
+    fcx = gcry_xmalloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     fcx->print_only_name = print_only;
     strcpy(fcx->fname, fname );
     if( !print_only )
-	a->real_fname = m_strdup( fname );
+	a->real_fname = gcry_xstrdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -655,10 +695,10 @@ iobuf_append( const char *fname )
     else if( !(fp = fopen(fname, "ab")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
-    fcx = m_alloc( sizeof *fcx + strlen(fname) );
+    fcx = gcry_xmalloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     strcpy(fcx->fname, fname );
-    a->real_fname = m_strdup( fname );
+    a->real_fname = gcry_xstrdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -682,10 +722,10 @@ iobuf_openrw( const char *fname )
     else if( !(fp = fopen(fname, "r+b")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
-    fcx = m_alloc( sizeof *fcx + strlen(fname) );
+    fcx = gcry_xmalloc( sizeof *fcx + strlen(fname) );
     fcx->fp = fp;
     strcpy(fcx->fname, fname );
-    a->real_fname = m_strdup( fname );
+    a->real_fname = gcry_xstrdup( fname );
     a->filter = file_filter;
     a->filter_ov = fcx;
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
@@ -711,7 +751,10 @@ iobuf_fopen( const char *fname, const char *mode )
     int print_only = 0;
 
     if( !fname || (*fname=='-' && !fname[1])  ) {
-	fp = stdin; /* fixme: set binary mode for msdoze */
+	fp = stdin;
+      #ifdef HAVE_DOSISH_SYSTEM
+	setmode ( fileno(fp) , O_BINARY );
+      #endif
 	fname = "[stdin]";
 	print_only = 1;
     }
@@ -719,7 +762,7 @@ iobuf_fopen( const char *fname, const char *mode )
 	return NULL;
     a = iobuf_alloc(1, 8192 );
     a->directfp = fp;
-    a->real_fname = m_strdup( fname );
+    a->real_fname = gcry_xstrdup( fname );
 
     if( DBG_IOBUF )
 	log_debug("iobuf_fopen -> %p\n", a->directfp );
@@ -760,12 +803,12 @@ iobuf_push_filter2( IOBUF a,
      * The contents of the buffers are transferred to the
      * new stream.
      */
-    b = m_alloc(sizeof *b);
+    b = gcry_xmalloc(sizeof *b);
     memcpy(b, a, sizeof *b );
     /* fixme: it is stupid to keep a copy of the name at every level
      * but we need the name somewhere because the name known by file_filter
      * may have been released when we need the name of the file */
-    b->real_fname = a->real_fname? m_strdup(a->real_fname):NULL;
+    b->real_fname = a->real_fname? gcry_xstrdup(a->real_fname):NULL;
     /* remove the filter stuff from the new stream */
     a->filter = NULL;
     a->filter_ov = NULL;
@@ -775,12 +818,12 @@ iobuf_push_filter2( IOBUF a,
 	a->use = 2;  /* make a write stream from a temp stream */
 
     if( a->use == 2 ) { /* allocate a fresh buffer for the original stream */
-	b->d.buf = m_alloc( a->d.size );
+	b->d.buf = gcry_xmalloc( a->d.size );
 	b->d.len = 0;
 	b->d.start = 0;
     }
     else { /* allocate a fresh buffer for the new stream */
-	a->d.buf = m_alloc( a->d.size );
+	a->d.buf = gcry_xmalloc( a->d.size );
 	a->d.len = 0;
 	a->d.start = 0;
     }
@@ -808,7 +851,7 @@ iobuf_push_filter2( IOBUF a,
     /* now we can initialize the new function if we have one */
     if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_INIT, a->chain,
 		       NULL, &dummy_len)) )
-	log_error("IOBUFCTRL_INIT failed: %s\n", g10_errstr(rc) );
+	log_error("IOBUFCTRL_INIT failed: %s\n", gpg_errstr(rc) );
     return rc;
 }
 
@@ -831,10 +874,10 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
     if( !a->filter ) { /* this is simple */
 	b = a->chain;
 	assert(b);
-	m_free(a->d.buf);
-	m_free(a->real_fname);
+	gcry_free(a->d.buf);
+	gcry_free(a->real_fname);
 	memcpy(a,b, sizeof *a);
-	m_free(b);
+	gcry_free(b);
 	return 0;
     }
     for(b=a ; b; b = b->chain )
@@ -845,17 +888,17 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 
     /* flush this stream if it is an output stream */
     if( a->use == 2 && (rc=iobuf_flush(b)) ) {
-	log_error("iobuf_flush failed in pop_filter: %s\n", g10_errstr(rc));
+	log_error("iobuf_flush failed in pop_filter: %s\n", gpg_errstr(rc));
 	return rc;
     }
     /* and tell the filter to free it self */
     if( b->filter && (rc = b->filter(b->filter_ov, IOBUFCTRL_FREE, b->chain,
 		       NULL, &dummy_len)) ) {
-	log_error("IOBUFCTRL_FREE failed: %s\n", g10_errstr(rc) );
+	log_error("IOBUFCTRL_FREE failed: %s\n", gpg_errstr(rc) );
 	return rc;
     }
     if( b->filter_ov && b->filter_ov_owner ) {
-	m_free( b->filter_ov );
+	gcry_free( b->filter_ov );
 	b->filter_ov = NULL;
     }
 
@@ -868,10 +911,10 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	 * a flush has been done on the to be removed entry
 	 */
 	b = a->chain;
-	m_free(a->d.buf);
-	m_free(a->real_fname);
+	gcry_free(a->d.buf);
+	gcry_free(a->real_fname);
 	memcpy(a,b, sizeof *a);
-	m_free(b);
+	gcry_free(b);
 	if( DBG_IOBUF )
 	   log_debug("iobuf-%d.%d: popped filter\n", a->no, a->subno );
     }
@@ -906,10 +949,10 @@ underflow(IOBUF a)
 	    if( DBG_IOBUF )
 		log_debug("iobuf-%d.%d: pop `%s' in underflow\n",
 					a->no, a->subno, a->desc );
-	    m_free(a->d.buf);
-	    m_free(a->real_fname);
+	    gcry_free(a->d.buf);
+	    gcry_free(a->real_fname);
 	    memcpy(a, b, sizeof *a);
-	    m_free(b);
+	    gcry_free(b);
 	    print_chain(a);
 	}
 	else
@@ -955,14 +998,14 @@ underflow(IOBUF a)
 
 	}
 	if( a->use == 1 && rc == -1 ) { /* EOF: we can remove the filter */
-	    size_t dummy_len;
+	    size_t dummy_len=0;
 
 	    /* and tell the filter to free itself */
 	    if( (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE, a->chain,
 			       NULL, &dummy_len)) )
-		log_error("IOBUFCTRL_FREE failed: %s\n", g10_errstr(rc) );
+		log_error("IOBUFCTRL_FREE failed: %s\n", gpg_errstr(rc) );
 	    if( a->filter_ov && a->filter_ov_owner ) {
-		m_free( a->filter_ov );
+		gcry_free( a->filter_ov );
 		a->filter_ov = NULL;
 	    }
 	    a->filter = NULL;
@@ -975,10 +1018,10 @@ underflow(IOBUF a)
 		    log_debug("iobuf-%d.%d: pop `%s' in underflow (!len)\n",
 					       a->no, a->subno, a->desc );
 		print_chain(a);
-		m_free(a->d.buf);
-		m_free(a->real_fname);
+		gcry_free(a->d.buf);
+		gcry_free(a->real_fname);
 		memcpy(a,b, sizeof *a);
-		m_free(b);
+		gcry_free(b);
 		print_chain(a);
 	    }
 	}
@@ -1019,9 +1062,9 @@ iobuf_flush(IOBUF a)
 
 	log_debug("increasing temp iobuf from %lu to %lu\n",
 		    (ulong)a->d.size, (ulong)newsize );
-	newbuf = m_alloc( newsize );
+	newbuf = gcry_xmalloc( newsize );
 	memcpy( newbuf, a->d.buf, a->d.len );
-	m_free(a->d.buf);
+	gcry_free(a->d.buf);
 	a->d.buf = newbuf;
 	a->d.size = newsize;
 	return 0;
@@ -1034,7 +1077,7 @@ iobuf_flush(IOBUF a)
     rc = a->filter( a->filter_ov, IOBUFCTRL_FLUSH, a->chain, a->d.buf, &len );
     if( !rc && len != a->d.len ) {
 	log_info("iobuf_flush did not write all!\n");
-	rc = G10ERR_WRITE_FILE;
+	rc = GPGERR_WRITE_FILE;
     }
     else if( rc )
 	a->error = 1;
@@ -1057,7 +1100,7 @@ iobuf_readbyte(IOBUF a)
     if( a->unget.buf ) {
 	if( a->unget.start < a->unget.len )
 	    return a->unget.buf[a->unget.start++];
-	m_free(a->unget.buf);
+	gcry_free(a->unget.buf);
 	a->unget.buf = NULL;
 	a->nofast &= ~2;
     }
@@ -1401,7 +1444,7 @@ iobuf_get_fname( IOBUF a )
 void
 iobuf_set_block_mode( IOBUF a, size_t n )
 {
-    block_filter_ctx_t *ctx = m_alloc_clear( sizeof *ctx );
+    block_filter_ctx_t *ctx = gcry_xcalloc( 1, sizeof *ctx );
 
     assert( a->use == 1 || a->use == 2 );
     ctx->use = a->use;
@@ -1423,7 +1466,7 @@ iobuf_set_block_mode( IOBUF a, size_t n )
 void
 iobuf_set_partial_block_mode( IOBUF a, size_t len )
 {
-    block_filter_ctx_t *ctx = m_alloc_clear( sizeof *ctx );
+    block_filter_ctx_t *ctx = gcry_xcalloc( 1, sizeof *ctx );
 
     assert( a->use == 1 || a->use == 2 );
     ctx->use = a->use;
@@ -1478,7 +1521,7 @@ iobuf_read_line( IOBUF a, byte **addr_of_buffer,
 
     if( !buffer ) { /* must allocate a new buffer */
 	length = 256;
-	buffer = m_alloc( length );
+	buffer = gcry_xmalloc( length );
 	*addr_of_buffer = buffer;
 	*length_of_buffer = length;
     }
@@ -1498,7 +1541,7 @@ iobuf_read_line( IOBUF a, byte **addr_of_buffer,
 	    }
 	    length += 3; /* correct for the reserved byte */
 	    length += length < 1024? 256 : 1024;
-	    buffer = m_realloc( buffer, length );
+	    buffer = gcry_xrealloc( buffer, length );
 	    *addr_of_buffer = buffer;
 	    *length_of_buffer = length;
 	    length -= 3; /* and reserve again */
