@@ -1,5 +1,5 @@
 /* logger.c  -	log functions
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,6 +24,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#include <gcrypt.h>
 
 #include "util.h"
 #include "i18n.h"
@@ -40,17 +41,19 @@ static FILE *logfp;
 void
 log_set_logfile( const char *name, int fd )
 {
-    if( name )
-	BUG();
-
     if( logfp && logfp != stderr && logfp != stdout )
-	fclose( logfp );
-    if( fd == 1 )
-	logfp = stdout;
-    else if( fd == 2 )
-	logfp = stderr;
-    else
-	logfp = fdopen( fd, "a" );
+        fclose( logfp );
+    if( name ) {
+        logfp = fopen ( name, "a" );
+    }
+    else {
+        if( fd == 1 )
+            logfp = stdout;
+        else if( fd == 2 )
+            logfp = stderr;
+        else
+            logfp = fdopen( fd, "a" );
+    }
     if( !logfp ) {
 	logfp = stderr;
 	log_fatal("can't open fd %d for logging: %s\n", fd, strerror(errno));
@@ -69,9 +72,9 @@ log_stream()
 void
 log_set_name( const char *name )
 {
-    m_free(pgm_name);
+    gcry_free(pgm_name);
     if( name )
-	pgm_name = m_strdup(name);
+	pgm_name = gcry_xstrdup(name);
     else
 	pgm_name = NULL;
 }
@@ -109,7 +112,7 @@ log_inc_errorcount()
 
 
 void
-g10_log_print_prefix(const char *text)
+gpg_log_print_prefix(const char *text)
 {
     if( !logfp )
 	logfp = stderr;
@@ -131,18 +134,18 @@ print_prefix_f(const char *text, const char *fname)
 }
 
 void
-g10_log_info( const char *fmt, ... )
+gpg_log_info( const char *fmt, ... )
 {
     va_list arg_ptr ;
 
-    g10_log_print_prefix("");
+    gpg_log_print_prefix("");
     va_start( arg_ptr, fmt ) ;
     vfprintf(logfp,fmt,arg_ptr) ;
     va_end(arg_ptr);
 }
 
 void
-g10_log_info_f( const char *fname, const char *fmt, ... )
+gpg_log_info_f( const char *fname, const char *fmt, ... )
 {
     va_list arg_ptr ;
 
@@ -153,11 +156,11 @@ g10_log_info_f( const char *fname, const char *fmt, ... )
 }
 
 void
-g10_log_error( const char *fmt, ... )
+gpg_log_error( const char *fmt, ... )
 {
     va_list arg_ptr ;
 
-    g10_log_print_prefix("");
+    gpg_log_print_prefix("");
     va_start( arg_ptr, fmt ) ;
     vfprintf(logfp,fmt,arg_ptr) ;
     va_end(arg_ptr);
@@ -165,7 +168,7 @@ g10_log_error( const char *fmt, ... )
 }
 
 void
-g10_log_error_f( const char *fname, const char *fmt, ... )
+gpg_log_error_f( const char *fname, const char *fmt, ... )
 {
     va_list arg_ptr ;
 
@@ -177,11 +180,11 @@ g10_log_error_f( const char *fname, const char *fmt, ... )
 }
 
 void
-g10_log_fatal( const char *fmt, ... )
+gpg_log_fatal( const char *fmt, ... )
 {
     va_list arg_ptr ;
 
-    g10_log_print_prefix("fatal: ");
+    gpg_log_print_prefix("fatal: ");
     va_start( arg_ptr, fmt ) ;
     vfprintf(logfp,fmt,arg_ptr) ;
     va_end(arg_ptr);
@@ -190,7 +193,7 @@ g10_log_fatal( const char *fmt, ... )
 }
 
 void
-g10_log_fatal_f( const char *fname, const char *fmt, ... )
+gpg_log_fatal_f( const char *fname, const char *fmt, ... )
 {
     va_list arg_ptr ;
 
@@ -203,12 +206,12 @@ g10_log_fatal_f( const char *fname, const char *fmt, ... )
 }
 
 void
-g10_log_bug( const char *fmt, ... )
+gpg_log_bug( const char *fmt, ... )
 {
     va_list arg_ptr ;
 
     putc('\n', stderr );
-    g10_log_print_prefix("Ohhhh jeeee: ");
+    gpg_log_print_prefix("Ohhhh jeeee: ");
     va_start( arg_ptr, fmt ) ;
     vfprintf(stderr,fmt,arg_ptr) ;
     va_end(arg_ptr);
@@ -219,31 +222,31 @@ g10_log_bug( const char *fmt, ... )
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
 void
-g10_log_bug0( const char *file, int line, const char *func )
+gpg_log_bug0( const char *file, int line, const char *func )
 {
     log_bug(_("... this is a bug (%s:%d:%s)\n"), file, line, func );
 }
 #else
 void
-g10_log_bug0( const char *file, int line )
+gpg_log_bug0( const char *file, int line )
 {
     log_bug(_("you found a bug ... (%s:%d)\n"), file, line);
 }
 #endif
 
 void
-g10_log_debug( const char *fmt, ... )
+gpg_log_debug( const char *fmt, ... )
 {
     va_list arg_ptr ;
 
-    g10_log_print_prefix("DBG: ");
+    gpg_log_print_prefix("DBG: ");
     va_start( arg_ptr, fmt ) ;
     vfprintf(logfp,fmt,arg_ptr) ;
     va_end(arg_ptr);
 }
 
 void
-g10_log_debug_f( const char *fname, const char *fmt, ... )
+gpg_log_debug_f( const char *fname, const char *fmt, ... )
 {
     va_list arg_ptr ;
 
@@ -256,11 +259,11 @@ g10_log_debug_f( const char *fname, const char *fmt, ... )
 
 
 void
-g10_log_hexdump( const char *text, const char *buf, size_t len )
+gpg_log_hexdump( const char *text, const char *buf, size_t len )
 {
     int i;
 
-    g10_log_print_prefix(text);
+    gpg_log_print_prefix(text);
     for(i=0; i < len; i++ )
 	fprintf(logfp, " %02X", ((const byte*)buf)[i] );
     fputc('\n', logfp);
