@@ -1,5 +1,5 @@
 /* ttyio.c -  tty i/O functions
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -45,16 +45,13 @@
 #endif
 #include <errno.h>
 #include <ctype.h>
+#include <gcrypt.h>
 #include "util.h"
 #include "memory.h"
 #include "ttyio.h"
 
 #define CONTROL_D ('D' - 'A' + 1)
-#ifdef __VMS
-  #define TERMDEVICE "/dev/tty"
-#else
-  #define TERMDEVICE "/dev/tty"
-#endif
+
 
 #ifdef __MINGW32__ /* use the odd Win32 functions */
 static struct {
@@ -124,7 +121,7 @@ init_ttyfp(void)
   #elif defined(__EMX__)
     ttyfp = stdout; /* Fixme: replace by the real functions: see wklib */
   #else
-    ttyfp = batchmode? stderr : fopen(TERMDEVICE, "r+");
+    ttyfp = batchmode? stderr : fopen("/dev/tty", "r+");
     if( !ttyfp ) {
 	log_error("cannot open /dev/tty: %s\n", strerror(errno) );
 	exit(2);
@@ -175,19 +172,19 @@ tty_printf( const char *fmt, ... )
 	     * it, so we use a static buffer for now */
 	do {
 	    if( n == -1 || !buf ) {
-		m_free(buf);
+		gcry_free(buf);
 		bufsize += 200;
 		/* better check the new size; (we use M$ functions) */
 		if( bufsize > 50000 )
 		    log_bug("vsnprintf probably failed\n");
-		buf = m_alloc( bufsize );
+		buf = gcry_xmalloc( bufsize );
 	    }
 	    n = _vsnprintf(buf, bufsize-1, fmt, arg_ptr);
 	} while( n == -1 );
       #else
 	if( !buf ) {
 	    bufsize += 1000;
-	    buf = m_alloc( bufsize );
+	    buf = gcry_xmalloc( bufsize );
 	}
 	n = vsprintf(buf, fmt, arg_ptr);
 	if( n == -1 )
@@ -197,7 +194,7 @@ tty_printf( const char *fmt, ... )
 	if( !WriteConsoleA( con.out, buf, n, &nwritten, NULL ) )
 	    log_fatal("WriteConsole failed: rc=%d", (int)GetLastError() );
 	if( n != nwritten )
-	    log_fatal("WriteConsole failed: %d != %d\n", n, nwritten );
+	    log_fatal("WriteConsole failed: %d != %ld\n", n, nwritten );
 	last_prompt_len += n;
     }
   #else
@@ -270,7 +267,7 @@ tty_print_utf8_string2( byte *p, size_t n, size_t max_n )
 	}
 	/*(utf8 conversion already does the control character quoting)*/
 	tty_printf("%s", buf );
-	m_free( buf );
+	gcry_free( buf );
     }
     else {
 	if( n > max_n ) {
@@ -280,6 +277,7 @@ tty_print_utf8_string2( byte *p, size_t n, size_t max_n )
     }
 }
 
+
 void
 tty_print_utf8_string( byte *p, size_t n )
 {
@@ -287,6 +285,8 @@ tty_print_utf8_string( byte *p, size_t n )
 }
 
 
+
+
 static char *
 do_get( const char *prompt, int hidden )
 {
@@ -308,8 +308,8 @@ do_get( const char *prompt, int hidden )
 	init_ttyfp();
 
     last_prompt_len = 0;
-    tty_printf( "%s", prompt );
-    buf = m_alloc(n=50);
+    tty_printf( prompt );
+    buf = gcry_xmalloc(n=50);
     i = 0;
 
   #ifdef __MINGW32__ /* windoze version */
@@ -338,7 +338,7 @@ do_get( const char *prompt, int hidden )
 	    continue;
 	if( !(i < n-1) ) {
 	    n += 50;
-	    buf = m_realloc( buf, n );
+	    buf = gcry_xrealloc( buf, n );
 	}
 	buf[i++] = c;
     }
@@ -378,7 +378,7 @@ do_get( const char *prompt, int hidden )
 	    continue;
 	if( !(i < n-1) ) {
 	    n += 50;
-	    buf = m_realloc( buf, n );
+	    buf = gcry_xrealloc( buf, n );
 	}
 	buf[i++] = c;
     }
@@ -450,7 +450,7 @@ tty_get_answer_is_yes( const char *prompt )
     char *p = tty_get( prompt );
     tty_kill_prompt();
     yes = answer_is_yes(p);
-    m_free(p);
+    gcry_free(p);
     return yes;
 }
 
